@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -46,7 +46,7 @@ struct post_class_s {
   /*
    * open a new instance of this plugin class
    */
-  post_plugin_t* (*open_plugin) (post_class_t *this, int inputs,
+  post_plugin_t* (*open_plugin) (post_class_t *this_gen, int inputs,
 				 xine_audio_port_t **audio_target,
 				 xine_video_port_t **video_target);
 
@@ -71,10 +71,10 @@ struct post_class_s {
    * free all class-related resources
    */
 
-  void (*dispose) (post_class_t *this);
+  void (*dispose) (post_class_t *this_gen);
 };
 
-#define default_post_class_dispose (void (*) (post_class_t *this))free
+#define default_post_class_dispose (void (*) (post_class_t *this_gen))free
 
 struct post_plugin_s {
 
@@ -91,7 +91,7 @@ struct post_plugin_s {
   /*
    * close down, free all resources
    */
-  void (*dispose) (post_plugin_t *this);
+  void (*dispose) (post_plugin_t *this_gen);
 
   /* plugins don't have to init the stuff below */
 
@@ -392,12 +392,17 @@ do {                                                               \
 /* macros to create parameter descriptors */
 
 #define START_PARAM_DESCR( param_t ) \
+typedef param_t temp_t; \
 static param_t temp_s; \
 static xine_post_api_parameter_t temp_p[] = {
 
+#ifndef offsetof
+#include <stddef.h>
+#endif
+
 #define PARAM_ITEM( param_type, var, enumv, min, max, readonly, descr ) \
 { param_type, #var, sizeof(temp_s.var), \
-  (char*)&temp_s.var-(char*)&temp_s, enumv, min, max, readonly, descr },
+  offsetof(temp_t, var), enumv, min, max, readonly, descr },
 
 #define END_PARAM_DESCR( name ) \
   { POST_PARAM_TYPE_LAST, NULL, 0, 0, NULL, 0, 0, 1, NULL } \
