@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -293,10 +293,10 @@ struct xine_video_port_s {
 #define VO_CHROMA_422        32 /* used by VDPAU, default is chroma_420 */
 #define VO_STILL_IMAGE       64
 
-/* ((mpeg_color_matrix << 1) | color_range) inside frame.flags bits 11-8 */
+/* ((mpeg_color_matrix << 1) | color_range) inside frame.flags bits 12-8 */
 #define VO_FULLRANGE 0x100
-#define VO_GET_FLAGS_CM(flags) ((flags >> 8) & 15)
-#define VO_SET_FLAGS_CM(cm,flags) flags = ((flags) & ~0xf00) | (((cm) & 15) << 8)
+#define VO_GET_FLAGS_CM(flags) ((flags >> 8) & 31)
+#define VO_SET_FLAGS_CM(cm,flags) flags = ((flags) & ~0x1f00) | (((cm) & 31) << 8)
 
 /* video driver capabilities */
 #define VO_CAP_YV12                   0x00000001 /* driver can handle YUV 4:2:0 pictures */
@@ -452,7 +452,7 @@ struct video_driver_class_s {
   void (*dispose) (video_driver_class_t *self);
 };
 
-#define default_video_driver_class_dispose (void (*) (video_driver_class_t *this))free
+#define default_video_driver_class_dispose (void (*) (video_driver_class_t *this_gen))free
 
 typedef struct rle_elem_s {
   uint16_t len;
