@@ -17,6 +17,8 @@
 
 #include <jack/jack.h>
 
+#include "speakers.h"
+
 #define AO_OUT_JACK_IFACE_VERSION 9
 
 #define GAP_TOLERANCE        AO_MAX_GAP
@@ -687,26 +689,7 @@ static ao_driver_t *open_jack_plugin (audio_driver_class_t *class_gen,
   char *jack_device;
   const char **matching_ports = NULL;
 
-  /* for usability reasons, keep this in sync with audio_oss_out.c */
-  static const char * const speaker_arrangement[] = {
-    "Mono 1.0", "Stereo 2.0", "Headphones 2.0", "Stereo 2.1",
-    "Surround 3.0", "Surround 4.0", "Surround 4.1", "Surround 5.0",
-    "Surround 5.1", "Surround 6.0",
-    "Surround 6.1", "Surround 7.1", "Pass Through", NULL
-  };
-#define MONO		0
-#define STEREO		1
-#define HEADPHONES	2
-#define SURROUND21	3
-#define SURROUND3	4
-#define SURROUND4	5
-#define SURROUND41	6
-#define SURROUND5	7
-#define SURROUND51	8
-#define SURROUND6	9
-#define SURROUND61	10
-#define SURROUND71	11
-#define A52_PASSTHRU	12
+  AUDIO_DEVICE_SPEAKER_ARRANGEMENT_TYPES;
   int speakers;
 
   /* Try to create a client called "xine[-NN]" */
@@ -738,34 +721,7 @@ static ao_driver_t *open_jack_plugin (audio_driver_class_t *class_gen,
   speakers =
     config->register_enum (config, "audio.output.speaker_arrangement",
 			   STEREO, speaker_arrangement,
-			   _("speaker arrangement"),
-			   _("Select how your speakers are arranged, "
-			     "this determines which speakers xine uses for sound output. "
-			     "The individual values are:\n\n"
-			     "Mono 1.0: You have only one speaker.\n"
-			     "Stereo 2.0: You have two speakers for left and right channel.\n"
-			     "Headphones 2.0: You use headphones.\n"
-			     "Stereo 2.1: You have two speakers for left and right channel, and one "
-			     "subwoofer for the low frequencies.\n"
-			     "Surround 3.0: You have three speakers for left, right and rear channel.\n"
-			     "Surround 4.0: You have four speakers for front left and right and rear "
-			     "left and right channels.\n"
-			     "Surround 4.1: You have four speakers for front left and right and rear "
-			     "left and right channels, and one subwoofer for the low frequencies.\n"
-			     "Surround 5.0: You have five speakers for front left, center and right and "
-			     "rear left and right channels.\n"
-			     "Surround 5.1: You have five speakers for front left, center and right and "
-			     "rear left and right channels, and one subwoofer for the low frequencies.\n"
-			     "Surround 6.0: You have six speakers for front left, center and right and "
-			     "rear left, center and right channels.\n"
-			     "Surround 6.1: You have six speakers for front left, center and right and "
-			     "rear left, center and right channels, and one subwoofer for the low frequencies.\n"
-			     "Surround 7.1: You have seven speakers for front left, center and right, "
-			     "left and right and rear left and right channels, and one subwoofer for the "
-			     "low frequencies.\n"
-			     "Pass Through: Your sound system will receive undecoded digital sound from xine. "
-			     "You need to connect a digital surround decoder capable of decoding the "
-			     "formats you want to play to your sound card's digital output."),
+                           AUDIO_DEVICE_SPEAKER_ARRANGEMENT_HELP,
 			   0, jack_speaker_arrangement_cb, this);
 
   int port_flags = JackPortIsInput;
