@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -71,6 +71,8 @@
 
 #include <sys/time.h>
 
+#include "speakers.h"
+
 #ifndef SNDCTL_DSP_SETFMT
 /* Deprecated OSS API */
 #define SNDCTL_DSP_SETFMT SOUND_PCM_SETFMT
@@ -707,23 +709,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   static const char * const sync_methods[] = {"auto", "getodelay", "getoptr", "softsync", "probebuffer", NULL};
   static const char * const devname_opts[] = {"auto", "/dev/dsp", "/dev/sound/dsp", NULL};
   int devname_val, devname_num;
-  /* for usability reasons, keep this in sync with audio_alsa_out.c */
-  static const char * const speaker_arrangement[] = {"Mono 1.0", "Stereo 2.0", "Headphones 2.0", "Stereo 2.1",
-    "Surround 3.0", "Surround 4.0", "Surround 4.1", "Surround 5.0", "Surround 5.1", "Surround 6.0",
-    "Surround 6.1", "Surround 7.1", "Pass Through", NULL};
-  #define MONO		0
-  #define STEREO	1
-  #define HEADPHONES	2
-  #define SURROUND21	3
-  #define SURROUND3	4
-  #define SURROUND4	5
-  #define SURROUND41	6
-  #define SURROUND5	7
-  #define SURROUND51	8
-  #define SURROUND6	9
-  #define SURROUND61	10
-  #define SURROUND71	11
-  #define A52_PASSTHRU	12
+
+  AUDIO_DEVICE_SPEAKER_ARRANGEMENT_TYPES;
   int speakers;
 
 
@@ -739,7 +726,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
 
   /* devname_val is offset used to select auto, /dev/dsp, or /dev/sound/dsp */
   devname_val = config->register_enum (config, "audio.device.oss_device_name", 0,
-				       devname_opts,
+				       (char **)devname_opts,
 				       _("OSS audio device name"),
 				       _("Specifies the base part of the audio device name, "
 					 "to which the OSS device number is appended to get the "
@@ -803,7 +790,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
    */
 
   this->sync_method = config->register_enum (config, "audio.oss_sync_method", OSS_SYNC_AUTO_DETECT,
-					     sync_methods,
+					     (char **)sync_methods,
 					     _("a/v sync method to use by OSS"),
 					     _("xine can use different methods to keep audio and video "
 					       "synchronized. Which setting works best depends on the "
@@ -921,36 +908,9 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
 
   /* for usability reasons, keep this in sync with audio_alsa_out.c */
   speakers = config->register_enum(config, "audio.output.speaker_arrangement", STEREO,
-			speaker_arrangement,
-			_("speaker arrangement"),
-			_("Select how your speakers are arranged, "
-			  "this determines which speakers xine uses for sound output. "
-			  "The individual values are:\n\n"
-			  "Mono 1.0: You have only one speaker.\n"
-			  "Stereo 2.0: You have two speakers for left and right channel.\n"
-			  "Headphones 2.0: You use headphones.\n"
-			  "Stereo 2.1: You have two speakers for left and right channel, and one "
-			  "subwoofer for the low frequencies.\n"
-			  "Surround 3.0: You have three speakers for left, right and rear channel.\n"
-			  "Surround 4.0: You have four speakers for front left and right and rear "
-			  "left and right channels.\n"
-			  "Surround 4.1: You have four speakers for front left and right and rear "
-			  "left and right channels, and one subwoofer for the low frequencies.\n"
-			  "Surround 5.0: You have five speakers for front left, center and right and "
-			  "rear left and right channels.\n"
-			  "Surround 5.1: You have five speakers for front left, center and right and "
-			  "rear left and right channels, and one subwoofer for the low frequencies.\n"
-			  "Surround 6.0: You have six speakers for front left, center and right and "
-			  "rear left, center and right channels.\n"
-			  "Surround 6.1: You have six speakers for front left, center and right and "
-			  "rear left, center and right channels, and one subwoofer for the low frequencies.\n"
-			  "Surround 7.1: You have seven speakers for front left, center and right, "
-			  "left and right and rear left and right channels, and one subwoofer for the "
-			  "low frequencies.\n"
-			  "Pass Through: Your sound system will receive undecoded digital sound from xine. "
-			  "You need to connect a digital surround decoder capable of decoding the "
-			  "formats you want to play to your sound card's digital output."),
-			0, oss_speaker_arrangement_cb, this);
+                                   (char **)speaker_arrangement,
+                                   AUDIO_DEVICE_SPEAKER_ARRANGEMENT_HELP,
+                                   0, oss_speaker_arrangement_cb, this);
 
 
   char *logmsg = strdup (_("audio_oss_out: supported modes are"));
