@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2014 the xine project
+ * Copyright (C) 2013-2017 the xine project
  * Copyright (C) 2013 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -47,6 +47,20 @@
 #include "ff_audio_list.h"
 
 #include "ffmpeg_compat.h"
+
+#ifndef LIBAVFORMAT_VERSION_INT
+#  if defined(LIBAVFORMAT_VERSION_MAJOR) && defined(LIBAVFORMAT_VERSION_MINOR)
+#    define LIBAVFORMAT_VERSION_INT XFF_INT_VERSION(LIBAVFORMAT_VERSION_MAJOR,LIBAVFORMAT_VERSION_MINOR,0)
+#  endif
+#endif
+
+#if LIBAVFORMAT_VERSION_INT < XFF_INT_VERSION(57,62,0)
+#define CODEC_ID(st)   ((st)->codec->codec_id)
+#else
+#define CODEC_ID(st)   ((st)->codecpar->codec_id)
+#define XFF_CODECPAR 1
+#endif
+
 /*
  * avformat dummy input plugin
  */
@@ -161,7 +175,7 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
     real_mrl = strdup(mrl);
     memmove(real_mrl + 4, real_mrl + 8, strlen(real_mrl) - 8 + 1);
   }
-  if (!strncmp(mrl, "rtsp+http", 9)) {
+  else if (!strncmp(mrl, "rtsp+http", 9)) {
     av_dict_set(&options, "rtsp_transport", "http", 0);
     real_mrl = strdup(mrl);
     memmove(real_mrl + 4, real_mrl + 9, strlen(real_mrl) - 9 + 1);
@@ -271,7 +285,7 @@ typedef struct {
 static void check_newpts(avformat_demux_plugin_t *this, int64_t pts) {
 
   int64_t diff = this->last_pts - pts;
-  if (this->seek_flag || this->send_newpts || (this->last_pts && abs(diff) > WRAP_THRESHOLD)) {
+  if (this->seek_flag || this->send_newpts || (this->last_pts && llabs(diff) > WRAP_THRESHOLD)) {
 
     _x_demux_control_newpts(this->stream, pts, this->seek_flag);
     this->send_newpts = 0;
@@ -280,9 +294,9 @@ static void check_newpts(avformat_demux_plugin_t *this, int64_t pts) {
   }
 }
 
-static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, int id) {
+static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
 
-  int i;
+  size_t i;
   for (i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++) {
     if (ff_video_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
@@ -294,9 +308,9 @@ static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, int id) {
   return 0;
 }
 
-static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, int id) {
+static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
 
-  int i;
+  size_t i;
   for (i = 0; i < sizeof(ff_audio_lookup)/sizeof(ff_codec_t); i++) {
     if (ff_audio_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
@@ -322,7 +336,7 @@ static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, int id) {
 /* copied from libavformat/utils.c (not available in recent libav API) */
 static AVProgram *_find_program_from_stream(AVFormatContext *ic, AVProgram *last, int s)
 {
-  int i, j;
+  unsigned i, j;
 
   for (i = 0; i < ic->nb_programs; i++) {
     if (ic->programs[i] == last) {
@@ -361,15 +375,15 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
 
   if (this->video_stream_idx >= 0) {
     AVStream *st = this->fmt_ctx->streams[this->video_stream_idx];
-    uint32_t xine_video_type = video_codec_lookup(this, st->codec->codec_id);
+    uint32_t xine_video_type = video_codec_lookup(this, CODEC_ID(st));
 
     if (!xine_video_type) {
       this->video_stream_idx = -1;
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg video codec id %d --> NO xine buffer type\n", st->codec->codec_id);
+               LOG_MODULE": ffmpeg video codec id %d --> NO xine buffer type\n", CODEC_ID(st));
     } else {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg video codec id %d --> xine buffer type 0x%08x\n", st->codec->codec_id, xine_video_type);
+               LOG_MODULE": ffmpeg video codec id %d --> xine buffer type 0x%08x\n", CODEC_ID(st), xine_video_type);
 
       this->xine_buf_type[this->video_stream_idx] = xine_video_type;
     }
@@ -391,17 +405,22 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
       continue;
     }
 
+#ifdef XFF_CODECPAR
+    if (st->codecpar && st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO &&
+        st->codecpar->sample_rate != 0 && st->codecpar->channels != 0) {
+#else
     if (st->codec && st->codec->codec_type == AVMEDIA_TYPE_AUDIO &&
         st->codec->sample_rate != 0 && st->codec->channels != 0) {
+#endif
 
-      int xine_audio_type = audio_codec_lookup(this, st->codec->codec_id);
+      uint32_t xine_audio_type = audio_codec_lookup(this, CODEC_ID(st));
       if (!xine_audio_type) {
         xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-                 LOG_MODULE": ffmpeg audio codec id %d --> NO xine buffer type\n", st->codec->codec_id);
+                 LOG_MODULE": ffmpeg audio codec id %d --> NO xine buffer type\n", CODEC_ID(st));
         continue;
       }
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               LOG_MODULE": ffmpeg audio codec id %d --> xine buffer type 0x%08x\n", st->codec->codec_id, xine_audio_type);
+               LOG_MODULE": ffmpeg audio codec id %d --> xine buffer type 0x%08x\n", CODEC_ID(st), xine_audio_type);
 
       this->audio_stream_idx[this->audio_track_count] = stream_index;
 
@@ -431,20 +450,28 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
 
 static void send_headers_audio(avformat_demux_plugin_t *this) {
 
-  int ii;
+  unsigned ii;
   for (ii = 0; ii < this->audio_track_count; ii++) {
 
+#ifdef XFF_CODECPAR
+  AVCodecParameters *ctx = this->fmt_ctx->streams[this->audio_stream_idx[ii]]->codecpar;
+#else
   AVCodecContext    *ctx = this->fmt_ctx->streams[this->audio_stream_idx[ii]]->codec;
+#endif
   buf_element_t     *buf = this->stream->audio_fifo->buffer_pool_alloc (this->stream->audio_fifo);
   size_t             extradata_size = ctx->extradata_size;
   xine_waveformatex *fmt = (xine_waveformatex *)buf->content;
 
-  if (!ctx->extradata || extradata_size + sizeof(xine_waveformatex) > buf->max_size) {
+  if (!ctx->extradata || extradata_size + sizeof(xine_waveformatex) > (size_t)buf->max_size) {
+    if (extradata_size) {
+      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": ignoring large audio extradata (%zd bytes)\n", extradata_size);
+    }
     extradata_size = 0;
   }
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_FOURCC, ctx->codec_tag);
 
+  memset(fmt, 0, sizeof(*fmt));
   fmt->cbSize          = extradata_size;
   fmt->nBlockAlign     = ctx->block_align;
   fmt->nAvgBytesPerSec = ctx->bit_rate / 8;
@@ -466,17 +493,25 @@ static void send_headers_audio(avformat_demux_plugin_t *this) {
 
 static void send_headers_video(avformat_demux_plugin_t *this) {
 
+#ifdef XFF_CODECPAR
+  AVCodecParameters *ctx = this->fmt_ctx->streams[this->video_stream_idx]->codecpar;
+#else
   AVCodecContext *ctx = this->fmt_ctx->streams[this->video_stream_idx]->codec;
+#endif
   buf_element_t  *buf = this->stream->video_fifo->buffer_pool_alloc (this->stream->video_fifo);
   size_t          extradata_size = ctx->extradata_size;
   xine_bmiheader *bih = (xine_bmiheader *)buf->content;
 
-  if (!ctx->extradata || extradata_size + sizeof(xine_bmiheader) > buf->max_size) {
+  if (!ctx->extradata || extradata_size + sizeof(xine_bmiheader) > (size_t)buf->max_size) {
+    if (extradata_size) {
+      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": ignoring large video extradata (%zd bytes)\n", extradata_size);
+    }
     extradata_size = 0;
   }
 
   _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_FOURCC, ctx->codec_tag);
 
+  memset(bih, 0, sizeof(*bih));
   bih->biSize     = sizeof(xine_bmiheader) + extradata_size;
   bih->biBitCount = ctx->bits_per_coded_sample;
   bih->biWidth    = ctx->width;
@@ -507,11 +542,12 @@ static int send_avpacket(avformat_demux_plugin_t *this)
 
   /* read frame from the file */
   if (av_read_frame(this->fmt_ctx, &pkt) < 0) {
+    xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": av_read_frame() failed\n");
     return -1;
   }
 
   /* map to xine fifo / buffer type */
-  if (pkt.stream_index < this->num_streams) {
+  if (pkt.stream_index >= 0 && (unsigned)pkt.stream_index < this->num_streams) {
     buffer_type = this->xine_buf_type[pkt.stream_index];
   } else {
     // TODO: new streams found
@@ -540,7 +576,7 @@ static int send_avpacket(avformat_demux_plugin_t *this)
                        input_normpos, input_time, total_time, 0/*frame_number*/);
   }
 
-  av_free_packet(&pkt);
+  XFF_PACKET_UNREF(&pkt);
 
   return 1;
 }
@@ -582,7 +618,7 @@ static int demux_avformat_get_optional_data(demux_plugin_t *this_gen,
 
   switch (data_type) {
     case DEMUX_OPTIONAL_DATA_AUDIOLANG:
-      if (channel >= 0 && channel < this->audio_track_count) {
+      if (channel >= 0 && (unsigned)channel < this->audio_track_count) {
 
         AVStream *st = this->fmt_ctx->streams[this->audio_stream_idx[channel]];
         AVDictionaryEntry *tag = NULL;
@@ -648,13 +684,16 @@ static int avformat_seek (avformat_demux_plugin_t *this,
     if (av_seek_frame(this->fmt_ctx, -1, pos, 0) >= 0) {
       return 0;
     }
+    return -1;
   }
 
+#if 0
   /* seek to byte offset */
   pos = (int64_t)start_pos * avio_size(this->fmt_ctx->pb) / 65535;
   if (av_seek_frame(this->fmt_ctx, -1, pos, AVSEEK_FLAG_BYTE) >= 0) {
     return 0;
   }
+#endif
 
   /* stream does not support seeking to byte offset. Final try with timestamp. */
   pos = (int64_t)start_pos * this->fmt_ctx->duration / 65535;
