@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2014 the xine project
+ * Copyright (C) 2001-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -72,10 +72,19 @@ typedef struct ff_audio_decoder_s {
   int               decoder_ok;
 
   AVCodecParserContext *parser_context;
-#if AVAUDIO > 3
+#if XFF_AUDIO > 3
   AVFrame          *av_frame;
 #endif
 
+  /* AAC ADTS */
+  uint32_t          buftype;
+#define AAC_MODE_PROBE -8
+#define AAC_MODE_OFF    0
+#define AAC_MODE_RAW    1
+#define AAC_MODE_ADTS   2
+  int               aac_mode;
+
+
   /* decoder settings */
   int               ff_channels;
   int               ff_bits;
@@ -103,33 +112,108 @@ typedef struct ff_audio_decoder_s {
 
 #include "ff_audio_list.h"
 
-#define malloc16(s) realloc16(NULL,s)
-#define free16(p) realloc16(p,0)
-
-static void *realloc16 (void *m, size_t s) {
-  unsigned long diff, diff2;
-  unsigned char *p = m, *q;
-  if (p) {
-    diff = p[-1];
-    if (s == 0) {
-      free (p - diff);
-      return (NULL);
+static void ff_aac_mode_set (ff_audio_decoder_t *this, int reset) {
+  if ((this->buftype == BUF_AUDIO_AAC) || (this->buftype == BUF_AUDIO_AAC_LATM)) {
+    if (reset) {
+      this->aac_mode = AAC_MODE_PROBE;
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+        "ffmpeg_audio_dec: looking for possible AAC ADTS syncwords...\n");
+    }
+    if ((this->aac_mode < 0) || (this->aac_mode == AAC_MODE_ADTS)) {
+      if (this->context && this->context->extradata_size) {
+        xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+          "ffmpeg_audio_dec: AAC raw mode with global header\n");
+        this->aac_mode = AAC_MODE_RAW;
+      }
     }
-    q = realloc (p - diff, s + 16);
-    if (!q) return (q);
-    diff2 = 16 - ((unsigned long)q & 15);
-    if (diff2 != diff) memmove (q + diff2, q + diff, s);
   } else {
-    if (s == 0) return (NULL);
-    q = malloc (s + 16);
-    if (!q) return (q);
-    diff2 = 16 - ((unsigned long)q & 15);
+    this->aac_mode = AAC_MODE_OFF;
   }
-  q += diff2;
-  q[-1] = diff2;
-  return (q);
 }
 
+static int ff_audio_open_codec (ff_audio_decoder_t *this, unsigned int codec_type);
+
+/* return -1 (need more data), 0 (no parsing done), > 0 (offset + size of found frame) */
+static int ff_aac_mode_parse (ff_audio_decoder_t *this, uint8_t *buf, int size, int *offs) {
+  int i;
+  uint32_t v;
+  *offs = 0;
+  if (this->aac_mode < 0) {
+    /* probe */
+    v = 0;
+    for (i = 0; i < size; i++) {
+      v <<= 8;
+      v |= buf[i];
+      /* also test the "layer" bits for 0 (mpeg layer 4 audio). */
+      /* dont get fooled by 0xff padding bytes. */
+      if ((v & 0xfff6) == 0xfff0) {
+        uint32_t s;
+        /* test header size */
+        if (size - i < 7 - 1)
+          continue;
+        /* read frame size */
+        s = (_X_BE_32 (buf + i + 2) >> 13) & 0x1fff;
+        if (s < 7)
+          continue;
+        /* test for next ADTS frame following */
+        if (size - i < s + 7 - 1)
+          continue;
+        if ((((buf[i + s - 1] << 8) | buf[i + s]) & 0xfff6) != 0xfff0)
+          continue;
+        *offs = --i;
+        xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+          "ffmpeg_audio_dec: found AAC ADTS syncword after %d bytes\n", i);
+        if (this->buftype == BUF_AUDIO_AAC_LATM) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+            "ffmpeg_audio_dec: stream says LATM but is ADTS -> switching decoders\n");
+          if (this->context && this->decoder_ok) {
+            pthread_mutex_lock (&ffmpeg_lock);
+            avcodec_close (this->context);
+            pthread_mutex_unlock (&ffmpeg_lock);
+            this->decoder_ok = 0;
+          }
+          this->codec = NULL;
+          ff_audio_open_codec (this, BUF_AUDIO_AAC);
+        }
+        this->aac_mode = AAC_MODE_ADTS - 1;
+        break;
+      }
+    }
+    this->aac_mode++;
+    if (this->aac_mode < 0) {
+      if (size >= 2 * 0x1fff)
+        this->aac_mode = AAC_MODE_OFF;
+      else
+        return -1;
+    }
+    if (this->aac_mode == AAC_MODE_OFF)
+      xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+        "ffmpeg_audio_dec: no ADTS frames found\n");
+  }
+  if (this->aac_mode == AAC_MODE_ADTS) {
+    v = 0;
+    for (i = *offs; i < size; i++) {
+      v <<= 8;
+      v |= buf[i];
+      if ((v & 0xfff6) == 0xfff0) {
+        uint32_t s;
+        /* test header size */
+        if (size - i < 7 - 1)
+          return -1;
+        /* read frame size */
+        s = (_X_BE_32 (buf + i + 2) >> 13) & 0x1fff;
+        if (s < 7)
+          continue;
+        *offs = --i;
+        if (size - i < s)
+          return -1;
+        return i + s;
+      }
+    }
+    return -1;
+  }
+  return 0;
+}
 
 static void ff_audio_ensure_buffer_size(ff_audio_decoder_t *this, int size) {
   if (size > this->bufsize) {
@@ -137,7 +221,7 @@ static void ff_audio_ensure_buffer_size(ff_audio_decoder_t *this, int size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
             _("ffmpeg_audio_dec: increasing buffer to %d to avoid overflow.\n"),
             this->bufsize);
-    this->buf = realloc16 (this->buf, this->bufsize + FF_INPUT_BUFFER_PADDING_SIZE);
+    this->buf = xine_realloc_aligned (this->buf, this->bufsize + FF_INPUT_BUFFER_PADDING_SIZE);
   }
 }
 
@@ -151,6 +235,8 @@ static void ff_audio_handle_special_buffer(ff_audio_decoder_t *this, buf_element
     this->context->extradata = malloc (buf->decoder_info[2] + FF_INPUT_BUFFER_PADDING_SIZE);
     memcpy (this->context->extradata, buf->decoder_info_ptr[2], buf->decoder_info[2]);
     memset (this->context->extradata + buf->decoder_info[2], 0, FF_INPUT_BUFFER_PADDING_SIZE);
+
+    ff_aac_mode_set (this, 0);
   }
 }
 
@@ -161,6 +247,8 @@ static void ff_audio_init_codec(ff_audio_decoder_t *this, unsigned int codec_typ
 
   for(i = 0; i < sizeof(ff_audio_lookup)/sizeof(ff_codec_t); i++)
     if(ff_audio_lookup[i].type == codec_type) {
+      this->buftype = codec_type;
+      ff_aac_mode_set (this, 1);
       pthread_mutex_lock (&ffmpeg_lock);
       this->codec = avcodec_find_decoder(ff_audio_lookup[i].id);
       pthread_mutex_unlock (&ffmpeg_lock);
@@ -177,7 +265,7 @@ static void ff_audio_init_codec(ff_audio_decoder_t *this, unsigned int codec_typ
     return;
   }
 
-#if AVAUDIO < 4
+#if XFF_AUDIO < 4
   /* Try to make the following true */
   this->context->request_sample_fmt = AV_SAMPLE_FMT_S16;
   /* For lavc v54+, we have our channel mixer that wants default float samples to fix
@@ -201,7 +289,7 @@ static void ff_audio_init_codec(ff_audio_decoder_t *this, unsigned int codec_typ
    *  - DVB streams where multiple AAC LATM frames are packed to single PES
    *  - DVB streams where MPEG audio frames do not follow PES packet boundaries
    */
-#if AVPARSE > 1
+#if XFF_PARSE > 1
   if (codec_type == BUF_AUDIO_AAC_LATM ||
       codec_type == BUF_AUDIO_EAC3 ||
       codec_type == BUF_AUDIO_MPEG) {
@@ -232,7 +320,7 @@ static int ff_audio_open_codec(ff_audio_decoder_t *this, unsigned int codec_type
   }
 
   pthread_mutex_lock (&ffmpeg_lock);
-  if (avcodec_open (this->context, this->codec) < 0) {
+  if (XFF_AVCODEC_OPEN (this->context, this->codec) < 0) {
     pthread_mutex_unlock (&ffmpeg_lock);
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
              _("ffmpeg_audio_dec: couldn't open decoder\n"));
@@ -551,9 +639,20 @@ static void ff_map_channels (ff_audio_decoder_t *this) {
 static int ff_audio_decode (ff_audio_decoder_t *this,
   int16_t *decode_buffer, int *decode_buffer_size, uint8_t *buf, int size) {
   int consumed;
-  int parser_consumed = 0;
+  int parser_consumed;
+  int offs;
 
-#if AVPARSE > 1
+  parser_consumed = ff_aac_mode_parse (this, buf, size, &offs);
+  if (parser_consumed < 0) {
+    *decode_buffer_size = 0;
+    return offs;
+  } else if (parser_consumed > 0) {
+    buf += offs;
+    size = parser_consumed - offs;
+  }
+
+#if XFF_PARSE > 1
+  else
   if (this->parser_context) {
     uint8_t *outbuf;
     int      outsize;
@@ -578,19 +677,19 @@ static int ff_audio_decode (ff_audio_decoder_t *this,
     buf  = outbuf;
     size = outsize;
   }
-#endif /* AVPARSE > 1 */
+#endif /* XFF_PARSE > 1 */
 
-#if AVAUDIO > 2
+#if XFF_AUDIO > 2
   AVPacket avpkt;
   av_init_packet (&avpkt);
   avpkt.data = buf;
   avpkt.size = size;
   avpkt.flags = AV_PKT_FLAG_KEY;
-#  if AVAUDIO > 3
+#  if XFF_AUDIO > 3
   int got_frame;
   float gain = this->class->gain;
   if (!this->av_frame)
-    this->av_frame = avcodec_alloc_frame ();
+    this->av_frame = XFF_ALLOC_FRAME ();
 
   consumed = avcodec_decode_audio4 (this->context, this->av_frame, &got_frame, &avpkt);
   if ((consumed >= 0) && got_frame) {
@@ -618,80 +717,90 @@ static int ff_audio_decode (ff_audio_decoder_t *this,
       channels = 2;
     gain /= (float)(1 << shift);
     switch (this->context->sample_fmt) {
-#define MIX_AUDIO(stype,planar,idx,num,dindx) {\
-    stype *p1, *p2, *p3, *p4;\
+      /* "* 0.75" serves same purpose as "gain3" below. */
+#define MIX_AUDIO(stype,planar,idx,num,dindx) do {\
+    const stype *p1, *p2, *p3, *p4;\
     int i, sstep;\
     int8_t *x = idx;\
     int16_t *dptr = (int16_t *)decode_buffer + dindx;\
     if (planar) {\
       p1 = (stype *)this->av_frame->extended_data[x[0]];\
+      if (!p1) break;\
       sstep = 1;\
     } else {\
-      p1 = (stype *)this->av_frame->extended_data[0] + x[0];\
+      p1 = (stype *)this->av_frame->extended_data[0];\
+      if (!p1) break;\
+      p1 += x[0];\
       sstep = this->ff_channels;\
     }\
     if (num == 1) {\
-      if (p1) for (i = 0; i < samples; i++) {\
+      for (i = 0; i < samples; i++) {\
         int32_t v = MIX_FIX(*p1);\
         p1       += sstep;\
         v       >>= shift;\
         *dptr     = (v);\
         dptr     += channels;\
       }\
-    } else {\
-      if (planar)\
-        p2 = (stype *)this->av_frame->extended_data[x[1]];\
-      else\
-        p2 = (stype *)this->av_frame->extended_data[0] + x[1];\
-      if (num == 2) {\
-        if (p1 && p2) for (i = 0; i < samples; i++) {\
-          int32_t v = MIX_FIX(*p1);\
-          p1       += sstep;\
-          v        += MIX_FIX(*p2);\
-          p2       += sstep;\
-          v       >>= shift;\
-          *dptr     = CLIP_16(v);\
-          dptr     += channels;\
-        }\
-      } else {\
-        if (planar)\
-          p3 = (stype *)this->av_frame->extended_data[x[2]];\
-        else\
-          p3 = (stype *)this->av_frame->extended_data[0] + x[2];\
-        if (num == 3) {\
-          if (p1 && p2 && p3) for (i = 0; i < samples; i++) {\
-            int32_t v = MIX_FIX(*p1);\
-            p1       += sstep;\
-            v        += MIX_FIX(*p2);\
-            p2       += sstep;\
-            v        += MIX_FIX(*p3);\
-            p3       += sstep;\
-            v       >>= shift;\
-            *dptr     = CLIP_16(v);\
-            dptr     += channels;\
-          }\
-        } else {\
-          if (planar)\
-            p4 = (stype *)this->av_frame->extended_data[x[3]];\
-          else\
-            p4 = (stype *)this->av_frame->extended_data[0] + x[3];\
-          if (p1 && p2 && p3 && p4) for (i = 0; i < samples; i++) {\
-            int32_t v = MIX_FIX(*p1);\
-            p1       += sstep;\
-            v        += MIX_FIX(*p2);\
-            p2       += sstep;\
-            v        += MIX_FIX(*p3);\
-            p3       += sstep;\
-            v        += MIX_FIX(*p4);\
-            p4       += sstep;\
-            v       >>= shift;\
-            *dptr     = CLIP_16(v);\
-            dptr     += channels;\
-          }\
-        }\
-      }\
+      break;\
     }\
-  }
+    if (planar) {\
+      p2 = (stype *)this->av_frame->extended_data[x[1]];\
+      if (!p2) break;\
+    } else\
+      p2 = (stype *)this->av_frame->extended_data[0] + x[1];\
+    if (num == 2) {\
+      for (i = 0; i < samples; i++) {\
+        int32_t v = MIX_FIX(*p2);\
+        p2       += sstep;\
+        v        -= v >> 2;\
+        v        += MIX_FIX(*p1);\
+        p1       += sstep;\
+        v       >>= shift;\
+        *dptr     = CLIP_16(v);\
+        dptr     += channels;\
+      }\
+      break;\
+    }\
+    if (planar) {\
+      p3 = (stype *)this->av_frame->extended_data[x[2]];\
+      if (!p3) break;\
+    } else\
+      p3 = (stype *)this->av_frame->extended_data[0] + x[2];\
+    if (num == 3) {\
+      for (i = 0; i < samples; i++) {\
+        int32_t v = MIX_FIX(*p2);\
+        p2       += sstep;\
+        v        += MIX_FIX(*p3);\
+        p3       += sstep;\
+        v        -= v >> 2;\
+        v        += MIX_FIX(*p1);\
+        p1       += sstep;\
+        v       >>= shift;\
+        *dptr     = CLIP_16(v);\
+        dptr     += channels;\
+      }\
+      break;\
+    }\
+    if (planar) {\
+      p4 = (stype *)this->av_frame->extended_data[x[3]];\
+      if (!p4) break;\
+    } else\
+      p4 = (stype *)this->av_frame->extended_data[0] + x[3];\
+    for (i = 0; i < samples; i++) {\
+      int32_t v = MIX_FIX(*p2);\
+      p2       += sstep;\
+      v        += MIX_FIX(*p3);\
+      p3       += sstep;\
+      v        += MIX_FIX(*p4);\
+      p4       += sstep;\
+      v        -= v >> 2;\
+      v        += MIX_FIX(*p1);\
+      p1       += sstep;\
+      v       >>= shift;\
+      *dptr     = CLIP_16(v);\
+      dptr     += channels;\
+    }\
+  }  while (0);
 #define MIX_FIX(v) (((int16_t)(v)<<8)^0x8000)
       case AV_SAMPLE_FMT_U8P:
         MIX_AUDIO (uint8_t, 1, this->left,  this->front_mixes, 0);
@@ -735,70 +844,83 @@ static int ff_audio_decode (ff_audio_decoder_t *this,
       break;
 #undef MIX_FIX
 #undef MIX_AUDIO
-#define MIX_AUDIO(stype,planar,idx,num,dindx) {\
-    stype *p1, *p2, *p3, *p4;\
+      /* Note on "gain3" below:                                          */
+      /* - center and lfe downmix to both front left and right.          */
+      /*   This effectively doubles their power, so compensate by -3dB.  */
+      /* - surround channels often contain some sound effects that may   */
+      /*   confuse when coming front, so the same -3dB do help here too. */
+#define MIX_AUDIO(stype,planar,idx,num,dindx) do {\
+    const stype *p1, *p2, *p3, *p4;\
     int i, sstep;\
+    float gain3;\
     int8_t *x = idx;\
     int16_t *dptr = (int16_t *)decode_buffer + dindx;\
     if (planar) {\
       p1 = (stype *)this->av_frame->extended_data[x[0]];\
+      if (!p1) break;\
       sstep = 1;\
     } else {\
-      p1 = (stype *)this->av_frame->extended_data[0] + x[0];\
+      p1 = (stype *)this->av_frame->extended_data[0];\
+      if (!p1) break;\
+      p1 += x[0];\
       sstep = this->ff_channels;\
     }\
     if (num == 1) {\
-      if (p1) for (i = 0; i < samples; i++) {\
+      for (i = 0; i < samples; i++) {\
         int32_t v = (*p1) * gain;\
         p1       += sstep;\
         *dptr     = CLIP_16(v);\
         dptr     += channels;\
       }\
-    } else {\
-      if (planar)\
-        p2 = (stype *)this->av_frame->extended_data[x[1]];\
-      else\
-        p2 = (stype *)this->av_frame->extended_data[0] + x[1];\
-      if (num == 2) {\
-        if (p1 && p2) for (i = 0; i < samples; i++) {\
-          int32_t v = (*p1 + *p2) * gain;\
-          p1       += sstep;\
-          p2       += sstep;\
-          *dptr     = CLIP_16(v);\
-          dptr     += channels;\
-        }\
-      } else {\
-        if (planar)\
-          p3 = (stype *)this->av_frame->extended_data[x[2]];\
-        else\
-          p3 = (stype *)this->av_frame->extended_data[0] + x[2];\
-        if (num == 3) {\
-          if (p1 && p2 && p3) for (i = 0; i < samples; i++) {\
-            int32_t v = (*p1 + *p2 + *p3) * gain;\
-            p1       += sstep;\
-            p2       += sstep;\
-            p3       += sstep;\
-            *dptr     = CLIP_16(v);\
-            dptr     += channels;\
-          }\
-        } else {\
-          if (planar)\
-            p4 = (stype *)this->av_frame->extended_data[x[3]];\
-          else\
-            p4 = (stype *)this->av_frame->extended_data[0] + x[3];\
-          if (p1 && p2 && p3 && p4) for (i = 0; i < samples; i++) {\
-            int32_t v = (*p1 + *p2 + *p3 + *p4) * gain;\
-            p1       += sstep;\
-            p2       += sstep;\
-            p3       += sstep;\
-            p4       += sstep;\
-            *dptr     = CLIP_16(v);\
-            dptr     += channels;\
-          }\
-        }\
-      }\
+      break;\
     }\
-  }
+    gain3 = gain * 0.7071;\
+    if (planar) {\
+      p2 = (stype *)this->av_frame->extended_data[x[1]];\
+      if (!p2) break;\
+    } else\
+      p2 = (stype *)this->av_frame->extended_data[0] + x[1];\
+    if (num == 2) {\
+      for (i = 0; i < samples; i++) {\
+        int32_t v = (*p1) * gain + (*p2) * gain3;\
+        p1       += sstep;\
+        p2       += sstep;\
+        *dptr     = CLIP_16(v);\
+        dptr     += channels;\
+      }\
+      break;\
+    }\
+    if (planar) {\
+      p3 = (stype *)this->av_frame->extended_data[x[2]];\
+      if (!p3) break;\
+    } else\
+      p3 = (stype *)this->av_frame->extended_data[0] + x[2];\
+    if (num == 3) {\
+      for (i = 0; i < samples; i++) {\
+        int32_t v = (*p1) * gain + (*p2 + *p3) * gain3;\
+        p1       += sstep;\
+        p2       += sstep;\
+        p3       += sstep;\
+        *dptr     = CLIP_16(v);\
+        dptr     += channels;\
+      }\
+      break;\
+    }\
+    if (planar) {\
+      p4 = (stype *)this->av_frame->extended_data[x[3]];\
+      if (!p4) break;\
+    } else\
+      p4 = (stype *)this->av_frame->extended_data[0] + x[3];\
+    for (i = 0; i < samples; i++) {\
+      int32_t v = (*p1) * gain + (*p2 + *p3 + *p4) * gain3;\
+      p1       += sstep;\
+      p2       += sstep;\
+      p3       += sstep;\
+      p4       += sstep;\
+      *dptr     = CLIP_16(v);\
+      dptr     += channels;\
+    }\
+  } while (0);
       case AV_SAMPLE_FMT_FLTP: /* the most popular one */
         MIX_AUDIO (float, 1, this->left,  this->front_mixes, 0);
         MIX_AUDIO (float, 1, this->right, this->front_mixes, 1);
@@ -889,6 +1011,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
       memset(&this->buf[this->size], 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
       while (this->size>=0) {
+
         decode_buffer_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
 
         bytes_consumed = ff_audio_decode (this, (int16_t *)this->decode_buffer, &decode_buffer_size,
@@ -942,7 +1065,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 	  }
 	}
 
-#if AVAUDIO < 4
+#if XFF_AUDIO < 4
         /* Old style postprocessing */
         if (codec_type == BUF_AUDIO_WMAPRO) {
           /* the above codecs output float samples, not 16-bit integers */
@@ -1070,17 +1193,18 @@ static void ff_audio_reset (audio_decoder_t *this_gen) {
 
   /* try to reset the wma decoder */
   if( this->decoder_ok ) {
-#if AVAUDIO > 3
-    avcodec_free_frame (&this->av_frame);
+#if XFF_AUDIO > 3
+    XFF_FREE_FRAME (this->av_frame);
 #endif
     pthread_mutex_lock (&ffmpeg_lock);
     avcodec_close (this->context);
-    if (avcodec_open (this->context, this->codec) < 0)
+    if (XFF_AVCODEC_OPEN (this->context, this->codec) < 0)
       this->decoder_ok = 0;
     pthread_mutex_unlock (&ffmpeg_lock);
   }
 
   ff_audio_reset_parser(this);
+  ff_aac_mode_set (this, 1);
 }
 
 static void ff_audio_discontinuity (audio_decoder_t *this_gen) {
@@ -1090,6 +1214,7 @@ static void ff_audio_discontinuity (audio_decoder_t *this_gen) {
   this->size = 0;
 
   ff_audio_reset_parser(this);
+  ff_aac_mode_set (this, 0);
 }
 
 static void ff_audio_dispose (audio_decoder_t *this_gen) {
@@ -1104,8 +1229,8 @@ static void ff_audio_dispose (audio_decoder_t *this_gen) {
   }
 
   if( this->context && this->decoder_ok ) {
-#if AVAUDIO > 3
-    avcodec_free_frame (&this->av_frame);
+#if XFF_AUDIO > 3
+    XFF_FREE_FRAME (this->av_frame);
 #endif
     pthread_mutex_lock (&ffmpeg_lock);
     avcodec_close (this->context);
@@ -1114,14 +1239,14 @@ static void ff_audio_dispose (audio_decoder_t *this_gen) {
 
   ff_audio_output_close(this);
 
-  free16 (this->buf);
-  free16 (this->decode_buffer);
+  xine_free_aligned (this->buf);
+  xine_free_aligned (this->decode_buffer);
 
-  if(this->context && this->context->extradata)
-    free(this->context->extradata);
-
-  if(this->context)
-    av_free(this->context);
+  if (this->context) {
+    _x_freep (&this->context->extradata);
+    this->context->extradata_size = 0;
+    XFF_FREE_CONTEXT (this->context);
+  }
 
   free (this_gen);
 }
@@ -1149,9 +1274,9 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
 
   ff_audio_ensure_buffer_size(this, AUDIOBUFSIZE);
 
-  this->context = avcodec_alloc_context();
-  this->decode_buffer = malloc16 (AVCODEC_MAX_AUDIO_FRAME_SIZE);
-#if AVAUDIO > 3
+  this->context = XFF_ALLOC_CONTEXT();
+  this->decode_buffer = xine_malloc_aligned (AVCODEC_MAX_AUDIO_FRAME_SIZE);
+#if XFF_AUDIO > 3
   this->av_frame = NULL;
 #endif
   return &this->audio_decoder;
