@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -97,6 +97,8 @@ typedef struct demux_flac_class_s {
 
 } demux_flac_class_t;
 
+void *demux_flac_init_class (xine_t *xine, void *data);
+
 /* FLAC Callbacks */
 static
 #ifdef LEGACY_FLAC
@@ -227,6 +229,10 @@ flac_length_callback (const FLAC__SeekableStreamDecoder *decoder,
 
     offset = input->get_length (input);
 
+    if (offset > 0) {
+      *stream_length = offset;
+    }
+
     /* FIXME, can flac handle -1 as offset ? */
 #ifdef LEGACY_FLAC
     return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
