@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -70,6 +70,9 @@
 #define DEBUG_ODML
 #define LOG
 */
+
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/demux.h>
@@ -1891,6 +1894,16 @@ static void demux_avi_send_headers (demux_plugin_t *this_gen) {
   for(i=0; i < this->avi->n_audio; i++) {
     this->avi->audio[i]->audio_type = _x_formattag_to_buf_audio (this->avi->audio[i]->wavex->wFormatTag);
 
+    if ((uint16_t)this->avi->audio[i]->wavex->wFormatTag == 0xfffe /*WAVE_FORMAT_EXTENSIBLE */ &&
+        this->avi->audio[i]->wavex_len >= 18 + 6 + 4 ) {
+
+      /* extended header */
+      char *p = (char *)this->avi->audio[i]->wavex + 18 /*sizeof(this->avi->audio[i]->wavex*/;
+      uint32_t fourcc = _X_LE_32(p + 6);
+
+      this->avi->audio[i]->audio_type = _x_formattag_to_buf_audio (fourcc);
+    }
+
     /* special case time: An AVI file encoded with Xan video will have Xan
      * DPCM audio marked as PCM; hack around this */
     if (this->avi->video_type == BUF_VIDEO_XXAN) {
@@ -2317,7 +2330,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 /*
  * demux avi class
  */
-static void *init_class (xine_t *xine, void *data) {
+void *demux_avi_init_class (xine_t *xine, void *data) {
   demux_avi_class_t     *this;
 
   this = calloc(1, sizeof(demux_avi_class_t));
@@ -2334,15 +2347,3 @@ static void *init_class (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_avi = {
-  10                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "avi", XINE_VERSION_CODE, &demux_info_avi, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
