@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -353,6 +353,8 @@ static void demux_flac_send_headers(demux_plugin_t *this_gen) {
   xine_waveformatex wave;
   int bits;
 
+  memset(&wave, 0, sizeof(wave));
+
   this->audio_fifo  = this->stream->audio_fifo;
 
   /* send start buffers */
@@ -536,7 +538,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   case METHOD_EXPLICIT:
 
     if (!open_flac_file(this)) {
-      free (this);
+      demux_flac_dispose (&this->demux_plugin);
       return NULL;
     }
 
