@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2013 the xine project
+ * Copyright (C) 2004-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,6 +49,9 @@
 /*
 #define LOG
 */
+
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/compat.h>
@@ -111,6 +114,17 @@ typedef struct {
    limited clip duration to 4:39:37.215. The backwards compatible solution:
    hand over 1 byte from stream ID. Hence the weird byte order 2-1-0-3 */
 #define gettimestamp(p,o) (((((((uint32_t)p[o+3]<<8)|p[o])<<8)|p[o+1])<<8)|p[o+2])
+#if defined(__GNUC__) && ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ > 3)) || defined(__clang__)
+#if defined(__i386) || defined(__i386__) || defined(__x86_64) || defined(__x86_64__)
+#undef gettimestamp
+#define gettimestamp(p,o) ({ \
+  uint32_t tempi; \
+  __builtin_memcpy (&tempi, (p + o), 4); \
+  __asm__ __volatile__ ("bswapl\t%0\n\trorl\t$8, %0" : "=r" (tempi) : "0" (tempi) : "cc"); \
+  tempi; \
+})
+#endif
+#endif
 
 #define FLV_FLAG_HAS_VIDEO       0x01
 #define FLV_FLAG_HAS_AUDIO       0x04
@@ -447,7 +461,7 @@ static int parse_amf (demux_flv_t *this, unsigned char *buf, int size) {
         tstruct = gmtime (&tsecs);
         if (tstruct) {
           char ts[200];
-          if (strftime (ts, 200, "%x %X", tstruct) >= 0)
+          if (strftime (ts, 200, "%x %X", tstruct) > 0)
             xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "%s\n", ts);
         }
       break;
@@ -1101,7 +1115,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_flv_init_class (xine_t *xine, void *data) {
   demux_flv_class_t     *this;
 
   this = calloc(1, sizeof (demux_flv_class_t));
@@ -1118,16 +1132,3 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_flv = {
-  10                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "flashvideo", XINE_VERSION_CODE, &demux_info_flv, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
