@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -43,6 +43,8 @@
 #define LOG
 */
 
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/demux.h>
@@ -62,7 +64,7 @@ static void check_newpts (demux_matroska_t *this, int64_t pts,
 
     diff = pts - track->last_pts;
 
-    if (pts && (this->send_newpts || (track->last_pts && abs(diff)>WRAP_THRESHOLD)) ) {
+    if (pts && (this->send_newpts || (track->last_pts && llabs(diff)>WRAP_THRESHOLD)) ) {
       int i;
 
       lprintf ("sending newpts %" PRId64 ", diff %" PRId64 ", track %d\n", pts, diff, track->track_num);
@@ -364,6 +366,7 @@ static int parse_content_encoding (demux_matroska_t *this, matroska_track_t *tra
         lprintf("ContentCompression\n");
         if (!ebml_read_master (ebml, &elem))
           return 0;
+	track->compress_algo = 0; /* default */
         if ((elem.len > 0) && !parse_content_compression(this, track))
           return 0;
         break;
@@ -990,34 +993,22 @@ static void handle_sub_utf8 (demux_plugin_t *this_gen, matroska_track_t *track,
 }
 
 
-/* Note: This function assumes that the VobSub track is compressed with zlib.
- * This is not necessarily true - or not enough. The Matroska 'content
- * encoding' elements allow for a layer of changes applied to the contents,
- * e.g. compression or encryption. Anyway, only zlib compression is used
- * at the moment, and everyone compresses the VobSubs.
- */
-static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
-                           int decoder_flags,
-                           uint8_t *data, size_t data_len,
-                           int64_t data_pts, int data_duration,
-                           int input_normpos, int input_time) {
-  demux_matroska_t *this = (demux_matroska_t *) this_gen;
-  buf_element_t *buf;
-
-  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB ||
-      track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+static int uncompress_zlib(demux_matroska_t *this,
+                           const uint8_t *data, size_t data_len, uint8_t **out_data, size_t *out_data_len)
+{
     z_stream zstream;
     uint8_t *dest;
-    int old_data_len, result;
+    int result;
+
+    *out_data = NULL;
 
-    old_data_len = data_len;
     zstream.zalloc = (alloc_func) 0;
     zstream.zfree = (free_func) 0;
     zstream.opaque = (voidpf) 0;
     if (inflateInit (&zstream) != Z_OK) {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-              "demux_matroska: VobSub: zlib inflateInit failed.\n");
-      return;
+              "demux_matroska: zlib inflateInit failed.\n");
+      return -1;
     }
     zstream.next_in = (Bytef *)data;
     zstream.avail_in = data_len;
@@ -1031,32 +1022,58 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
       result = inflate (&zstream, Z_NO_FLUSH);
       if ((result != Z_OK) && (result != Z_STREAM_END)) {
         xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-                "demux_matroska: VobSub: zlib decompression failed for track %d (result = %d).\n",
-                (int)track->track_num, result);
+                "demux_matroska: zlib decompression failed: %d\n", result);
         free(dest);
         inflateEnd(&zstream);
-
-        if (result == Z_DATA_ERROR && track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
-          track->compress_algo = MATROSKA_COMPRESS_NONE;
-          data_len = old_data_len;
-          xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-                  "demux_matroska: VobSub: falling back to uncompressed mode.\n");
-          break;
-        }
-        return;
+	return 0;
       }
       zstream.avail_out += 4000;
     } while ((zstream.avail_out == 4000) &&
             (zstream.avail_in != 0) && (result != Z_STREAM_END));
 
-    if (track->compress_algo != MATROSKA_COMPRESS_NONE) {
-      data_len = zstream.total_out;
-      inflateEnd(&zstream);
+    *out_data = dest;
+    *out_data_len = zstream.total_out;
 
-      data = dest;
+    inflateEnd(&zstream);
+
+    return 1;
+}
+
+/* Note: This function assumes that the VobSub track is compressed with zlib.
+ * This is not necessarily true - or not enough. The Matroska 'content
+ * encoding' elements allow for a layer of changes applied to the contents,
+ * e.g. compression or encryption. Anyway, only zlib compression is used
+ * at the moment, and everyone compresses the VobSubs.
+ */
+static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
+                           int decoder_flags,
+                           uint8_t *data, size_t data_len,
+                           int64_t data_pts, int data_duration,
+                           int input_normpos, int input_time) {
+  demux_matroska_t *this = (demux_matroska_t *) this_gen;
+  buf_element_t *buf;
+  uint8_t *new_data = NULL;
+  size_t new_data_len = 0;
+
+  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB ||
+      track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+
+    if (uncompress_zlib(this, data, data_len, &new_data, &new_data_len) < 0) {
+      return;
+    }
+
+    if (!new_data) {
+      if (track->compress_algo == MATROSKA_COMPRESS_UNKNOWN) {
+	track->compress_algo = MATROSKA_COMPRESS_NONE;
+	xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+		"demux_matroska: VobSub: falling back to uncompressed mode.\n");
+      } else {
+	return;
+      }
+    } else {
       track->compress_algo = MATROSKA_COMPRESS_ZLIB;
-      lprintf("VobSub: decompression for track %d from %d to %d\n",
-              (int)track->track_num, old_data_len, data_len);
+      data = new_data;
+      data_len = new_data_len;
     }
   }
   else
@@ -1088,8 +1105,143 @@ static void handle_vobsub (demux_plugin_t *this_gen, matroska_track_t *track,
     buf->free_buffer(buf);
   }
 
-  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB)
-    free(data);
+  free(new_data);
+}
+
+static void handle_hdmv_pgs (demux_plugin_t *this_gen, matroska_track_t *track,
+                             int decoder_flags,
+                             uint8_t *data, size_t data_len,
+                             int64_t data_pts, int data_duration,
+                             int input_normpos, int input_time) {
+  demux_matroska_t *this = (demux_matroska_t *) this_gen;
+  uint8_t *new_data = NULL;
+  size_t new_data_len = 0;
+
+  if (track->compress_algo == MATROSKA_COMPRESS_ZLIB) {
+    uncompress_zlib(this, data, data_len, &new_data, &new_data_len);
+    if (!new_data)
+      return;
+    data = new_data;
+    data_len = new_data_len;
+  }
+
+  _x_demux_send_data(track->fifo, data, data_len,
+		     data_pts, track->buf_type, decoder_flags,
+		     input_normpos, input_time,
+		     0, 0);
+
+  free(new_data);
+}
+
+static void handle_hdmv_textst (demux_plugin_t *this_gen, matroska_track_t *track,
+                                int decoder_flags,
+                                uint8_t *data, size_t data_len,
+                                int64_t data_pts, int data_duration,
+                                int input_normpos, int input_time) {
+  buf_element_t *buf;
+  uint32_t *val;
+  char *dest;
+  int dest_len;
+  int dest_pos;
+  int i;
+  uint8_t *ptr;
+  int palette_update_flag;
+  int num_regions;
+
+  ptr = data;
+  if (*ptr != 0x82) {
+    lprintf("Not a dialog presentation segment\n");
+    return;
+  }
+  ++ptr;
+
+  // Skip length and start/end pts values
+  ptr += 12;
+
+  palette_update_flag = *ptr++;
+  if (palette_update_flag) {
+    lprintf("Dunno what to do with palette updates\n");
+    return;
+  }
+
+  buf = track->fifo->buffer_pool_alloc(track->fifo);
+  buf->type = track->buf_type;
+  buf->decoder_flags = decoder_flags | BUF_FLAG_SPECIAL;
+  buf->decoder_info[1] = BUF_SPECIAL_CHARSET_ENCODING;
+  buf->decoder_info_ptr[2] = "utf-8";
+  buf->decoder_info[2] = strlen(buf->decoder_info_ptr[2]);
+
+  val = (uint32_t *)buf->content;
+  *val++ = data_pts / 90;                    /* start time */
+  *val++ = (data_pts + data_duration) / 90;  /* end time   */
+
+  dest = buf->content + 8;
+  dest_len = buf->max_size - 8;
+  dest_pos = 0;
+
+  num_regions = *ptr++;
+  for (i = 0; i < num_regions; ++i) {
+    int length;
+    uint8_t *end;
+
+    // skip flags and style id reference
+    ptr += 2;
+
+    length = (ptr[0]<<8) | ptr[1];
+    ptr += 2;
+
+    end = ptr + length;
+    while (ptr < end && dest_pos < dest_len - 2) {
+      int marker;
+      int type;
+      int length;
+
+      marker = *ptr++;
+      if (marker != 0x1B)
+        continue;
+
+      type = *ptr++;
+      length = *ptr++;
+
+      if (type == 0x01) {
+        int k;
+        for (k = 0; k < length && dest_pos < dest_len - 1; ++k)
+          dest[dest_pos++] = ptr[k];
+      } else if (type == 0x0A) {
+        dest[dest_pos++] = '\n';
+      } else {
+        lprintf("Unknown TextST data %02X\n", type);
+      }
+
+      ptr += length;
+    }
+  }
+
+  dest[dest_pos] = 0;
+
+  track->fifo->put(track->fifo, buf);
+}
+
+static void fill_extra_data(matroska_track_t *track, uint32_t fourcc) {
+
+  xine_bmiheader *bih;
+
+  if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
+    track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
+
+  /* create a bitmap info header struct */
+  bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
+  bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
+  bih->biCompression = fourcc;
+  bih->biWidth = track->video_track->pixel_width;
+  bih->biHeight = track->video_track->pixel_height;
+  _x_bmiheader_le2me(bih);
+
+  /* add bih extra data */
+  memcpy(bih + 1, track->codec_private, track->codec_private_len);
+  free(track->codec_private);
+  track->codec_private = (uint8_t *)bih;
+  track->codec_private_len = bih->biSize;
 }
 
 static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
@@ -1276,49 +1428,16 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
     } else if ((!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_SP)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_ASP)) ||
                (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AP))) {
-      xine_bmiheader *bih;
 
       lprintf("MATROSKA_CODEC_ID_V_MPEG4_*\n");
-      if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
-        track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
-
-      /* create a bitmap info header struct for MPEG 4 */
-      bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
-      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
-      bih->biCompression = ME_FOURCC('M', 'P', '4', 'S');
-      bih->biWidth = track->video_track->pixel_width;
-      bih->biHeight = track->video_track->pixel_height;
-      _x_bmiheader_le2me(bih);
-
-      /* add bih extra data */
-      memcpy(bih + 1, track->codec_private, track->codec_private_len);
-      free(track->codec_private);
-      track->codec_private = (uint8_t *)bih;
-      track->codec_private_len = bih->biSize;
+      fill_extra_data(track, ME_FOURCC('M', 'P', '4', 'S'));
       track->buf_type = BUF_VIDEO_MPEG4;
 
       /* init as a vfw decoder */
       init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AVC)) {
-      xine_bmiheader *bih;
-
       lprintf("MATROSKA_CODEC_ID_V_MPEG4_AVC\n");
-      if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
-        track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
-
-      /* create a bitmap info header struct for h264 */
-      bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
-      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
-      bih->biCompression = ME_FOURCC('a', 'v', 'c', '1');
-      bih->biWidth = track->video_track->pixel_width;
-      bih->biHeight = track->video_track->pixel_height;
-      _x_bmiheader_le2me(bih);
-
-      /* add bih extra data */
-      memcpy(bih + 1, track->codec_private, track->codec_private_len);
-      free(track->codec_private);
-      track->codec_private = (uint8_t *)bih;
-      track->codec_private_len = bih->biSize;
+      fill_extra_data(track, ME_FOURCC('a', 'v', 'c', '1'));
       track->buf_type = BUF_VIDEO_H264;
 
       /* init as a vfw decoder */
@@ -1334,27 +1453,19 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_VIDEO_MPEG;
       init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_VP8)) {
-      xine_bmiheader *bih;
-
       lprintf("MATROSKA_CODEC_ID_V_VP8\n");
-      if (track->codec_private_len > 0x7fffffff - sizeof(xine_bmiheader))
-        track->codec_private_len = 0x7fffffff - sizeof(xine_bmiheader);
-
-      /* create a bitmap info header struct for vp8 */
-      bih = calloc(1, sizeof(xine_bmiheader) + track->codec_private_len);
-      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
-      bih->biCompression = ME_FOURCC('v', 'p', '8', '0');
-      bih->biWidth = track->video_track->pixel_width;
-      bih->biHeight = track->video_track->pixel_height;
-      _x_bmiheader_le2me(bih);
-
-      /* add bih extra data */
-      memcpy(bih + 1, track->codec_private, track->codec_private_len);
-      free(track->codec_private);
-      track->codec_private = (uint8_t *)bih;
-      track->codec_private_len = bih->biSize;
+      fill_extra_data(track, ME_FOURCC('v', 'p', '8', '0'));
       track->buf_type = BUF_VIDEO_VP8;
-
+      init_codec = init_codec_video;
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_VP9)) {
+      lprintf("MATROSKA_CODEC_ID_V_VP9\n");
+      fill_extra_data(track, ME_FOURCC('v', 'p', '9', '0'));
+      track->buf_type = BUF_VIDEO_VP9;
+      init_codec = init_codec_video;
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_HEVC)) {
+      lprintf("MATROSKA_CODEC_ID_V_HEVC\n");
+      fill_extra_data(track, ME_FOURCC('h', 'e', 'v', 'c'));
+      track->buf_type = BUF_VIDEO_HEVC;
       init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_REAL_RV10)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_REAL_RV20)) {
@@ -1383,7 +1494,15 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_audio;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_INT_BE)) {
+      lprintf("MATROSKA_CODEC_ID_A_PCM_INT_BE");
+      track->buf_type = BUF_AUDIO_LPCM_BE;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_INT_LE)) {
+      lprintf("MATROSKA_CODEC_ID_A_PCM_INT_LE");
+      track->buf_type = BUF_AUDIO_LPCM_LE;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_PCM_FLOAT)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_AC3)) {
       lprintf("MATROSKA_CODEC_ID_A_AC3\n");
@@ -1395,6 +1514,11 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_AUDIO_EAC3;
       init_codec = init_codec_audio;
 
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_TRUEHD)) {
+      lprintf("MATROSKA_CODEC_ID_A_TRUEHD\n");
+      track->buf_type = BUF_AUDIO_TRUEHD;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_FLAC)) {
       lprintf("MATROSKA_CODEC_ID_A_FLAC\n");
       track->buf_type = BUF_AUDIO_FLAC;
@@ -1411,6 +1535,11 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       track->buf_type = BUF_AUDIO_VORBIS;
       init_codec = init_codec_xiph;
 
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_OPUS)) {
+      lprintf("MATROSKA_CODEC_ID_A_OPUS\n");
+      track->buf_type = BUF_AUDIO_OPUS;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_ACM)) {
       xine_waveformatex *wfh;
       lprintf("MATROSKA_CODEC_ID_A_ACM\n");
@@ -1479,6 +1608,12 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_S_HDMV_PGS)) {
       lprintf("MATROSKA_CODEC_ID_S_HDMV_PGS\n");
       track->buf_type = BUF_SPU_HDMV;
+      track->handle_content = handle_hdmv_pgs;
+      init_codec = init_codec_spu;
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_S_HDMV_TEXTST)) {
+      lprintf("MATROSKA_CODEC_ID_S_HDMV_TEXTST\n");
+      track->buf_type = BUF_SPU_OGM;
+      track->handle_content = handle_hdmv_textst;
       init_codec = init_codec_spu;
     } else {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "unknown codec %s\n", track->codec_id);
@@ -1982,7 +2117,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
       break;
 
       case MATROSKA_FIXED_SIZE_LACING: {
-        int frame_size;
+        size_t frame_size;
 
         lprintf("fixed size lacing\n");
 
@@ -1990,7 +2125,7 @@ static int parse_block (demux_matroska_t *this, size_t block_size,
         for (i = 0; i < lace_num; i++) {
           frame[i] = frame_size;
         }
-        frame[lace_num] = block_size_left - (lace_num * frame_size);
+        frame[lace_num] = block_size_left - ((size_t)lace_num * frame_size);
         block_size_left = 0;
       }
       break;
@@ -2135,7 +2270,7 @@ static int parse_block_group(demux_matroska_t *this,
         if (!read_block_data(this, elem.len, this->compress_maxlen))
           return 0;
 
-          has_block = 1;
+        has_block = 1;
         break;
       case MATROSKA_ID_CL_BLOCKDURATION:
         /* should override track duration */
@@ -3031,8 +3166,8 @@ error:
 
   if (this != NULL && this->event_queue != NULL) {
     xine_event_dispose_queue(this->event_queue);
-    free(this);
   }
+  free(this);
 
   return NULL;
 }
@@ -3041,7 +3176,7 @@ error:
 /*
  * demux matroska class
  */
-static void *init_class (xine_t *xine, void *data) {
+void *demux_matroska_init_class (xine_t *xine, void *data) {
 
   demux_matroska_class_t     *this;
 
@@ -3061,15 +3196,3 @@ static void *init_class (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_matroska = {
-  10                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "matroska", XINE_VERSION_CODE, &demux_info_matroska, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
