@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -39,6 +39,8 @@
 #define LOG
 */
 
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/demux.h>
 #include <xine/xineutils.h>
@@ -1221,7 +1223,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_mpeg_init_class (xine_t *xine, void *data) {
   demux_mpeg_class_t     *this;
 
   this = calloc(1, sizeof(demux_mpeg_class_t));
@@ -1238,15 +1240,3 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_mpeg = {
-  9                        /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "mpeg", XINE_VERSION_CODE, &demux_info_mpeg, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
