@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -31,18 +31,14 @@
 #include <unistd.h>
 #include <string.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <mem.h>
-#else
-#  include <libavutil/mem.h>
-#endif
-
 #define LOG_MODULE "demux_mpeg_block"
 #define LOG_VERBOSE
 /*
 #define LOG
 */
 
+#include "group_video.h"
+
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/demux.h>
@@ -73,8 +69,6 @@ typedef struct demux_mpeg_block_s {
   int                   blocksize;
   int                   rate;
 
-  char                  cur_mrl[256];
-
   int64_t               nav_last_end_pts;
   int64_t               nav_last_start_pts;
   int64_t               last_pts[2];
@@ -1329,26 +1323,6 @@ static int demux_mpeg_block_seek (demux_plugin_t *this_gen,
 }
 
 
-static void demux_mpeg_block_accept_input (demux_mpeg_block_t *this,
-					   input_plugin_t *input) {
-
-  this->input = input;
-
-  if (strcmp (this->cur_mrl, input->get_mrl(input))) {
-
-    this->rate = 0;
-
-    strncpy (this->cur_mrl, input->get_mrl(input), 256);
-
-    lprintf ("mrl %s is new\n", this->cur_mrl);
-
-  }
-  else {
-    lprintf ("mrl %s is known, bitrate: %d\n",
-	     this->cur_mrl, this->rate * 50 * 8);
-  }
-}
-
 static int demux_mpeg_block_get_stream_length (demux_plugin_t *this_gen) {
 
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
@@ -1443,7 +1417,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
         input->seek(input, 0, SEEK_SET);
 
-        demux_mpeg_block_accept_input (this, input);
         lprintf("open_plugin:Accepting detection_method XINE_DEMUX_CONTENT_STRATEGY blocksize=%d\n",
                 this->blocksize);
 
@@ -1469,8 +1442,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
       free (this);
       return NULL;
     }
-
-    demux_mpeg_block_accept_input (this, input);
   }
   break;
 
@@ -1482,7 +1453,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_mpeg_block_init_class (xine_t *xine, void *data) {
 
   demux_mpeg_block_class_t     *this;
 
@@ -1500,15 +1471,3 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_mpeg_block = {
-  10                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "mpeg_block", XINE_VERSION_CODE, &demux_info_mpeg_block, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
