@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine project
+ * Copyright (C) 2001-2009 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -34,10 +34,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "demux.h"
-#include "buffer.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/demux.h>
+#include <xine/buffer.h>
 #include "bswap.h"
 #include "group_audio.h"
 
@@ -365,11 +365,8 @@ static int demux_ra_seek (demux_plugin_t *this_gen,
 static void demux_ra_dispose (demux_plugin_t *this_gen) {
   demux_ra_t *this = (demux_ra_t *) this_gen;
 
-  if(this->header)
-    free(this->header);
-  if (this->frame_buffer)
-    free(this->frame_buffer);
-
+  free(this->header);
+  free(this->frame_buffer);
   free(this);
 }
 
@@ -417,19 +414,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_EXTENSION: {
-    const char *extensions, *mrl;
-
-    mrl = input->get_mrl (input);
-    extensions = class_gen->get_extensions (class_gen);
-
-    if (!_x_demux_check_extension (mrl, extensions)) {
-      free (this);
-      return NULL;
-    }
-  }
-  /* falling through is intended */
-
+  case METHOD_BY_MRL:
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -448,39 +433,17 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static const char *get_description (demux_class_t *this_gen) {
-  return "RealAudio file demux plugin";
-}
-
-static const char *get_identifier (demux_class_t *this_gen) {
-  return "RA";
-}
-
-static const char *get_extensions (demux_class_t *this_gen) {
-  return "ra";
-}
-
-static const char *get_mimetypes (demux_class_t *this_gen) {
-  return "audio/x-realaudio: ra: RealAudio File;";
-}
-
-static void class_dispose (demux_class_t *this_gen) {
-  demux_ra_class_t *this = (demux_ra_class_t *) this_gen;
-
-  free (this);
-}
-
 void *demux_realaudio_init_plugin (xine_t *xine, void *data) {
   demux_ra_class_t     *this;
 
   this = calloc(1, sizeof(demux_ra_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.get_description = get_description;
-  this->demux_class.get_identifier  = get_identifier;
-  this->demux_class.get_mimetypes   = get_mimetypes;
-  this->demux_class.get_extensions  = get_extensions;
-  this->demux_class.dispose         = class_dispose;
+  this->demux_class.description     = N_("RealAudio file demux plugin");
+  this->demux_class.identifier      = "RA";
+  this->demux_class.mimetypes       = "audio/x-realaudio: ra: RealAudio File;";
+  this->demux_class.extensions      = "ra";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
 }
