@@ -1,6 +1,6 @@
 
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -148,17 +148,14 @@
 #include <string.h>
 #include <arpa/inet.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <crc.h>
-#else
-#  include <libavutil/crc.h>
-#endif
-
 #define LOG_MODULE "demux_ts"
 #define LOG_VERBOSE
 /*
 #define LOG
 */
+/* #define LOG_DYNAMIC_PMT */
+
+#include "group_video.h"
 
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
@@ -242,6 +239,8 @@
       ISO_14496_PART2_VIDEO = 0x10,     /* ISO/IEC 14496-2 Visual (MPEG-4) */
       ISO_14496_PART3_AUDIO = 0x11,     /* ISO/IEC 14496-3 Audio with LATM transport syntax */
       ISO_14496_PART10_VIDEO = 0x1b,    /* ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264) */
+      STREAM_VIDEO_HEVC = 0x24,
+
       STREAM_VIDEO_MPEG      = 0x80,
       STREAM_AUDIO_AC3       = 0x81,
 
@@ -337,7 +336,6 @@ typedef struct {
   xine_t           *xine;
   config_values_t  *config;
 
-  const AVCRC      *av_crc;
 } demux_ts_class_t;
 
 typedef struct {
@@ -935,7 +933,7 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
   crc32 |= (uint32_t) pkt[section_length - 2] << 8;
   crc32 |= (uint32_t) pkt[section_length - 1];
 
-  calc_crc32 = htonl (av_crc (this->class->av_crc, 0xffffffff, pkt, section_length - 4));
+  calc_crc32 = htonl (xine_crc32_ieee (0xffffffff, pkt, section_length - 4));
   if (crc32 != calc_crc32) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
 	     "demux_ts: demux error! PAT with invalid CRC32: packet_crc32: %.8x calc_crc32: %.8x\n",
@@ -1098,6 +1096,11 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
     return header_len;
   }
 
+  if (m->descriptor_tag == STREAM_VIDEO_HEVC) {
+    m->type      = BUF_VIDEO_HEVC;
+    return header_len;
+  }
+
   if (m->descriptor_tag == HDMV_SPU_BITMAP) {
     m->type |= BUF_SPU_HDMV;
     m->buf->decoder_info[2] = m->pes_bytes_left;
@@ -1319,7 +1322,7 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       m->buf->free_buffer(m->buf);
       m->buf = NULL;
 
-      m->corrupted_pes++;
+      m->corrupted_pes = 1;
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
               "demux_ts: PID 0x%.4x: corrupted pes encountered\n", m->pid);
     } else {
@@ -1580,7 +1583,7 @@ static void demux_ts_parse_pmt (demux_ts_t     *this,
   crc32 |= (uint32_t) pkt[section_length - 2] << 8;
   crc32 |= (uint32_t) pkt[section_length - 1];
 
-  calc_crc32 = htonl (av_crc (this->class->av_crc, 0xffffffff, pkt, section_length - 4));
+  calc_crc32 = htonl (xine_crc32_ieee (0xffffffff, pkt, section_length - 4));
   if (crc32 != calc_crc32) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
       "demux_ts: demux error! PMT with invalid CRC32: packet_crc32: %#.8x calc_crc32: %#.8x\n",
@@ -1668,6 +1671,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     case ISO_13818_VIDEO:
     case ISO_14496_PART2_VIDEO:
     case ISO_14496_PART10_VIDEO:
+    case STREAM_VIDEO_HEVC:
     case STREAM_VIDEO_VC1:
       if (this->videoPid == INVALID_PID) {
 #ifdef TS_PMT_LOG
@@ -1715,6 +1719,19 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 #endif
       break;
     case ISO_13818_PES_PRIVATE:
+      {
+        uint32_t format_identifier=0;
+        demux_ts_get_reg_desc(this, &format_identifier, stream + 5, stream_info_length);
+        if (format_identifier == 0x48455643 /* HEVC */ ) {
+          mi = demux_ts_dynamic_pmt_find (this, pid, BUF_VIDEO_BASE, STREAM_VIDEO_HEVC);
+          if (mi >= 0) {
+            this->videoMedia = mi;
+            this->videoPid = pid;
+          }
+          break;
+        }
+      }
+
       for (i = 5; i < coded_length; i += stream[i+1] + 2) {
 
         if ((stream[i] == DESCRIPTOR_AC3) || (stream[i] == DESCRIPTOR_EAC3) || (stream[i] == DESCRIPTOR_DTS)) {
@@ -2278,8 +2295,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 
   /* PMT */
   program_count=0;
-  while ((this->program_number[program_count] != INVALID_PROGRAM) &&
-         (program_count < MAX_PMTS)) {
+  while (program_count < MAX_PMTS &&
+          (this->program_number[program_count] != INVALID_PROGRAM)) {
     if (pid == this->pmt_pid[program_count]) {
 
 #ifdef TS_LOG
@@ -2515,6 +2532,12 @@ static int demux_ts_seek (demux_plugin_t *this_gen,
     this->buf_flag_seek = 1;
     _x_demux_flush_engine(this->stream);
 
+    /* Append sequence end code to video stream. */
+    /* Keep ffmpeg h.264 video decoder from piling up too many DR1 frames, */
+    /* and thus freezing video out. */
+    if (this->videoPid != INVALID_PID && this->stream->video_fifo)
+      post_sequence_end (this->stream->video_fifo, this->media[this->videoMedia].type);
+
   }
 
   demux_ts_tbre_reset (this);
@@ -2720,7 +2743,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 /*
  * ts demuxer class
  */
-static void *init_class (xine_t *xine, void *data) {
+void *demux_ts_init_class (xine_t *xine, void *data) {
 
   demux_ts_class_t     *this;
 
@@ -2740,22 +2763,6 @@ static void *init_class (xine_t *xine, void *data) {
   this->demux_class.extensions      = "ts m2t trp m2ts mts dvb:// dvbs:// dvbc:// dvbt://";
   this->demux_class.dispose         = default_demux_class_dispose;
 
-  this->av_crc = av_crc_get_table(AV_CRC_32_IEEE);
-
   return this;
 }
 
-
-/*
- * exported plugin catalog entry
- */
-static const demuxer_info_t demux_info_ts = {
-  10                       /* priority */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 27, "mpeg-ts", XINE_VERSION_CODE, &demux_info_ts, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
