@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -338,6 +338,7 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_A_REAL_ATRC    "A_REAL/ATRC"
 #define MATROSKA_CODEC_ID_A_FLAC         "A_FLAC"
 #define MATROSKA_CODEC_ID_A_OPUS         "A_OPUS"
+#define MATROSKA_CODEC_ID_A_TRUEHD       "A_TRUEHD"
 
 #define MATROSKA_CODEC_ID_S_TEXT_UTF8    "S_TEXT/UTF8"
 #define MATROSKA_CODEC_ID_S_TEXT_SSA     "S_TEXT/SSA"
@@ -348,6 +349,7 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_S_ASS          "S_ASS"         /* deprecated */
 #define MATROSKA_CODEC_ID_S_VOBSUB       "S_VOBSUB"
 #define MATROSKA_CODEC_ID_S_HDMV_PGS     "S_HDMV/PGS"
+#define MATROSKA_CODEC_ID_S_HDMV_TEXTST  "S_HDMV/TEXTST"
 
 /* block lacing */
 #define MATROSKA_NO_LACING               0x0
