@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  * Copyright (C) 2009-2011 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -176,6 +176,7 @@ typedef struct {
   uint8_t            error : 1;
   uint8_t            menu_open : 1;
   uint8_t            stream_flushed : 1;
+  uint8_t            stream_reset_done : 1;
   uint8_t            demux_action_req : 1;
   uint8_t            end_of_title : 1;
   uint8_t            pg_enable : 1;
@@ -292,12 +293,13 @@ static void open_overlay(bluray_input_plugin_t *this, int plane, uint16_t x, uin
   }
 
   this->osd[plane] = xine_osd_new(this->stream, x, y, w, h);
+  xine_osd_set_extent(this->osd[plane], w, h);
   clear_overlay(this->osd[plane]);
 }
 
 static void draw_bitmap(xine_osd_t *osd, const BD_OVERLAY * const ov)
 {
-  unsigned i;
+  size_t i;
 
   /* convert and set palette */
   if (ov->palette) {
@@ -319,8 +321,8 @@ static void draw_bitmap(xine_osd_t *osd, const BD_OVERLAY * const ov)
   /* uncompress and draw bitmap */
   if (ov->img && ov->w > 0 && ov->h > 0) {
     const BD_PG_RLE_ELEM *rlep = ov->img;
-    uint8_t *img = malloc(ov->w * ov->h);
-    unsigned pixels = ov->w * ov->h;
+    size_t pixels = (size_t)ov->w * ov->h;
+    uint8_t *img = malloc(pixels);
 
     for (i = 0; i < pixels; i += rlep->len, rlep++) {
       memset(img + i, rlep->color, rlep->len);
@@ -414,14 +416,14 @@ static void overlay_proc(void *this_gen, const BD_OVERLAY * const ov)
 
 static void open_argb_overlay(bluray_input_plugin_t *this, int plane, uint16_t x, uint16_t y, uint16_t w, uint16_t h)
 {
-  lprintf("open_argb_overlay(#%d,%d,%d)\n", plane, ov->w, ov->h);
+  lprintf("open_argb_overlay(#%d,%d,%d)\n", plane, w, h);
 
   open_overlay(this, plane, x, y, w, h);
 
   if (xine_osd_get_capabilities(this->osd[plane]) & XINE_OSD_CAP_ARGB_LAYER) {
     this->osd_buf.buf.width = w;
     this->osd_buf.buf.height = h;
-    this->osd_buf.buf.buf[plane] = calloc(sizeof(uint32_t), w * h);
+    this->osd_buf.buf.buf[plane] = calloc(sizeof(uint32_t), (size_t)w * h);
   } else {
     LOGMSG("open_argb_overlay() failed: video driver does not support ARGB overlays.\n");
   }
@@ -528,8 +530,10 @@ static void update_title_name(bluray_input_plugin_t *this)
     for (i = 0; i < this->meta_dl->toc_count; i++)
       if (this->meta_dl->toc_entries[i].title_number == (unsigned)this->current_title)
         if (this->meta_dl->toc_entries[i].title_name)
-          if (strlen(this->meta_dl->toc_entries[i].title_name) > 2)
+          if (strlen(this->meta_dl->toc_entries[i].title_name) > 2) {
             strncpy(title_name, this->meta_dl->toc_entries[i].title_name, sizeof(title_name));
+            title_name[sizeof(title_name) - 1] = 0;
+          }
   }
 
   /* title name */
@@ -643,7 +647,7 @@ static void fifos_wait(bluray_input_plugin_t *this)
 
 static void stream_flush(bluray_input_plugin_t *this)
 {
-  if (this->stream_flushed || !this->stream)
+  if (!this || this->stream_flushed || !this->stream)
     return;
 
   lprintf("Stream flush\n");
@@ -663,7 +667,7 @@ static void stream_flush(bluray_input_plugin_t *this)
 
 static void stream_reset(bluray_input_plugin_t *this)
 {
-  if (!this || !this->stream)
+  if (!this || this->stream_reset_done || !this->stream)
     return;
 
   lprintf("Stream reset\n");
@@ -682,6 +686,7 @@ static void stream_reset(bluray_input_plugin_t *this)
   xine_event_send (this->stream, &event);
 
   this->demux_action_req = 1;
+  this->stream_reset_done = 1;
 }
 
 static void wait_secs(bluray_input_plugin_t *this, unsigned seconds)
@@ -780,7 +785,7 @@ static void handle_libbluray_event(bluray_input_plugin_t *this, BD_EVENT ev)
 
       case BD_EVENT_STILL:
         lprintf("BD_EVENT_STILL %d\n", ev.param);
-        int paused = _x_get_fine_speed(this->stream) == XINE_SPEED_PAUSE;
+        unsigned int paused = _x_get_fine_speed(this->stream) == XINE_SPEED_PAUSE;
         if (paused != ev.param) {
           _x_set_fine_speed(this->stream, ev.param ? XINE_SPEED_PAUSE : XINE_SPEED_NORMAL);
         }
@@ -1164,6 +1169,7 @@ static off_t bluray_plugin_read (input_plugin_t *this_gen, void *buf, off_t len)
 
   if (result > 0) {
     this->stream_flushed = 0;
+    this->stream_reset_done = 0;
   }
 
   return result;
@@ -1597,6 +1603,7 @@ static int bluray_plugin_open (input_plugin_t *this_gen)
   bluray_input_plugin_t *this    = (bluray_input_plugin_t *) this_gen;
   int                    title   = -1;
   int                    chapter = 0;
+  int major, minor, micro;
 
   lprintf("bluray_plugin_open '%s'\n",this->mrl);
 
@@ -1610,12 +1617,14 @@ static int bluray_plugin_open (input_plugin_t *this_gen)
   if (!this->disc_root)
     this->disc_root = strdup(this->class->mountpoint);
 
-  /* mount .iso image */
-  if (is_iso_image(this->disc_root)) {
-    _x_message (this->stream, XINE_MSG_GENERAL_WARNING,
-                "Can't play BluRay .iso image",
-                "", NULL);
-    return -1;
+  bd_get_version(&major, &minor, &micro);
+  if (BLURAY_VERSION_CODE(major, minor, micro) < BLURAY_VERSION_CODE(0, 8, 0)) {
+    if (is_iso_image(this->disc_root)) {
+      _x_message (this->stream, XINE_MSG_GENERAL_WARNING,
+                  "Can't play BluRay .iso image. Update libbluray.",
+                  "", NULL);
+      return -1;
+    }
   }
 
   /* open libbluray */
@@ -1725,8 +1734,6 @@ static input_plugin_t *bluray_class_get_instance (input_class_t *cls_gen, xine_s
 {
   bluray_input_plugin_t *this;
 
-  lprintf("bluray_class_get_instance\n");
-
   if (strncasecmp(mrl, "bluray:", 7) && strncasecmp(mrl, "bd:", 3))
     return NULL;
 
