@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project,
+ * Copyright (C) 2000-2017 the xine project,
  *                         Rich Wareham <richwareham@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
@@ -88,8 +88,8 @@
 #  endif
 #else
 #  define DVDNAV_COMPILE
-#  include "dvdnav.h"
-#  include "nav_read.h"
+#  include "libdvdnav/dvdnav.h"
+#  include "libdvdnav/nav_read.h"
 #endif
 
 /* Xine includes */
@@ -366,9 +366,6 @@ static int update_title_display(dvd_input_plugin_t *this) {
   int tt=-1, pr=-1;
   int num_tt = 0;
 
-  if(!this || !(this->stream))
-   return 0;
-
   /* Set title/chapter display */
 
   dvdnav_current_title_info(this->dvdnav, &tt, &pr);
@@ -569,8 +566,8 @@ static void dvd_plugin_free_buffer(buf_element_t *buf) {
   if (this->freeing && !this->mem_stack) {
     /* all buffers returned, we can free the plugin now */
     pthread_mutex_destroy(&this->buf_mutex);
-    free(this->mem);
-    free(this->mrl);
+    _x_freep(&this->mem);
+    _x_freep(&this->mrl);
     free(this);
   }
 }
@@ -1401,14 +1398,16 @@ static int dvd_parse_try_open(dvd_input_plugin_t *this, const char *locator)
     /* we have an alternative dvd_path */
     intended_dvd_device = locator;
     /* do not use the raw device for the alternative */
-    xine_setenv("DVDCSS_RAW_DEVICE", "", 1);
+    /*xine_setenv("DVDCSS_RAW_DEVICE", "", 1);*/
   } else {
     /* use default DVD device */
     dvd_input_class_t *class = (dvd_input_class_t*)this->input_plugin.input_class;
+    /*
     xine_cfg_entry_t raw_device;
     if (xine_config_lookup_entry(this->stream->xine,
 	"media.dvd.raw_device", &raw_device))
       xine_setenv("DVDCSS_RAW_DEVICE", raw_device.str_value, 1);
+    */
     intended_dvd_device = class->dvd_device;
   }
 
@@ -1726,7 +1725,7 @@ static void dvd_class_dispose(input_class_t *this_gen) {
   config->unregister_callback(config, "media.dvd.seek_behaviour");
   config->unregister_callback(config, "media.dvd.play_single_chapter");
 
-  free(this->eject_device);
+  _x_freep(&this->eject_device);
   free(this);
 }
 
@@ -1785,10 +1784,11 @@ static void *init_class (xine_t *xine, void *data) {
 #endif
   {
     /* we have found libdvdcss, enable the specific config options */
-    char *raw_device;
+    /* char *raw_device; */
     static const char *const decrypt_modes[] = { "key", "disc", "title", NULL };
     int mode;
 
+    /*
     raw_device = config->register_filename(config, "media.dvd.raw_device",
 					 RDVD_PATH, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 					 _("raw device set up for DVD access"),
@@ -1803,6 +1803,7 @@ static void *init_class (xine_t *xine, void *data) {
 					   "(man raw) for further information."),
 					 10, NULL, NULL);
     if (raw_device) xine_setenv("DVDCSS_RAW_DEVICE", raw_device, 0);
+    */
 
     mode = config->register_enum(config, "media.dvd.css_decryption_method", 0,
 				 (char **)decrypt_modes, _("CSS decryption method"),
