@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2014 the xine project
  * 2002 Bastien Nocera <hadess@hadess.net>
  *
  * This file is part of totem,
@@ -363,8 +363,10 @@ static void
 		return NULL;
 	}
 
+#if !GLIB_CHECK_VERSION(2,32,0)
 	if (!g_thread_supported ())
 		g_thread_init (NULL);
+#endif
 
 	this = g_new0 (gnomevfs_input_class_t, 1);
 	this->xine = xine;
@@ -379,7 +381,7 @@ static void
 	return (input_class_t *) this;
 }
 
-static input_info_t input_info_gnomevfs = {
+static const input_info_t input_info_gnomevfs = {
 	100                       /* priority */
 };
 
