@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -40,12 +40,6 @@
 
 #include <sys/time.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <base64.h>
-#else
-#  include <libavutil/base64.h>
-#endif
-
 #define LOG_MODULE "input_http"
 #define LOG_VERBOSE
 /*
@@ -207,12 +201,14 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
       /* special-case domain beginning with '=' -> is a host name */
       if (domain[0] == '=' && strcmp(target, domain + 1) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+        free(no_proxy);
 	return 0;
       }
       noprox_len = strlen(domain);
       /* special-case host==domain, avoiding dot checks */
       if (host_len == noprox_len && strcmp(target, domain) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+        free(no_proxy);
 	return 0;
       }
       /* check for host in domain, and require that (if matched) the domain
@@ -223,6 +219,7 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
 	  && (domain[0] == '.' || target[host_len - noprox_len - 1] == '.')
 	  && strcmp(target + host_len - noprox_len, domain) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
+        free(no_proxy);
 	return 0;
       }
       lprintf("host '%s' isn't in no-proxy domain '%s'\n", target, domain);
@@ -238,12 +235,12 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
 static void http_plugin_basicauth (const char *user, const char *password, char** dest) {
   const size_t totlen = strlen(user) + (password ? strlen(password) : 0) + 1;
   const size_t enclen = ((totlen + 2) * 4 ) / 3 + 12;
-  char         tmp[totlen + 1];
+  char         tmp[totlen + 5];
 
   snprintf(tmp, totlen + 1, "%s:%s", user, password ? : "");
 
   *dest = malloc(enclen);
-  av_base64_encode(*dest, enclen, tmp, totlen);
+  xine_base64_encode(tmp, *dest, totlen);
 }
 
 static int http_plugin_read_metainf (http_input_plugin_t *this) {
@@ -287,8 +284,7 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
 
           lprintf ("http_plugin_read_metainf: songtitle: %s\n", songtitle);
 
-          if (this->shoutcast_songtitle)
-            free(this->shoutcast_songtitle);
+          free(this->shoutcast_songtitle);
           this->shoutcast_songtitle = strdup(songtitle);
 
           _x_meta_info_set(this->stream, XINE_META_INFO_TITLE, songtitle);
@@ -604,12 +600,12 @@ static void http_plugin_dispose (input_plugin_t *this_gen ) {
     this->nbc = NULL;
   }
 
-  if (this->mrl) free(this->mrl);
-  if (this->proto) free(this->proto);
-  if (this->host) free(this->host);
-  if (this->user) free(this->user);
-  if (this->password) free(this->password);
-  if (this->uri) free(this->uri);
+  _x_freep (&this->mrl);
+  _x_freep (&this->proto);
+  _x_freep (&this->host);
+  _x_freep (&this->user);
+  _x_freep (&this->password);
+  _x_freep (&this->uri);
   free (this);
 }
 
@@ -1041,7 +1037,7 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
 static void http_class_dispose (input_class_t *this_gen) {
   http_input_class_t  *this = (http_input_class_t *) this_gen;
 
-  free(this->proxyhost_env);
+  _x_freep(&this->proxyhost_env);
 
   free (this);
 }
