@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -200,6 +200,7 @@ static int host_connect_attempt(struct in_addr ia, int port,
   if ((setsockopt(s, SOL_SOCKET, SO_RCVBUF,
 		  &optval, sizeof(optval))) < 0) {
     LOG_MSG(xine, _("setsockopt(SO_RCVBUF): %s.\n"), strerror(errno));
+    close(s);
     return -1;
   }
 
@@ -208,6 +209,7 @@ static int host_connect_attempt(struct in_addr ia, int port,
     if ((setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 		    &saddr.in, sizeof(saddr.in))) < 0) {
       LOG_MSG(xine, _("setsockopt(SO_REUSEADDR): %s.\n"), strerror(errno));
+      close(s);
       return -1;
     }
   }
@@ -215,6 +217,7 @@ static int host_connect_attempt(struct in_addr ia, int port,
   /* datagram socket */
   if (bind(s, &saddr.sa, sizeof(saddr.in))) {
     LOG_MSG(xine, _("bind(): %s.\n"), strerror(errno));
+    close(s);
     return -1;
   }
 
@@ -232,7 +235,8 @@ static int host_connect_attempt(struct in_addr ia, int port,
      */
 
     if (interface != NULL) {
-      strncpy(ifreq.ifr_name, interface, IFNAMSIZ);
+      memset(&ifreq, 0, sizeof(ifreq));
+      strncpy(ifreq.ifr_name, interface, IFNAMSIZ - 1);
       if (ioctl(s, SIOCGIFADDR, &ifreq) < 0) {
 	LOG_MSG(xine, _("Can't find address for iface %s:%s\n"),
 		interface, strerror(errno));
@@ -254,6 +258,7 @@ static int host_connect_attempt(struct in_addr ia, int port,
     if (setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq))) {
       LOG_MSG(xine, _("setsockopt(IP_ADD_MEMBERSHIP) failed (multicast kernel?): %s.\n"),
 	      strerror(errno));
+      close(s);
       return -1;
     }
   }
