@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -1023,7 +1023,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
   return this->mrls;
 }
 
-static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_files) {
+static const char * const * vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_files) {
 
   vcd_input_class_t  *this = (vcd_input_class_t *) this_gen;
   int i, fd;
