@@ -7,7 +7,8 @@
 **  published by the Free Software Foundation.  A copy of this license is
 **  included with this software distribution in the file COPYING.  If you
 **  do not have a copy, you may obtain a copy by writing to the Free
-**  Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+**  Software Foundation, 51 Franklin St, Fifth Floor, Boston, MA
+**  02110-1301, USA.
 **
 **  This software is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -173,8 +174,8 @@ static void DeinterlaceGreedy2Frame_SSE2(uint8_t *output, int outstride,
         */
             "movdqa  (%2), %%xmm1		\n\t" /* xmm1 = T1 */
             "movdqa  (%3), %%xmm0		\n\t" /* xmm0 = T0 */
-            "movdqa  (%q4,%2), %%xmm3		\n\t" /* xmm3 = B1 */
-            "movdqa  (%q4,%3), %%xmm2		\n\t" /* xmm2 = B0 */
+            "movdqa  (%4,%2), %%xmm3		\n\t" /* xmm3 = B1 */
+            "movdqa  (%4,%3), %%xmm2		\n\t" /* xmm2 = B0 */
 
             /* calculate |T1-T0| keep T1 put result in xmm5 */
             "movdqa  %%xmm1, %%xmm5		\n\t"
@@ -191,11 +192,11 @@ static void DeinterlaceGreedy2Frame_SSE2(uint8_t *output, int outstride,
             "pcmpgtb %0, %%xmm5			\n\t"
             "pcmpeqd %%xmm7, %%xmm5		\n\t"
 
-            "prefetcht0  64(%q4,%2)		\n\t"
-            "prefetcht0  64(%q4,%3)		\n\t"
+            "prefetcht0  64(%4,%2)		\n\t"
+            "prefetcht0  64(%4,%3)		\n\t"
           :
           : "m" (GreedyTwoFrameThreshold128),
-            "m" (*Destc), "r" (T1), "r" (T0), "r" (Pitch) );
+            "m" (*Destc), "r" (T1), "r" (T0), "r" ((void*)(intptr_t)Pitch) );
 
           asm volatile (
             /* calculate |B1-B0| keep B1 put result in xmm4 */
