@@ -1,4 +1,7 @@
-static Motif CONV_MOTIF2 = {
+static const union {
+    const char  u8[sizeof(Motif)];
+    const Motif motif;
+} CONV_MOTIF2 = {{
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
@@ -1023,4 +1026,4 @@ static Motif CONV_MOTIF2 = {
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
 	15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15
-	};
+        }};
