@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2008 the xine project
+ * Copyright (C) 2000-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -33,6 +33,7 @@
 void *oscope_init_plugin(xine_t *xine, void *data);
 void *fftscope_init_plugin(xine_t *xine, void *data);
 void *fftgraph_init_plugin(xine_t *xine, void *data);
+void *tdaan_init_plugin(xine_t *xine, void *data);
 
 /*
  * exported plugin catalog entries
@@ -42,12 +43,13 @@ void *fftgraph_init_plugin(xine_t *xine, void *data);
 static const post_info_t oscope_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
 static const post_info_t fftscope_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
 static const post_info_t fftgraph_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
-
+static const post_info_t tdaan_special_info = { XINE_POST_TYPE_AUDIO_VISUALIZATION };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_POST, 10, "oscope", XINE_VERSION_CODE, &oscope_special_info, &oscope_init_plugin },
   { PLUGIN_POST, 10, "fftscope", XINE_VERSION_CODE, &fftscope_special_info, &fftscope_init_plugin },
   { PLUGIN_POST, 10, "fftgraph", XINE_VERSION_CODE, &fftgraph_special_info, &fftgraph_init_plugin },
+  { PLUGIN_POST, 10, "tdaudioanalyzer", XINE_VERSION_CODE, &tdaan_special_info, &tdaan_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
