@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 the xine project
+ * Copyright (C) 2010-2014 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -35,11 +35,156 @@
 #include <xine/spu.h>
 #include <xine/osd.h>
 
-#define MAX_REGIONS 7
+#define MAX_REGIONS 16
 
 #define SPU_MAX_WIDTH 1920
 #define SPU_MAX_HEIGHT 1080
 
+/* sparse_array - handle large arrays efficiently when only a few entries are used */
+
+typedef struct {
+  uint32_t key, value;
+} sparse_array_entry_t;
+
+typedef struct {
+  uint32_t sorted_entries, used_entries, max_entries;
+  sparse_array_entry_t *entries;
+} sparse_array_t;
+
+static void sparse_array_new (sparse_array_t *sa) {
+  sa->sorted_entries =
+  sa->used_entries   =
+  sa->max_entries    = 0;
+  sa->entries        = NULL;
+}
+
+static void sparse_array_delete (sparse_array_t *sa) {
+  sa->sorted_entries =
+  sa->used_entries   =
+  sa->max_entries    = 0;
+  free (sa->entries);
+  sa->entries        = NULL;
+}
+
+static int _sparse_array_find (sparse_array_t *sa, uint32_t key, uint32_t *pos) {
+  uint32_t b = 0, m, e = sa->sorted_entries, l = e, k;
+  while (1) {
+    m = (b + e) >> 1;
+    if (m == l)
+      break;
+    l = m;
+    k = sa->entries[m].key;
+    if (k > key)
+      e = m;
+    else if (k < key)
+      b = m;
+    else {
+      *pos = m;
+      return 1;
+    }
+  }
+  *pos = e;
+  return 0;
+}
+
+static void _sparse_array_sort (sparse_array_t *sa) {
+  uint32_t left = sa->max_entries - sa->used_entries;
+  /* move unsorted part to end of buf */
+  uint32_t i = left + sa->sorted_entries;
+  memmove (sa->entries + i, sa->entries + sa->sorted_entries,
+    (sa->used_entries - sa->sorted_entries) * sizeof (sparse_array_entry_t));
+  /* iterate it */
+  while (i < sa->max_entries) {
+    uint32_t j, pos, startkey, stopkey, lastkey;
+    startkey = sa->entries[i].key;
+    if (_sparse_array_find (sa, startkey, &pos)) {
+      /* eliminate duplicate */
+      sa->entries[pos].value = sa->entries[i].value;
+      i++;
+      continue;
+    }
+    /* find sorted range */
+    stopkey = pos < sa->sorted_entries ? sa->entries[pos].key : 0xffffffff;
+    lastkey = startkey;
+    for (j = i + 1; j < sa->max_entries; j++) {
+      uint32_t thiskey = sa->entries[j].key;
+      if ((thiskey <= lastkey) || (thiskey >= stopkey))
+        break;
+      lastkey = thiskey;
+    }
+    j -= i;
+    if (j > left)
+      j = left;
+    /* insert it */
+    if (pos < sa->sorted_entries)
+      memmove (sa->entries + pos + j, sa->entries + pos,
+        (sa->sorted_entries - pos) * sizeof (sparse_array_entry_t));
+    memcpy (sa->entries + pos, sa->entries + i, j * sizeof (sparse_array_entry_t));
+    sa->sorted_entries += j;
+    i += j;
+  }
+  sa->used_entries = sa->sorted_entries;
+  lprintf ("sparse_array_sort: %u entries\n", (unsigned int)sa->used_entries);
+}
+
+static int sparse_array_set (sparse_array_t *sa, uint32_t key, uint32_t value) {
+  /* give some room for later sorting too */
+  if (!sa->entries || (sa->used_entries + 8 >= sa->max_entries)) {
+    uint32_t n = sa->max_entries + 128;
+    sparse_array_entry_t *se = realloc (sa->entries, n * sizeof (sparse_array_entry_t));
+    if (!se)
+      return 0;
+    sa->max_entries = n;
+    sa->entries = se;
+  }
+  sa->entries[sa->used_entries].key = key;
+  sa->entries[sa->used_entries++].value = value;
+  return 1;
+}
+
+static int sparse_array_get (sparse_array_t *sa, uint32_t key, uint32_t *value) {
+  uint32_t pos;
+  if (sa->sorted_entries != sa->used_entries)
+    _sparse_array_sort (sa);
+  if (!_sparse_array_find (sa, key, &pos))
+    return 0;
+  *value = sa->entries[pos].value;
+  return 1;
+}
+
+static void sparse_array_unset (sparse_array_t *sa, uint32_t key, uint32_t mask) {
+  sparse_array_entry_t *here = sa->entries, *p = NULL, *q = sa->entries;
+  uint32_t i, n = 0;
+  if (sa->sorted_entries != sa->used_entries)
+    _sparse_array_sort (sa);
+  key &= mask;
+  for (i = sa->used_entries; i > 0; i--) {
+    if ((here->key & mask) == key) {
+      if (p) {
+        n = here - p;
+        if (n && (p != q))
+          memmove (q, p, n * sizeof (sparse_array_entry_t));
+        p = NULL;
+        q += n;
+      }
+    } else {
+      if (!p)
+        p = here;
+    }
+    here++;
+  }
+  if (p) {
+    n = here - p;
+    if (n && (p != q))
+      memmove (q, p, n * sizeof (sparse_array_entry_t));
+    q += n;
+  }
+  sa->sorted_entries =
+  sa->used_entries   = q - sa->entries;
+}
+
+/* ! sparse_array */
+
 typedef struct {
   int			x, y;
   unsigned char	is_visible;
@@ -61,11 +206,15 @@ typedef struct {
   int			CLUT_id;
   int			objects_start;
   int			objects_end;
-  unsigned int		object_pos[65536];
   unsigned char	*img;
   osd_object_t          *osd;
 } region_t;
 
+typedef union {
+  clut_t   c;
+  uint32_t u32;
+} clut_union_t;
+
 typedef struct {
 /* dvbsub stuff */
   int			x;
@@ -77,13 +226,15 @@ typedef struct {
   int			i_bits;
   int			in_scanline;
   int			compat_depth;
+  int                   max_regions;
   page_t		page;
   region_t		regions[MAX_REGIONS];
-  clut_t		colours[MAX_REGIONS*256];
+  clut_union_t		colours[MAX_REGIONS*256];
   unsigned char		trans[MAX_REGIONS*256];
   struct {
     unsigned char	  lut24[4], lut28[4], lut48[16];
   }			lut[MAX_REGIONS];
+  sparse_array_t	object_pos;
 } dvbsub_func_t;
 
 typedef struct		dvb_spu_class_s {
@@ -471,12 +622,12 @@ static void recalculate_trans (dvb_spu_decoder_t *this)
   int i;
 
   _x_spu_get_opacity (this->stream->xine, &opacity);
-  for (i = 0; i < MAX_REGIONS * 256; ++i) {
+  for (i = 0; i < dvbsub->max_regions * 256; ++i) {
     /* ETSI-300-743 says "full transparency if Y == 0". */
-    if (dvbsub->colours[i].y == 0)
+    if (dvbsub->colours[i].c.y == 0)
       dvbsub->trans[i] = 0;
     else {
-      int v = _x_spu_calculate_opacity (&dvbsub->colours[i], dvbsub->colours[i].foo, &opacity);
+      int v = _x_spu_calculate_opacity (&dvbsub->colours[i].c, dvbsub->colours[i].c.foo, &opacity);
       dvbsub->trans[i] = v * 14 / 255 + 1;
     }
   }
@@ -491,22 +642,22 @@ static void set_clut(dvb_spu_decoder_t *this,int CLUT_id,int CLUT_entry_id,int Y
     return;
   }
 
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].y=Y_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].cr=Cr_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].cb=Cb_value;
-  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].foo = T_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.y   = Y_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.cr  = Cr_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.cb  = Cb_value;
+  dvbsub->colours[(CLUT_id*256)+CLUT_entry_id].c.foo = T_value;
 }
 
 static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
-  int page_id,
+  int /*page_id,*/
       segment_length,
-      CLUT_id,
-      CLUT_version_number;
+      CLUT_id/*,
+      CLUT_version_number*/;
 
   int CLUT_entry_id,
-      CLUT_flag_8_bit,
-      CLUT_flag_4_bit,
-      CLUT_flag_2_bit,
+      /*CLUT_flag_8_bit,*/
+      /*CLUT_flag_4_bit,*/
+      /*CLUT_flag_2_bit,*/
       full_range_flag,
       Y_value,
       Cr_value,
@@ -516,20 +667,21 @@ static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
 
   int j;
 
-  page_id=(dvbsub->buf[dvbsub->i]<<8)|dvbsub->buf[dvbsub->i+1]; dvbsub->i+=2;
+  /*page_id=(dvbsub->buf[dvbsub->i]<<8)|dvbsub->buf[dvbsub->i+1];*/ dvbsub->i+=2;
   segment_length=(dvbsub->buf[dvbsub->i]<<8)|dvbsub->buf[dvbsub->i+1]; dvbsub->i+=2;
   j=dvbsub->i+segment_length;
 
   CLUT_id=dvbsub->buf[dvbsub->i++];
-  CLUT_version_number=(dvbsub->buf[dvbsub->i]&0xf0)>>4;
+  lprintf ("process_CLUT_definition_segment: CLUT_id %d\n", CLUT_id);
+  /*CLUT_version_number=(dvbsub->buf[dvbsub->i]&0xf0)>>4;*/
   dvbsub->i++;
 
   while (dvbsub->i < j) {
     CLUT_entry_id=dvbsub->buf[dvbsub->i++];
 
-    CLUT_flag_2_bit=(dvbsub->buf[dvbsub->i]&0x80)>>7;
-    CLUT_flag_4_bit=(dvbsub->buf[dvbsub->i]&0x40)>>6;
-    CLUT_flag_8_bit=(dvbsub->buf[dvbsub->i]&0x20)>>5;
+    /*CLUT_flag_2_bit=(dvbsub->buf[dvbsub->i]&0x80)>>7;*/
+    /*CLUT_flag_4_bit=(dvbsub->buf[dvbsub->i]&0x40)>>6;*/
+    /*CLUT_flag_8_bit=(dvbsub->buf[dvbsub->i]&0x20)>>5;*/
     full_range_flag=dvbsub->buf[dvbsub->i]&1;
     dvbsub->i++;
 
@@ -549,7 +701,7 @@ static void process_CLUT_definition_segment(dvb_spu_decoder_t *this) {
   }
 }
 
-static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o, int ofs, int n)
+static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o, unsigned int pos, int ofs, int n)
 {
   int data_type;
   int j;
@@ -558,8 +710,8 @@ static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o
 
   j = dvbsub->i + n;
 
-  dvbsub->x = (dvbsub->regions[r].object_pos[o]) >> 16;
-  dvbsub->y = ((dvbsub->regions[r].object_pos[o]) & 0xffff) + ofs;
+  dvbsub->x = pos >> 16;
+  dvbsub->y = (pos & 0xffff) + ofs;
   while (dvbsub->i < j) {
     data_type = dvbsub->buf[dvbsub->i++];
 
@@ -593,7 +745,7 @@ static void process_pixel_data_sub_block (dvb_spu_decoder_t * this, int r, int o
       break;
     case 0xf0:
       dvbsub->in_scanline = 0;
-      dvbsub->x = (dvbsub->regions[r].object_pos[o]) >> 16;
+      dvbsub->x = pos >> 16;
       dvbsub->y += 2;
       break;
     default:
@@ -629,7 +781,7 @@ static void process_page_composition_segment (dvb_spu_decoder_t * this)
   dvbsub->i++;
 
   int r;
-  for (r=0; r<MAX_REGIONS; r++) { /* reset */
+  for (r=0; r<dvbsub->max_regions; r++) { /* reset */
     dvbsub->page.regions[r].is_visible = 0;
   }
 
@@ -640,6 +792,12 @@ static void process_page_composition_segment (dvb_spu_decoder_t * this)
     dvbsub->i += 2;
     region_y = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
+    lprintf ("process_page_composition_segment: page_id %d, region_id %d, x %d, y %d\n",
+      dvbsub->page.page_id, region_id, region_x, region_y);
+    if (region_id >= MAX_REGIONS)
+      continue;
+    if (region_id >= dvbsub->max_regions)
+      dvbsub->max_regions = region_id + 1;
 
     dvbsub->page.regions[region_id].x = region_x;
     dvbsub->page.regions[region_id].y = region_y;
@@ -653,10 +811,10 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
   int segment_length,
     region_id,
     region_version_number,
-    region_fill_flag, region_width, region_height, region_level_of_compatibility, region_depth, CLUT_id, region_8_bit_pixel_code, region_4_bit_pixel_code, region_2_bit_pixel_code;
-  int object_id, object_type, object_provider_flag, object_x, object_y, foreground_pixel_code, background_pixel_code;
+    region_fill_flag, region_width, region_height, region_level_of_compatibility, region_depth, CLUT_id,
+    /*region_8_bit_pixel_code,*/ region_4_bit_pixel_code /*, region_2_bit_pixel_code*/;
+  int object_id, object_type, /*object_provider_flag,*/ object_x, object_y /*, foreground_pixel_code, background_pixel_code*/;
   int j;
-  int o;
   dvbsub_func_t *dvbsub = this->dvbsub;
 
   dvbsub->page.page_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
@@ -678,14 +836,18 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
   dvbsub->compat_depth = region_level_of_compatibility << 3 | region_depth;
   dvbsub->i++;
   CLUT_id = dvbsub->buf[dvbsub->i++];
-  region_8_bit_pixel_code = dvbsub->buf[dvbsub->i++];
+  /*region_8_bit_pixel_code = dvbsub->buf[*/dvbsub->i++/*]*/;
   region_4_bit_pixel_code = (dvbsub->buf[dvbsub->i] & 0xf0) >> 4;
-  region_2_bit_pixel_code = (dvbsub->buf[dvbsub->i] & 0x0c) >> 2;
+  /*region_2_bit_pixel_code = (dvbsub->buf[dvbsub->i] & 0x0c) >> 2;*/
   dvbsub->i++;
+  lprintf ("process_region_composition_segment: region_id %d (%d x %d)\n", region_id, region_width, region_height);
 
   if(region_id>=MAX_REGIONS)
     return;
 
+  if (region_id >= dvbsub->max_regions)
+    dvbsub->max_regions = region_id + 1;
+
   if ( dvbsub->regions[region_id].version_number == region_version_number )
     return;
 
@@ -699,25 +861,23 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
   dvbsub->regions[region_id].objects_start = dvbsub->i;
   dvbsub->regions[region_id].objects_end = j;
 
-  for (o = 0; o < 65536; o++) {
-    dvbsub->regions[region_id].object_pos[o] = 0xffffffff;
-  }
+  sparse_array_unset (&dvbsub->object_pos, region_id << 16, 0xff0000);
 
   while (dvbsub->i < j) {
     object_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
     object_type = (dvbsub->buf[dvbsub->i] & 0xc0) >> 6;
-    object_provider_flag = (dvbsub->buf[dvbsub->i] & 0x30) >> 4;
+    /*object_provider_flag = (dvbsub->buf[dvbsub->i] & 0x30) >> 4;*/
     object_x = ((dvbsub->buf[dvbsub->i] & 0x0f) << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
     object_y = ((dvbsub->buf[dvbsub->i] & 0x0f) << 8) | dvbsub->buf[dvbsub->i + 1];
     dvbsub->i += 2;
 
-    dvbsub->regions[region_id].object_pos[object_id] = (object_x << 16) | object_y;
+    sparse_array_set (&dvbsub->object_pos, (region_id << 16) | object_id, (object_x << 16) | object_y);
 
     if ((object_type == 0x01) || (object_type == 0x02)) {
-      foreground_pixel_code = dvbsub->buf[dvbsub->i++];
-      background_pixel_code = dvbsub->buf[dvbsub->i++];
+      /*foreground_pixel_code = dvbsub->buf[*/dvbsub->i++/*]*/;
+      /*background_pixel_code = dvbsub->buf[*/dvbsub->i++/*]*/;
     }
   }
 
@@ -725,7 +885,7 @@ static void process_region_composition_segment (dvb_spu_decoder_t * this)
 
 static void process_object_data_segment (dvb_spu_decoder_t * this)
 {
-  int segment_length, object_id, object_version_number, object_coding_method, non_modifying_colour_flag;
+  int /*segment_length,*/ object_id/*, object_version_number*/, object_coding_method/*, non_modifying_colour_flag*/;
 
   int top_field_data_block_length, bottom_field_data_block_length;
 
@@ -736,23 +896,24 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 
   dvbsub->page.page_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
   dvbsub->i += 2;
-  segment_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
+  /*segment_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];*/
   dvbsub->i += 2;
 
   object_id = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
   dvbsub->i += 2;
   dvbsub->curr_obj = object_id;
-  object_version_number = (dvbsub->buf[dvbsub->i] & 0xf0) >> 4;
+  /*object_version_number = (dvbsub->buf[dvbsub->i] & 0xf0) >> 4;*/
   object_coding_method = (dvbsub->buf[dvbsub->i] & 0x0c) >> 2;
-  non_modifying_colour_flag = (dvbsub->buf[dvbsub->i] & 0x02) >> 1;
+  /*non_modifying_colour_flag = (dvbsub->buf[dvbsub->i] & 0x02) >> 1;*/
   dvbsub->i++;
 
   old_i = dvbsub->i;
-  for (r = 0; r < MAX_REGIONS; r++) {
+  for (r = 0; r < dvbsub->max_regions; r++) {
 
     /* If this object is in this region... */
     if (dvbsub->regions[r].img) {
-      if (dvbsub->regions[r].object_pos[object_id] != 0xffffffff) {
+      uint32_t pos;
+      if (sparse_array_get (&dvbsub->object_pos, (r << 16) | object_id, &pos)) {
 	dvbsub->i = old_i;
 	if (object_coding_method == 0) {
 	  top_field_data_block_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
@@ -760,7 +921,7 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 	  bottom_field_data_block_length = (dvbsub->buf[dvbsub->i] << 8) | dvbsub->buf[dvbsub->i + 1];
 	  dvbsub->i += 2;
 
-	  process_pixel_data_sub_block (this, r, object_id, 0, top_field_data_block_length);
+	  process_pixel_data_sub_block (this, r, object_id, pos, 0, top_field_data_block_length);
 
 	  if (bottom_field_data_block_length == 0)
 	  {
@@ -769,7 +930,7 @@ static void process_object_data_segment (dvb_spu_decoder_t * this)
 	    dvbsub->i =  old_i + 4;
 	  }
 
-	  process_pixel_data_sub_block (this, r, object_id, 1, bottom_field_data_block_length);
+	  process_pixel_data_sub_block (this, r, object_id, pos, 1, bottom_field_data_block_length);
 	}
       }
     }
@@ -825,7 +986,7 @@ static void* dvbsub_timer_func(void *this_gen)
          Hide the OSD, then wait until we're signalled. */
       if(this && this->stream && this->stream->osd_renderer)
       {
-	for ( i=0; i<MAX_REGIONS; i++ ) {
+	for ( i=0; i<this->dvbsub->max_regions; i++ ) {
 	  if ( this->dvbsub->regions[i].osd ) {
 	    this->stream->osd_renderer->hide( this->dvbsub->regions[i].osd, 0 );
 	    lprintf("thread hiding = %d\n",i);
@@ -866,14 +1027,14 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 
   /* render all regions onto the page */
 
-  for ( r=0; r<MAX_REGIONS; r++ ) {
+  for ( r=0; r<this->dvbsub->max_regions; r++ ) {
     if ( this->dvbsub->page.regions[r].is_visible )
       display++;
   }
   if ( !display )
     return;
 
-  for (r = 0; r < MAX_REGIONS; r++) {
+  for (r = 0; r < this->dvbsub->max_regions; r++) {
     if (this->dvbsub->regions[r].img) {
       if (this->dvbsub->page.regions[r].is_visible && !this->dvbsub->regions[r].empty) {
         update_osd( this, r );
@@ -890,7 +1051,11 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 	  reg = this->dvbsub->regions[r].img;
 	  reg_width = this->dvbsub->regions[r].width;
 	}
-	this->stream->osd_renderer->set_palette( this->dvbsub->regions[r].osd, (uint32_t*)(&this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256]), &this->dvbsub->trans[this->dvbsub->regions[r].CLUT_id*256]);
+        /* All DVB subs I have seen so far use same color matrix as main video. */
+        _X_SET_CLUT_CM (&this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256].u32, 4);
+	this->stream->osd_renderer->set_palette( this->dvbsub->regions[r].osd,
+                                                 &this->dvbsub->colours[this->dvbsub->regions[r].CLUT_id*256].u32,
+                                                 &this->dvbsub->trans[this->dvbsub->regions[r].CLUT_id*256]);
 	this->stream->osd_renderer->draw_bitmap( this->dvbsub->regions[r].osd, reg, 0, 0, reg_width, this->dvbsub->regions[r].height, NULL );
       }
     }
@@ -898,7 +1063,7 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 
   pthread_mutex_lock(&this->dvbsub_osd_mutex);
   lprintf("this->vpts=%"PRId64"\n",this->vpts);
-  for ( r=0; r<MAX_REGIONS; r++ ) {
+  for ( r=0; r<this->dvbsub->max_regions; r++ ) {
     lprintf("region=%d, visible=%d, osd=%d, empty=%d\n", r, this->dvbsub->page.regions[r].is_visible, this->dvbsub->regions[r].osd?1:0, this->dvbsub->regions[r].empty );
     if ( this->dvbsub->page.regions[r].is_visible && this->dvbsub->regions[r].osd && !this->dvbsub->regions[r].empty ) {
       this->stream->osd_renderer->set_position( this->dvbsub->regions[r].osd, this->dvbsub->page.regions[r].x, this->dvbsub->page.regions[r].y );
@@ -922,11 +1087,11 @@ static void draw_subtitles (dvb_spu_decoder_t * this)
 static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
 {
   dvb_spu_decoder_t *this = (dvb_spu_decoder_t *) this_gen;
-      int new_i;
-      int data_identifier, subtitle_stream_id;
-      int segment_length, segment_type;
-      int PES_packet_length;
-      int i;
+  int new_i;
+  /*int data_identifier, subtitle_stream_id;*/
+  int segment_length, segment_type;
+  int PES_packet_length;
+  int i;
 
   if((buf->type & 0xffff0000)!=BUF_SPU_DVB)
     return;
@@ -936,7 +1101,7 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
       if (buf->decoder_info[2] == 0) {
         /* Hide the osd - note that if the timeout thread times out, it'll rehide, which is harmless */
         pthread_mutex_lock(&this->dvbsub_osd_mutex);
-	for ( i=0; i<MAX_REGIONS; i++ ) {
+	for ( i=0; i<this->dvbsub->max_regions; i++ ) {
 	  if ( this->dvbsub->regions[i].osd )
 	    this->stream->osd_renderer->hide( this->dvbsub->regions[i].osd, 0 );
 	}
@@ -996,8 +1161,8 @@ static void spudec_decode_data (spu_decoder_t * this_gen, buf_element_t * buf)
 
       this->dvbsub->i = 0;
 
-      data_identifier = this->dvbsub->buf[this->dvbsub->i++];
-      subtitle_stream_id = this->dvbsub->buf[this->dvbsub->i++];
+      /*data_identifier = this->dvbsub->buf[*/this->dvbsub->i++/*]*/;
+      /*subtitle_stream_id = this->dvbsub->buf[*/this->dvbsub->i++/*]*/;
 
       while (this->dvbsub->i <= (PES_packet_length)) {
 	/* SUBTITLING SEGMENT */
@@ -1093,8 +1258,10 @@ static void spudec_dispose (spu_decoder_t * this_gen)
   if (this->pes_pkt)
     free (this->pes_pkt);
 
-  if (this->dvbsub)
+  if (this->dvbsub) {
+    sparse_array_delete (&this->dvbsub->object_pos);
     free (this->dvbsub);
+  }
 
   free (this);
 }
@@ -1164,9 +1331,11 @@ static spu_decoder_t *dvb_spu_class_open_plugin (spu_decoder_class_t * class_gen
     t = _x_spu_calculate_opacity (&black, 0, &opacity);
 
     for (i = 0; i < MAX_REGIONS * 256; i++)
-      this->dvbsub->colours[i].foo = t;
+      this->dvbsub->colours[i].c.foo = t;
   }
 
+  sparse_array_new (&this->dvbsub->object_pos);
+
   pthread_mutex_init(&this->dvbsub_osd_mutex, NULL);
   pthread_cond_init(&this->dvbsub_restart_timeout, NULL);
   this->dvbsub_hide_timeout.tv_nsec = 0;
