@@ -1,5 +1,5 @@
 /* kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
- * Copyright (C) 2008-2013 the xine project
+ * Copyright (C) 2008-2016 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -23,6 +23,7 @@
  */
 
 #include "alterh264_decode.h"
+#include "group_vdpau.h"
 
 
 #define MAX_DPB_SIZE 16
@@ -1802,7 +1803,7 @@ decode_render (vdpau_h264_alter_decoder_t * vd, int bad_frame)
   }
   st =
     accel->vdp_decoder_render (vd->decoder, accel->surface,
-			       (VdpPictureInfo *) & info,
+			       CAST_VdpPictureInfo_PTR &info,
 			       seq->slices_count * 2, vbits);
   if (st != VDP_STATUS_OK)
     lprintf ("**********************DECODING failed! - surface = %d - %s\n",
@@ -2243,7 +2244,7 @@ vdpau_h264_alter_flush (video_decoder_t * this_gen)
 {
   vdpau_h264_alter_decoder_t *this = (vdpau_h264_alter_decoder_t *) this_gen;
 
-  printf ("vdpau_h264_alter_flush\n");
+  lprintf ("vdpau_h264_alter_flush\n");
   dpb_draw_frames (this, MAX_POC, DPB_DRAW_REFS);
 }
 
@@ -2268,7 +2269,7 @@ static void
 vdpau_h264_alter_discontinuity (video_decoder_t * this_gen)
 {
   vdpau_h264_alter_decoder_t *this = (vdpau_h264_alter_decoder_t *) this_gen;
-  printf ("vdpau_h264_alter_discontinuity\n");
+  lprintf ("vdpau_h264_alter_discontinuity\n");
 
   dpb_clear_all_pts (&this->sequence);
   this->sequence.reset = VO_NEW_SEQUENCE_FLAG;
@@ -2399,8 +2400,8 @@ open_plugin (video_decoder_class_t * class_gen, xine_stream_t * stream)
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-static void *
-init_plugin (xine_t * xine, void *data)
+void *
+h264_alter_init_plugin (xine_t * xine, void *data)
 {
 
   vdpau_h264_alter_class_t *this;
@@ -2420,38 +2421,3 @@ init_plugin (xine_t * xine, void *data)
   return this;
 }
 
-
-/*
- * This is a list of all of the internal xine video buffer types that
- * this decoder is able to handle. Check src/xine-engine/buffer.h for a
- * list of valid buffer types (and add a new one if the one you need does
- * not exist). Terminate the list with a 0.
- */
-static const uint32_t video_types[] = {
-  BUF_VIDEO_H264,
-  0
-};
-
-
-/*
- * This data structure combines the list of supported xine buffer types and
- * the priority that the plugin should be given with respect to other
- * plugins that handle the same buffer type. A plugin with priority (n+1)
- * will be used instead of a plugin with priority (n).
- */
-static const decoder_info_t dec_info_video = {
-  video_types,			/* supported types */
-  9				/* priority        */
-};
-
-
-/*
- * The plugin catalog entry. This is the only information that this plugin
- * will export to the public.
- */
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* { type, API, "name", version, special_info, init_function } */
-  {PLUGIN_VIDEO_DECODER, 19, "vdpau_h264_alter", XINE_VERSION_CODE,
-   &dec_info_video, init_plugin},
-  {PLUGIN_NONE, 0, "", 0, NULL, NULL}
-};
