@@ -1,6 +1,6 @@
 /* kate: tab-indent on; indent-width 4; mixedindent off; indent-mode cstyle; remove-trailing-space on; */
 /*
- * Copyright (C) 2008 the xine project
+ * Copyright (C) 2008-2013 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -305,7 +305,6 @@ typedef struct
   uint16_t prevFrameNumOffset;
   uint8_t prevMMC5;
 
-  VdpColorStandard color_standard;
   int chroma;
   int top_field_first;
   VdpDecoderProfile profile;
@@ -330,6 +329,8 @@ typedef struct
   uint8_t flag_header;
   uint32_t frame_header_size;
 
+  int color_matrix;
+
 } sequence_t;
 
 
