@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2008-2016 the xine project
  * Copyright (C) 2008 Julian Scheel
  *
  * kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
@@ -44,6 +45,11 @@
 #include "h264_parser.h"
 #include "dpb.h"
 #include "cpb.h"
+#include "dpb.c"
+#include "cpb.c"
+#include "nal.c"
+#include "h264_parser.c"
+#include "group_vdpau.h"
 
 //#define DEBUG_H264
 
@@ -591,7 +597,7 @@ static int vdpau_decoder_render(video_decoder_t *this_gen, VdpBitstreamBuffer *v
   /*xprintf(this->xine, XINE_VERBOSITY_DEBUG,
       "Decode: NUM: %d, REF: %d, BYTES: %d, PTS: %lld\n", pic.frame_num, pic.is_reference, vdp_buffer->bitstream_bytes, this->completed_pic->pts);*/
   VdpStatus status = this->vdpau_accel->vdp_decoder_render(this->decoder,
-      surface, (VdpPictureInfo*)&pic, 1, vdp_buffer);
+      surface, CAST_VdpPictureInfo_PTR &pic, 1, vdp_buffer);
 
   /* free the image data */
   if(((uint8_t*)vdp_buffer->bitstream) != NULL) {
@@ -741,7 +747,7 @@ static void vdpau_h264_decode_data (video_decoder_t *this_gen,
       this->wait_for_frame_start = 0;
       len += parse_frame(this->nal_parser, buf->content + len, buf->size - len,
           buf->pts,
-          (uint8_t**)&vdp_buffer.bitstream, &vdp_buffer.bitstream_bytes, &this->completed_pic);
+          &vdp_buffer.bitstream, &vdp_buffer.bitstream_bytes, &this->completed_pic);
 
       if(this->decoder == VDP_INVALID_HANDLE &&
           this->completed_pic &&
@@ -954,7 +960,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-static void *init_plugin (xine_t *xine, void *data) {
+void *h264_init_plugin (xine_t *xine, void *data) {
 
   vdpau_h264_class_t *this;
 
@@ -970,35 +976,3 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * This is a list of all of the internal xine video buffer types that
- * this decoder is able to handle. Check src/xine-engine/buffer.h for a
- * list of valid buffer types (and add a new one if the one you need does
- * not exist). Terminate the list with a 0.
- */
-static const uint32_t video_types[] = {
-  /* BUF_VIDEO_FOOVIDEO, */
-  BUF_VIDEO_H264,
-  0
-};
-
-/*
- * This data structure combines the list of supported xine buffer types and
- * the priority that the plugin should be given with respect to other
- * plugins that handle the same buffer type. A plugin with priority (n+1)
- * will be used instead of a plugin with priority (n).
- */
-static const decoder_info_t dec_info_video = {
-  video_types,         /* supported types */
-  7                    /* priority        */
-};
-
-/*
- * The plugin catalog entry. This is the only information that this plugin
- * will export to the public.
- */
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* { type, API, "name", version, special_info, init_function } */
-  { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 19, "vdpau_h264", XINE_VERSION_CODE, &dec_info_video, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
