@@ -1,7 +1,7 @@
 /*
  * kate: space-indent on; indent-width 2; mixedindent off; indent-mode cstyle; remove-trailing-space on;
  *
- * Copyright (C) 2010-2013 the xine project
+ * Copyright (C) 2010-2016 the xine project
  * Copyright (C) 2010 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -44,6 +44,7 @@
 #include <xine/xineutils.h>
 #include "accel_vdpau.h"
 #include "bits_reader.h"
+#include "group_vdpau.h"
 
 #include <vdpau/vdpau.h>
 
@@ -847,7 +848,7 @@ static void decode_render( vdpau_mpeg4_decoder_t *vd, vdpau_accel_t *accel, uint
   vbit.struct_version = VDP_BITSTREAM_BUFFER_VERSION;
   vbit.bitstream = buf;
   vbit.bitstream_bytes = len;
-  st = accel->vdp_decoder_render( vd->decoder, accel->surface, (VdpPictureInfo*)&pic->vdp_infos, 1, &vbit );
+  st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
   if ( st!=VDP_STATUS_OK )
     fprintf(stderr, "vdpau_mpeg4: decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
   else {
@@ -1140,7 +1141,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-static void *init_plugin (xine_t *xine, void *data) {
+void *mpeg4_init_plugin (xine_t *xine, void *data) {
 
   vdpau_mpeg4_class_t *this;
 
@@ -1155,38 +1156,3 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   return this;
 }
-
-/*
- * This is a list of all of the internal xine video buffer types that
- * this decoder is able to handle. Check src/xine-engine/buffer.h for a
- * list of valid buffer types (and add a new one if the one you need does
- * not exist). Terminate the list with a 0.
- */
-static const uint32_t video_types[] = {
-  BUF_VIDEO_MPEG4,
-  BUF_VIDEO_XVID,
-  BUF_VIDEO_DIVX5,
-  BUF_VIDEO_3IVX,
-  0
-};
-
-/*
- * This data structure combines the list of supported xine buffer types and
- * the priority that the plugin should be given with respect to other
- * plugins that handle the same buffer type. A plugin with priority (n+1)
- * will be used instead of a plugin with priority (n).
- */
-static const decoder_info_t dec_info_video = {
-  video_types,         /* supported types */
-  0                    /* priority        */
-};
-
-/*
- * The plugin catalog entry. This is the only information that this plugin
- * will export to the public.
- */
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* { type, API, "name", version, special_info, init_function } */
-  { PLUGIN_VIDEO_DECODER, 19, "vdpau_mpeg4", XINE_VERSION_CODE, &dec_info_video, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
