@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2013 the xine project
+ * Copyright (C) 2008-2016 the xine project
  * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
  *
  * This file is part of xine, a free video player.
@@ -41,6 +41,7 @@
 #include <xine/xineutils.h>
 #include "accel_vdpau.h"
 #include "bits_reader.h"
+#include "group_vdpau.h"
 
 #include <vdpau/vdpau.h>
 
@@ -662,7 +663,7 @@ static void decode_render( vdpau_vc1_decoder_t *vd, vdpau_accel_t *accel, uint8_
   vbit.bitstream_bytes = len;
   if ( pic->field )
     vbit.bitstream_bytes = pic->field;
-  st = accel->vdp_decoder_render( vd->decoder, accel->surface, (VdpPictureInfo*)&pic->vdp_infos, 1, &vbit );
+  st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
   if ( st!=VDP_STATUS_OK )
     fprintf(stderr, "vdpau_vc1: decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
   else {
@@ -704,7 +705,7 @@ static void decode_render( vdpau_vc1_decoder_t *vd, vdpau_accel_t *accel, uint8_
     }
     vbit.bitstream = buf+pic->field+4;
     vbit.bitstream_bytes = len-pic->field-4;
-    st = accel->vdp_decoder_render( vd->decoder, accel->surface, (VdpPictureInfo*)&pic->vdp_infos, 1, &vbit );
+    st = accel->vdp_decoder_render( vd->decoder, accel->surface, CAST_VdpPictureInfo_PTR &pic->vdp_infos, 1, &vbit );
     if ( st!=VDP_STATUS_OK )
       fprintf(stderr, "vdpau_vc1: decoder failed : %d!! %s\n", st, accel->vdp_get_error_string( st ) );
     else {
@@ -1135,7 +1136,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
  * This function allocates a private video decoder class and initializes
  * the class's member functions.
  */
-static void *init_plugin (xine_t *xine, void *data) {
+void *vc1_init_plugin (xine_t *xine, void *data) {
 
   vdpau_vc1_class_t *this;
 
@@ -1151,34 +1152,3 @@ static void *init_plugin (xine_t *xine, void *data) {
   return this;
 }
 
-/*
- * This is a list of all of the internal xine video buffer types that
- * this decoder is able to handle. Check src/xine-engine/buffer.h for a
- * list of valid buffer types (and add a new one if the one you need does
- * not exist). Terminate the list with a 0.
- */
-static const uint32_t video_types[] = {
-  BUF_VIDEO_VC1, BUF_VIDEO_WMV9,
-  0
-};
-
-/*
- * This data structure combines the list of supported xine buffer types and
- * the priority that the plugin should be given with respect to other
- * plugins that handle the same buffer type. A plugin with priority (n+1)
- * will be used instead of a plugin with priority (n).
- */
-static const decoder_info_t dec_info_video = {
-  video_types,         /* supported types */
-  8                    /* priority        */
-};
-
-/*
- * The plugin catalog entry. This is the only information that this plugin
- * will export to the public.
- */
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* { type, API, "name", version, special_info, init_function } */
-  { PLUGIN_VIDEO_DECODER, 19, "vdpau_vc1", XINE_VERSION_CODE, &dec_info_video, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
