@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -45,50 +45,52 @@ typedef struct {
   double               ratio;
   int                  format;
   xine_t               *xine;
-} none_frame_t;
+} vo_none_frame_t;
 
 typedef struct {
   vo_driver_t          vo_driver;
   config_values_t     *config;
   int                  ratio;
   xine_t               *xine;
-} none_driver_t;
+} vo_none_driver_t;
 
 typedef struct {
   video_driver_class_t  driver_class;
   config_values_t      *config;
   xine_t               *xine;
-} none_class_t;
+} vo_none_class_t;
 
 
-static void free_framedata(none_frame_t* frame) {
+static void vo_none_free_framedata(vo_none_frame_t* frame) {
   if(frame->vo_frame.base[0]) {
-    free(frame->vo_frame.base[0]);
+    xine_free_aligned(frame->vo_frame.base[0]);
     frame->vo_frame.base[0] = NULL;
     frame->vo_frame.base[1] = NULL;
     frame->vo_frame.base[2] = NULL;
   }
 }
 
-static void none_frame_dispose(vo_frame_t *vo_frame) {
-  none_frame_t *frame = (none_frame_t *)vo_frame;
-  free_framedata(frame);
+static void vo_none_frame_dispose(vo_frame_t *vo_frame) {
+  vo_none_frame_t *frame = (vo_none_frame_t *)vo_frame;
+  vo_none_free_framedata(frame);
   free (frame);
 }
 
-static void none_frame_field(vo_frame_t *vo_frame, int which_field) {
+static void vo_none_frame_field(vo_frame_t *vo_frame, int which_field) {
   /* do nothing */
 }
 
-static uint32_t none_get_capabilities(vo_driver_t *vo_driver) {
-  return VO_CAP_YV12 | VO_CAP_YUY2;
+static uint32_t vo_none_get_capabilities(vo_driver_t *vo_driver) {
+  /* No, we dont crop. Neither do we interpret color matrix or range. */
+  /* But we also dont ask decoders to convert data just for the trash ;-) */
+  return VO_CAP_YV12 | VO_CAP_YUY2 | VO_CAP_CROP | VO_CAP_COLOR_MATRIX | VO_CAP_FULLRANGE;
 }
 
-static vo_frame_t *none_alloc_frame(vo_driver_t *vo_driver) {
+static vo_frame_t *vo_none_alloc_frame(vo_driver_t *vo_driver) {
   /* none_driver_t *this = (none_driver_t *) vo_driver; */
-  none_frame_t  *frame;
+  vo_none_frame_t  *frame;
 
-  frame = calloc(1, sizeof(none_frame_t));
+  frame = calloc(1, sizeof(vo_none_frame_t));
   if(!frame)
     return NULL;
 
@@ -100,22 +102,22 @@ static vo_frame_t *none_alloc_frame(vo_driver_t *vo_driver) {
 
   frame->vo_frame.proc_slice = NULL;
   frame->vo_frame.proc_frame = NULL;
-  frame->vo_frame.field      = none_frame_field;
-  frame->vo_frame.dispose    = none_frame_dispose;
+  frame->vo_frame.field      = vo_none_frame_field;
+  frame->vo_frame.dispose    = vo_none_frame_dispose;
   frame->vo_frame.driver     = vo_driver;
 
   return (vo_frame_t *)frame;
 }
 
-static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_frame,
+static void vo_none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_frame,
 				     uint32_t width, uint32_t height,
 				     double ratio, int format, int flags) {
-  none_driver_t *this = (none_driver_t *) vo_driver;
-  none_frame_t  *frame = (none_frame_t *) vo_frame;
+  vo_none_driver_t *this = (vo_none_driver_t *) vo_driver;
+  vo_none_frame_t  *frame = (vo_none_frame_t *) vo_frame;
 
   if((frame->width != width) || (frame->height != height) || (frame->format != format)) {
 
-    free_framedata(frame);
+    vo_none_free_framedata(frame);
 
     frame->width  = width;
     frame->height = height;
@@ -134,17 +136,32 @@ static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_fram
 	y_size  = frame->vo_frame.pitches[0] * height;
 	uv_size = frame->vo_frame.pitches[1] * ((height+1)/2);
 
-	frame->vo_frame.base[0] = malloc (y_size + 2*uv_size);
-	frame->vo_frame.base[1] = frame->vo_frame.base[0]+y_size+uv_size;
-	frame->vo_frame.base[2] = frame->vo_frame.base[0]+y_size;
+	frame->vo_frame.base[0] = xine_malloc_aligned (y_size + 2*uv_size);
+        if (frame->vo_frame.base[0]) {
+          frame->vo_frame.base[1] = frame->vo_frame.base[0] + y_size;
+          frame->vo_frame.base[2] = frame->vo_frame.base[0] + y_size + uv_size;
+        } else {
+          frame->vo_frame.base[1] = NULL;
+          frame->vo_frame.base[2] = NULL;
+          xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+            "video_out_none: error. (framedata allocation failed: out of memory)\n");
+          frame->width = 0;
+          frame->vo_frame.width = 0;
+        }
       }
       break;
 
     case XINE_IMGFMT_YUY2:
       frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
-      frame->vo_frame.base[0] = malloc(frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[0] = xine_malloc_aligned(frame->vo_frame.pitches[0] * height);
       frame->vo_frame.base[1] = NULL;
       frame->vo_frame.base[2] = NULL;
+      if (!frame->vo_frame.base[0]) {
+        xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+          "video_out_none: error. (framedata allocation failed: out of memory)\n");
+        frame->width = 0;
+        frame->vo_frame.width = 0;
+      }
       break;
 
     default:
@@ -152,30 +169,20 @@ static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_fram
       break;
 
     }
-
-    if((format == XINE_IMGFMT_YV12
-	&& (frame->vo_frame.base[0] == NULL
-	    || frame->vo_frame.base[1] == NULL
-	    || frame->vo_frame.base[2] == NULL))
-       || (format == XINE_IMGFMT_YUY2 && frame->vo_frame.base[0] == NULL)) {
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
-	       "video_out_none: error. (framedata allocation failed: out of memory)\n");
-      free_framedata(frame);
-    }
   }
 
   frame->ratio = ratio;
 }
 
-static void none_display_frame(vo_driver_t *vo_driver, vo_frame_t *vo_frame) {
-  /* none_driver_t  *driver = (none_driver_t *)vo_driver; */
-  none_frame_t   *frame = (none_frame_t *)vo_frame;
+static void vo_none_display_frame(vo_driver_t *vo_driver, vo_frame_t *vo_frame) {
+  /* vo_none_driver_t  *driver = (vo_none_driver_t *)vo_driver; */
+  vo_none_frame_t   *frame = (vo_none_frame_t *)vo_frame;
 
   frame->vo_frame.free(&frame->vo_frame);
 }
 
-static int none_get_property(vo_driver_t *vo_driver, int property) {
-  none_driver_t  *driver = (none_driver_t *)vo_driver;
+static int vo_none_get_property(vo_driver_t *vo_driver, int property) {
+  vo_none_driver_t  *driver = (vo_none_driver_t *)vo_driver;
 
   switch(property) {
 
@@ -190,8 +197,8 @@ static int none_get_property(vo_driver_t *vo_driver, int property) {
   return 0;
 }
 
-static int none_set_property(vo_driver_t *vo_driver, int property, int value) {
-  none_driver_t  *driver = (none_driver_t *)vo_driver;
+static int vo_none_set_property(vo_driver_t *vo_driver, int property, int value) {
+  vo_none_driver_t  *driver = (vo_none_driver_t *)vo_driver;
 
   switch(property) {
 
@@ -208,14 +215,14 @@ static int none_set_property(vo_driver_t *vo_driver, int property, int value) {
   return value;
 }
 
-static void none_get_property_min_max(vo_driver_t *vo_driver,
+static void vo_none_get_property_min_max(vo_driver_t *vo_driver,
 				      int property, int *min, int *max) {
   *min = 0;
   *max = 0;
 }
 
-static int none_gui_data_exchange(vo_driver_t *vo_driver, int data_type, void *data) {
-/*   none_driver_t     *this = (none_driver_t *) vo_driver; */
+static int vo_none_gui_data_exchange(vo_driver_t *vo_driver, int data_type, void *data) {
+/*   vo_none_driver_t     *this = (vo_none_driver_t *) vo_driver; */
 
   switch (data_type) {
   case XINE_GUI_SEND_COMPLETION_EVENT:
@@ -229,39 +236,39 @@ static int none_gui_data_exchange(vo_driver_t *vo_driver, int data_type, void *d
 
   return 0;
 }
-static void none_dispose(vo_driver_t *vo_driver) {
-  none_driver_t *this = (none_driver_t *) vo_driver;
+static void vo_none_dispose(vo_driver_t *vo_driver) {
+  vo_none_driver_t *this = (vo_none_driver_t *) vo_driver;
 
   free(this);
 }
 
-static int none_redraw_needed(vo_driver_t *vo_driver) {
+static int vo_none_redraw_needed(vo_driver_t *vo_driver) {
   return 0;
 }
 
-static vo_driver_t *open_plugin(video_driver_class_t *driver_class, const void *visual) {
-  none_class_t    *class = (none_class_t *) driver_class;
-  none_driver_t   *driver;
+static vo_driver_t *vo_none_open_plugin(video_driver_class_t *driver_class, const void *visual) {
+  vo_none_class_t    *class = (vo_none_class_t *) driver_class;
+  vo_none_driver_t   *driver;
 
-  driver = calloc(1, sizeof(none_driver_t));
+  driver = calloc(1, sizeof(vo_none_driver_t));
 
   driver->config = class->config;
   driver->xine   = class->xine;
   driver->ratio  = XINE_VO_ASPECT_AUTO;
 
-  driver->vo_driver.get_capabilities     = none_get_capabilities;
-  driver->vo_driver.alloc_frame          = none_alloc_frame ;
-  driver->vo_driver.update_frame_format  = none_update_frame_format;
+  driver->vo_driver.get_capabilities     = vo_none_get_capabilities;
+  driver->vo_driver.alloc_frame          = vo_none_alloc_frame ;
+  driver->vo_driver.update_frame_format  = vo_none_update_frame_format;
   driver->vo_driver.overlay_begin        = NULL;
   driver->vo_driver.overlay_blend        = NULL;
   driver->vo_driver.overlay_end          = NULL;
-  driver->vo_driver.display_frame        = none_display_frame;
-  driver->vo_driver.get_property         = none_get_property;
-  driver->vo_driver.set_property         = none_set_property;
-  driver->vo_driver.get_property_min_max = none_get_property_min_max;
-  driver->vo_driver.gui_data_exchange    = none_gui_data_exchange;
-  driver->vo_driver.dispose              = none_dispose;
-  driver->vo_driver.redraw_needed        = none_redraw_needed;
+  driver->vo_driver.display_frame        = vo_none_display_frame;
+  driver->vo_driver.get_property         = vo_none_get_property;
+  driver->vo_driver.set_property         = vo_none_set_property;
+  driver->vo_driver.get_property_min_max = vo_none_get_property_min_max;
+  driver->vo_driver.gui_data_exchange    = vo_none_gui_data_exchange;
+  driver->vo_driver.dispose              = vo_none_dispose;
+  driver->vo_driver.redraw_needed        = vo_none_redraw_needed;
 
   return &driver->vo_driver;
 }
@@ -269,12 +276,12 @@ static vo_driver_t *open_plugin(video_driver_class_t *driver_class, const void *
 /*
  * Class related functions.
  */
-static void *init_class (xine_t *xine, void *visual) {
-  none_class_t        *this;
+static void *vo_none_init_class (xine_t *xine, void *visual) {
+  vo_none_class_t        *this;
 
-  this = calloc(1, sizeof(none_class_t));
+  this = calloc(1, sizeof(vo_none_class_t));
 
-  this->driver_class.open_plugin     = open_plugin;
+  this->driver_class.open_plugin     = vo_none_open_plugin;
   this->driver_class.identifier      = "none";
   this->driver_class.description     = N_("xine video output plugin which displays nothing");
   this->driver_class.dispose         = default_video_driver_class_dispose;
@@ -290,8 +297,13 @@ static const vo_info_t vo_info_none = {
   XINE_VISUAL_TYPE_NONE     /* Visual type */
 };
 
+#define VO_NONE_CATALOG { PLUGIN_VIDEO_OUT, 22, "none", XINE_VERSION_CODE, &vo_info_none, vo_none_init_class }
+
+#ifndef XINE_MAKE_BUILTINS
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_OUT, 22, "none", XINE_VERSION_CODE, &vo_info_none, init_class },
+  VO_NONE_CATALOG,
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+#endif
+
