@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -47,11 +47,7 @@
 #include <ctype.h>
 #include <pthread.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <mem.h>
-#else
-#  include <libavutil/mem.h>
-#endif
+
 
 /* defines for debugging extensions only */
 /* #define GL_GLEXT_LEGACY */
@@ -1010,7 +1006,7 @@ static int render_setup_fp_yuv (opengl_driver_t *this) {
     return 0;
 
   xprintf (this->xine, XINE_VERBOSITY_LOG,
-    "video_out_open_opengl_fragprog: b %d c %d s %d [%s]\n",
+    "video_out_opengl_fragprog: b %d c %d s %d [%s]\n",
     this->brightness, this->contrast, this->saturation, cm_names[this->cm_fragprog]);
 
   if (this->fprog == (GLuint)-1)
@@ -1361,10 +1357,10 @@ static void opengl_frame_dispose (vo_frame_t *vo_img) {
 
   frame->yuv2rgb->dispose (frame->yuv2rgb);
 
-  av_free (frame->vo_frame.base[0]);
-  av_free (frame->vo_frame.base[1]);
-  av_free (frame->vo_frame.base[2]);
-  av_free (frame->rgb);
+  xine_free_aligned (frame->vo_frame.base[0]);
+  xine_free_aligned (frame->vo_frame.base[1]);
+  xine_free_aligned (frame->vo_frame.base[2]);
+  xine_free_aligned (frame->rgb);
   free (frame);
 }
 
@@ -1412,6 +1408,7 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
   opengl_frame_t   *frame = (opengl_frame_t *) frame_gen;
   int     g_width, g_height;
   double  g_pixel_aspect;
+  int     changed = 0;
 
   /* Check output size to signal render thread output size changes */
   this->sc.dest_size_cb (this->sc.user_data, width, height,
@@ -1431,34 +1428,45 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
   }
 
   /* Check frame size and format and reallocate if necessary */
-  if ((frame->width != width)
-      || (frame->height != height)
-      || (frame->format != format)
-      || (frame->flags  != flags)) {
+  if ((frame->width != width) || (frame->height != height) || (frame->format != format)) {
 /*     lprintf ("updating frame to %d x %d (ratio=%g, format=%08x)\n", width, height, ratio, format); */
 
-    flags &= VO_BOTH_FIELDS;
-
+    /* make sure we dont pull the rug from under the renderer */
     XLockDisplay (this->display);
 
     /* (re-) allocate render space */
-    av_freep(&frame->vo_frame.base[0]);
-    av_freep(&frame->vo_frame.base[1]);
-    av_freep(&frame->vo_frame.base[2]);
-    av_freep(&frame->rgb);
+    xine_freep_aligned(&frame->vo_frame.base[0]);
+    xine_freep_aligned(&frame->vo_frame.base[1]);
+    xine_freep_aligned(&frame->vo_frame.base[2]);
+    xine_freep_aligned(&frame->rgb);
 
     if (format == XINE_IMGFMT_YV12) {
       frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
       frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
       frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
-      frame->vo_frame.base[0] = av_mallocz(frame->vo_frame.pitches[0] * height);
-      frame->vo_frame.base[1] = av_mallocz(frame->vo_frame.pitches[1] * ((height+1)/2));
-      frame->vo_frame.base[2] = av_mallocz(frame->vo_frame.pitches[2] * ((height+1)/2));
+      frame->vo_frame.base[0] = xine_mallocz_aligned(frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[1] = xine_mallocz_aligned(frame->vo_frame.pitches[1] * ((height+1)/2));
+      frame->vo_frame.base[2] = xine_mallocz_aligned(frame->vo_frame.pitches[2] * ((height+1)/2));
     } else {
       frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
-      frame->vo_frame.base[0] = av_mallocz(frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[0] = xine_mallocz_aligned(frame->vo_frame.pitches[0] * height);
     }
-    frame->rgb = av_mallocz(BYTES_PER_PIXEL*width*height);
+    frame->rgb = xine_mallocz_aligned(BYTES_PER_PIXEL*width*height);
+
+    changed = 1;
+
+  } else if ((frame->flags ^ flags) & VO_BOTH_FIELDS) {
+
+    XLockDisplay (this->display);
+    changed = 1;
+
+  }
+
+  frame->flags = flags;
+
+  if (changed) {
+
+    flags &= VO_BOTH_FIELDS;
 
     /* set up colorspace converter */
     switch (flags) {
@@ -1501,23 +1509,23 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
 
 static void opengl_overlay_clut_yuv2rgb(opengl_driver_t  *this, vo_overlay_t *overlay,
 				      opengl_frame_t *frame) {
-  int     i;
-  clut_t* clut = (clut_t*) overlay->color;
+  int i;
+  uint32_t *rgb;
 
   if (!overlay->rgb_clut) {
-    for (i = 0; i < sizeof(overlay->color)/sizeof(overlay->color[0]); i++) {
-      *((uint32_t *)&clut[i]) =
-	frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, clut[i].y,
-						  clut[i].cb, clut[i].cr);
+    rgb = overlay->color;
+    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
+      clut_t *yuv = (clut_t *)rgb;
+      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
     }
     overlay->rgb_clut++;
   }
+
   if (!overlay->hili_rgb_clut) {
-    clut = (clut_t*) overlay->hili_color;
-    for (i = 0; i < sizeof(overlay->color)/sizeof(overlay->color[0]); i++) {
-      *((uint32_t *)&clut[i]) =
-	frame->yuv2rgb->yuv2rgb_single_pixel_fun(frame->yuv2rgb, clut[i].y,
-						 clut[i].cb, clut[i].cr);
+    rgb = overlay->hili_color;
+    for (i = sizeof (overlay->color) / sizeof (overlay->color[0]); i > 0; i--) {
+      clut_t *yuv = (clut_t *)rgb;
+      *rgb++ = frame->yuv2rgb->yuv2rgb_single_pixel_fun (frame->yuv2rgb, yuv->y, yuv->cb, yuv->cr);
     }
     overlay->hili_rgb_clut++;
   }
@@ -1977,7 +1985,7 @@ static vo_driver_t *opengl_open_plugin (video_driver_class_t *class_gen, const v
     render_fun_names[i] = opengl_rb[i].name;
   render_fun_names[i] = NULL;
   this->render_fun_id = config->register_enum (config, "video.output.opengl_renderer",
-					       0, render_fun_names,
+					       0, (char **)render_fun_names,
 					       _("OpenGL renderer"),
 					       _("The OpenGL plugin provides several render modules:\n\n"
 						 "2D_Tex_Fragprog\n"
