@@ -1,6 +1,6 @@
 
 /*
- * Copyright (C) 2007-2013 the xine project
+ * Copyright (C) 2007-2016 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -55,11 +55,7 @@
 #include "yuv2rgb.h"
 #include <xine/xineutils.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <mem.h>
-#else
-#  include <libavutil/mem.h>
-#endif
+
 
 typedef struct {
   vo_frame_t         vo_frame;
@@ -85,7 +81,6 @@ typedef struct {
 
   int ovl_changed;
   raw_overlay_t overlays[XINE_VORAW_MAX_OVL];
-  yuv2rgb_t *ovl_yuv2rgb;
 
   int doYV12;
   int doYUY2;
@@ -102,28 +97,6 @@ typedef struct {
 } raw_class_t;
 
 
-
-static void raw_overlay_clut_yuv2rgb(raw_driver_t  *this, vo_overlay_t *overlay, raw_frame_t *frame)
-{
-  int i;
-  clut_t* clut = (clut_t*) overlay->color;
-
-  if (!overlay->rgb_clut) {
-    for ( i=0; i<sizeof(overlay->color)/sizeof(overlay->color[0]); i++ ) {
-      *((uint32_t *)&clut[i]) = this->ovl_yuv2rgb->yuv2rgb_single_pixel_fun(frame->yuv2rgb, clut[i].y, clut[i].cb, clut[i].cr);
-    }
-    overlay->rgb_clut++;
-  }
-  if (!overlay->hili_rgb_clut) {
-    clut = (clut_t*) overlay->hili_color;
-    for ( i=0; i<sizeof(overlay->color)/sizeof(overlay->color[0]); i++) {
-      *((uint32_t *)&clut[i]) = this->ovl_yuv2rgb->yuv2rgb_single_pixel_fun(frame->yuv2rgb, clut[i].y, clut[i].cb, clut[i].cr);
-    }
-    overlay->hili_rgb_clut++;
-  }
-}
-
-
 static int raw_process_ovl( raw_driver_t *this_gen, vo_overlay_t *overlay )
 {
   raw_overlay_t *ovl = &this_gen->overlays[this_gen->ovl_changed-1];
@@ -138,50 +111,7 @@ static int raw_process_ovl( raw_driver_t *this_gen, vo_overlay_t *overlay )
   ovl->ovl_x = overlay->x;
   ovl->ovl_y = overlay->y;
 
-  int num_rle = overlay->num_rle;
-  rle_elem_t *rle = overlay->rle;
-  uint8_t *rgba = ovl->ovl_rgba;
-  clut_t *low_colors = (clut_t*)overlay->color;
-  clut_t *hili_colors = (clut_t*)overlay->hili_color;
-  uint8_t *low_trans = overlay->trans;
-  uint8_t *hili_trans = overlay->hili_trans;
-  clut_t *colors;
-  uint8_t *trans;
-  uint8_t alpha;
-  int rlelen = 0;
-  uint8_t clr = 0;
-  int i, pos=0, x, y;
-
-  while ( num_rle>0 ) {
-    x = pos%ovl->ovl_w;
-    y = pos/ovl->ovl_w;
-    if ( (x>=overlay->hili_left && x<=overlay->hili_right) && (y>=overlay->hili_top && y<=overlay->hili_bottom) ) {
-	colors = hili_colors;
-	trans = hili_trans;
-    }
-    else {
-	colors = low_colors;
-	trans = low_trans;
-    }
-    rlelen = rle->len;
-    clr = rle->color;
-    alpha = trans[clr];
-    for ( i=0; i<rlelen; ++i ) {
-      if ( alpha == 0 ) {
-        rgba[0] = rgba[1] = rgba[2] = rgba[3] = 0;
-      }
-      else {
-        rgba[0] = colors[clr].y;
-        rgba[1] = colors[clr].cr;
-        rgba[2] = colors[clr].cb;
-        rgba[3] = alpha*255/15;
-      }
-      rgba+= 4;
-	++pos;
-    }
-    ++rle;
-    --num_rle;
-  }
+  _x_overlay_to_argb32(overlay, (uint32_t*)ovl->ovl_rgba, overlay->width, "RGBA");
   return 1;
 }
 
@@ -200,14 +130,13 @@ static void raw_overlay_begin (vo_driver_t *this_gen, vo_frame_t *frame_gen, int
 static void raw_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen, vo_overlay_t *overlay)
 {
   raw_driver_t  *this = (raw_driver_t *) this_gen;
-  raw_frame_t *frame = (raw_frame_t *) frame_gen;
 
   if ( !this->ovl_changed || this->ovl_changed>XINE_VORAW_MAX_OVL )
     return;
 
   if (overlay->rle) {
     if (!overlay->rgb_clut || !overlay->hili_rgb_clut)
-      raw_overlay_clut_yuv2rgb (this, overlay, frame);
+      _x_overlay_clut_yuv2rgb (overlay, 0);
     if ( raw_process_ovl( this, overlay ) )
       ++this->ovl_changed;
   }
@@ -287,10 +216,10 @@ static void raw_frame_dispose (vo_frame_t *vo_img)
 
   frame->yuv2rgb->dispose (frame->yuv2rgb);
 
-  av_free (frame->vo_frame.base[0]);
-  av_free (frame->vo_frame.base[1]);
-  av_free (frame->vo_frame.base[2]);
-  av_free (frame->rgb);
+  xine_free_aligned (frame->vo_frame.base[0]);
+  xine_free_aligned (frame->vo_frame.base[1]);
+  xine_free_aligned (frame->vo_frame.base[2]);
+  xine_free_aligned (frame->rgb);
   free (frame);
 }
 
@@ -343,25 +272,25 @@ static void raw_update_frame_format (vo_driver_t *this_gen, vo_frame_t *frame_ge
 /*     lprintf ("updating frame to %d x %d (ratio=%g, format=%08x)\n", width, height, ratio, format); */
 
     /* (re-) allocate render space */
-    av_free (frame->vo_frame.base[0]);
-    av_free (frame->vo_frame.base[1]);
-    av_free (frame->vo_frame.base[2]);
-    av_free (frame->rgb);
+    xine_free_aligned (frame->vo_frame.base[0]);
+    xine_free_aligned (frame->vo_frame.base[1]);
+    xine_free_aligned (frame->vo_frame.base[2]);
+    xine_free_aligned (frame->rgb);
 
     if (format == XINE_IMGFMT_YV12) {
       frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
       frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
       frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
-      frame->vo_frame.base[0] = av_mallocz (frame->vo_frame.pitches[0] * height);
-      frame->vo_frame.base[1] = av_mallocz (frame->vo_frame.pitches[1] * ((height+1)/2));
-      frame->vo_frame.base[2] = av_mallocz (frame->vo_frame.pitches[2] * ((height+1)/2));
+      frame->vo_frame.base[0] = xine_mallocz_aligned (frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[1] = xine_mallocz_aligned (frame->vo_frame.pitches[1] * ((height+1)/2));
+      frame->vo_frame.base[2] = xine_mallocz_aligned (frame->vo_frame.pitches[2] * ((height+1)/2));
     } else {
       frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
-      frame->vo_frame.base[0] = av_mallocz (frame->vo_frame.pitches[0] * height);
+      frame->vo_frame.base[0] = xine_mallocz_aligned (frame->vo_frame.pitches[0] * height);
       frame->vo_frame.base[1] = NULL;
       frame->vo_frame.base[2] = NULL;
     }
-    frame->rgb = av_mallocz (BYTES_PER_PIXEL*width*height);
+    frame->rgb = xine_mallocz_aligned (BYTES_PER_PIXEL*width*height);
 
     /* set up colorspace converter */
     switch (flags & VO_BOTH_FIELDS) {
@@ -553,12 +482,6 @@ static vo_driver_t *raw_open_plugin (video_driver_class_t *class_gen, const void
   }
   this->ovl_changed = 0;
 
-  /* we have to use a second converter for overlays
-  *  because "MODE_24_BGR, 1 (swap)" breaks overlays conversion */
-  yuv2rgb_factory_t *factory = yuv2rgb_factory_init (MODE_24_BGR, 0, NULL);
-  this->ovl_yuv2rgb = factory->create_converter( factory );
-  factory->dispose( factory );
-
   return &this->vo_driver;
 }
 
