@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -149,7 +149,7 @@ struct xv_driver_s {
 
   /* color matrix switching */
   int                cm_active, cm_state;
-  xcb_atom_t         cm_atom;
+  xcb_atom_t         cm_atom, cm_atom2;
   int                fullrange_mode;
 };
 
@@ -240,6 +240,11 @@ static void create_ximage(xv_driver_t *this, xv_frame_t *frame, int width, int h
     _x_abort();
   }
 
+  if (frame->xv_format == 0) {
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unsupported format %08x\n", format);
+    return;
+  }
+
   query_attributes_cookie = xcb_xv_query_image_attributes(this->connection, this->xv_port, frame->xv_format, width, height);
   query_attributes_reply = xcb_xv_query_image_attributes_reply(this->connection, query_attributes_cookie, NULL);
 
@@ -387,10 +392,26 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
       dispose_ximage(this, frame);
 
     create_ximage(this, frame, width, height, format);
+    if (!frame->image) {
+      pthread_mutex_unlock(&this->main_mutex);
+      frame->vo_frame.base[0] = NULL;
+      frame->vo_frame.base[1] = NULL;
+      frame->vo_frame.base[2] = NULL;
+      frame->req_width = 0;
+      frame->vo_frame.width = 0;
+      return;
+    }
 
     if(format == XINE_IMGFMT_YUY2) {
       frame->vo_frame.pitches[0] = frame->xv_pitches[0];
       frame->vo_frame.base[0] = frame->image + frame->xv_offsets[0];
+      {
+        const union {uint8_t bytes[4]; uint32_t word;} black = {{0, 128, 0, 128}};
+        uint32_t *q = (uint32_t *)frame->vo_frame.base[0];
+        int i;
+        for (i = frame->vo_frame.pitches[0] * frame->xv_height / 4; i > 0; i--)
+          *q++ = black.word;
+      }
     }
     else {
       frame->vo_frame.pitches[0] = frame->xv_pitches[0];
@@ -399,6 +420,9 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
       frame->vo_frame.base[0] = frame->image + frame->xv_offsets[0];
       frame->vo_frame.base[1] = frame->image + frame->xv_offsets[2];
       frame->vo_frame.base[2] = frame->image + frame->xv_offsets[1];
+      memset (frame->vo_frame.base[0], 0, frame->vo_frame.pitches[0] * frame->xv_height);
+      memset (frame->vo_frame.base[1], 128, frame->vo_frame.pitches[1] * frame->xv_height / 2);
+      memset (frame->vo_frame.base[2], 128, frame->vo_frame.pitches[2] * frame->xv_height / 2);
     }
 
     /* allocated frame size may not match requested size */
@@ -628,13 +652,21 @@ static void xv_new_color (xv_driver_t *this, int cm) {
     xcb_xv_set_port_attribute (this->connection, this->xv_port, atom, satu);
   pthread_mutex_unlock(&this->main_mutex);
 
-  /* so far only binary nvidia drivers support this. why not nuveau? */
   if (this->cm_atom != XCB_NONE) {
+    /* 0 = 601 (SD), 1 = 709 (HD) */
+    /* so far only binary nvidia drivers support this. why not nuveau? */
     cm2 = (0xc00c >> cm) & 1;
     pthread_mutex_lock(&this->main_mutex);
     xcb_xv_set_port_attribute (this->connection, this->xv_port, this->cm_atom, cm2);
     pthread_mutex_unlock(&this->main_mutex);
     cm2 = cm2 ? 2 : 10;
+  } else if (this->cm_atom2 != XCB_NONE) {
+    /* radeonhd: 0 = size based auto, 1 = 601 (SD), 2 = 709 (HD) */
+    cm2 = ((0xc00c >> cm) & 1) + 1;
+    pthread_mutex_lock(&this->main_mutex);
+    xcb_xv_set_port_attribute (this->connection, this->xv_port, this->cm_atom2, cm2);
+    pthread_mutex_unlock(&this->main_mutex);
+    cm2 = cm2 == 2 ? 2 : 10;
   } else {
     cm2 = 10;
   }
@@ -1013,6 +1045,7 @@ static void xv_restore_port_attributes(xv_driver_t *this) {
   pthread_mutex_unlock(&this->main_mutex);
 
   xine_list_delete( this->port_attributes );
+  this->port_attributes = NULL;
 }
 
 static void xv_dispose (vo_driver_t *this_gen) {
@@ -1020,11 +1053,17 @@ static void xv_dispose (vo_driver_t *this_gen) {
   int          i;
 
   /* restore port attributes to their initial values */
-  xv_restore_port_attributes(this);
+  if (this->port_attributes) {
+    xv_restore_port_attributes(this);
+  }
 
   pthread_mutex_lock(&this->main_mutex);
-  xcb_xv_ungrab_port(this->connection, this->xv_port, XCB_CURRENT_TIME);
-  xcb_free_gc(this->connection, this->gc);
+  if (this->xv_port) {
+    xcb_xv_ungrab_port(this->connection, this->xv_port, XCB_CURRENT_TIME);
+  }
+  if (this->gc) {
+    xcb_free_gc(this->connection, this->gc);
+  }
   pthread_mutex_unlock(&this->main_mutex);
 
   for( i=0; i < VO_NUM_RECENT_FRAMES; i++ ) {
@@ -1288,10 +1327,6 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   if (!this)
     return NULL;
 
-  pthread_mutex_init(&this->main_mutex, NULL);
-
-  _x_alphablend_init(&this->alphablend_extra_data, class->xine);
-
   this->connection        = visual->connection;
   this->screen            = visual->screen;
   this->window            = visual->window;
@@ -1304,6 +1339,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   query_extension_reply = xcb_get_extension_data(this->connection, &xcb_xv_id);
   if (!query_extension_reply || !query_extension_reply->present) {
     xprintf (class->xine, XINE_VERBOSITY_LOG, _("%s: Xv extension not present.\n"), LOG_MODULE);
+    free(this);
     return NULL;
   }
 
@@ -1316,6 +1352,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 
   if (!query_adaptors_reply) {
     xprintf(class->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": XvQueryAdaptors failed.\n");
+    free(this);
     return NULL;
   }
 
@@ -1324,7 +1361,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 				  VIDEO_DEVICE_XV_PORT_HELP,
 				  20, NULL, NULL);
   prefer_type = config->register_enum (config, "video.device.xv_preferred_method", 0,
-				       prefer_labels, VIDEO_DEVICE_XV_PREFER_TYPE_HELP,
+				       (char **)prefer_labels, VIDEO_DEVICE_XV_PREFER_TYPE_HELP,
 				       10, NULL, NULL);
 
   if (xv_port != 0) {
@@ -1359,6 +1396,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	    LOG_MODULE);
 
     /* XvFreeAdaptorInfo (adaptor_info); this crashed on me (gb)*/
+    free(this);
     return NULL;
   }
   else
@@ -1369,6 +1407,10 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 
   this->xv_port           = xv_port;
 
+  pthread_mutex_init(&this->main_mutex, NULL);
+
+  _x_alphablend_init(&this->alphablend_extra_data, class->xine);
+
   _x_vo_scale_init (&this->sc, 1, 0, config );
   this->sc.frame_output_cb   = visual->frame_output_cb;
   this->sc.user_data         = visual->user_data;
@@ -1383,6 +1425,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   this->ovl_changed             = 0;
   this->xine                    = class->xine;
   this->cm_atom                 = XCB_NONE;
+  this->cm_atom2                = XCB_NONE;
 
   this->vo_driver.get_capabilities     = xv_get_capabilities;
   this->vo_driver.alloc_frame          = xv_alloc_frame;
@@ -1465,7 +1508,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	  xv_check_capability (this, VO_PROP_GAMMA, attribute_it.data,
 			       adaptor_it.data->base_id,
 			       NULL, NULL, NULL);
-	} else if(!strcmp(name, "XV_ITURBT_709")) {
+	} else if(!strcmp(name, "XV_ITURBT_709")) { /* nvidia */
 	  xcb_xv_get_port_attribute_cookie_t get_attribute_cookie;
 	  xcb_xv_get_port_attribute_reply_t *get_attribute_reply;
 	  xcb_intern_atom_cookie_t atom_cookie;
@@ -1489,6 +1532,30 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	    }
 	  }
 	  pthread_mutex_unlock(&this->main_mutex);
+	} else if(!strcmp(name, "XV_COLORSPACE")) { /* radeonhd */
+	  xcb_xv_get_port_attribute_cookie_t get_attribute_cookie;
+	  xcb_xv_get_port_attribute_reply_t *get_attribute_reply;
+	  xcb_intern_atom_cookie_t atom_cookie;
+	  xcb_intern_atom_reply_t *atom_reply;
+	  pthread_mutex_lock(&this->main_mutex);
+	  atom_cookie = xcb_intern_atom (this->connection, 0, 13, name);
+	  atom_reply = xcb_intern_atom_reply (this->connection, atom_cookie, NULL);
+	  if (atom_reply) {
+	    this->cm_atom2 = atom_reply->atom;
+	    free (atom_reply);
+	  }
+	  if (this->cm_atom2 != XCB_NONE) {
+	    get_attribute_cookie = xcb_xv_get_port_attribute (this->connection,
+	      this->xv_port, this->cm_atom2);
+	    get_attribute_reply = xcb_xv_get_port_attribute_reply (this->connection,
+	      get_attribute_cookie, NULL);
+	    if (get_attribute_reply) {
+	      this->cm_active = get_attribute_reply->value == 2 ? 2 : (get_attribute_reply->value == 1 ? 10 : 0);
+	      free(get_attribute_reply);
+	      this->capabilities |= VO_CAP_COLOR_MATRIX;
+	    }
+	  }
+	  pthread_mutex_unlock(&this->main_mutex);
 	} else if(!strcmp(name, "XV_COLORKEY")) {
 	  this->capabilities |= VO_CAP_COLORKEY;
 	  xv_check_capability (this, VO_PROP_COLORKEY, attribute_it.data,
@@ -1531,7 +1598,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	} else if(!strcmp(name, "XV_BICUBIC")) {
 	  int xv_bicubic =
 	    config->register_enum (config, "video.device.xv_bicubic", 2,
-				   bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP,
+				   (char **)bicubic_types, VIDEO_DEVICE_XV_BICUBIC_HELP,
 				   20, xv_update_XV_BICUBIC, this);
 	  config->update_num(config,"video.device.xv_bicubic",xv_bicubic);
 	}
@@ -1608,6 +1675,20 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 
   free(list_formats_reply);
 
+  // XXX
+  // Bail out if both YV12 and YUY2 are not supported.
+  // Proper fix would be implementing format conversions.
+  //  - decoder should check video output CAP_ flags
+  //  - video out could insert automatic conversion filter
+  //  - ...
+  if (!this->xv_format_yuy2 || !this->xv_format_yv12) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG,
+            LOG_MODULE ": this adaptor does not support YV12 and YUY2 formats.\n");
+    xv_dispose(&this->vo_driver);
+    return NULL;
+  }
+
+
   this->use_pitch_alignment =
     config->register_bool (config, "video.device.xv_pitch_alignment", 0,
 			   VIDEO_DEVICE_XV_PITCH_ALIGNMENT_HELP,
