@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -208,7 +208,7 @@ static void xv_frame_dispose (vo_frame_t *vo_img) {
     }
     else {
       LOCK_DISPLAY(this);
-      free (frame->image->data);
+      xine_free_aligned (frame->image->data);
       XFree (frame->image);
       UNLOCK_DISPLAY(this);
     }
@@ -286,6 +286,11 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
     _x_abort();
   }
 
+  if (xv_format == 0) {
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unsupported format %08x\n",format);
+    return NULL;
+  }
+
   if (this->use_shm) {
 
     /*
@@ -390,6 +395,7 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
     shminfo->shmid = -1;
 
   finishShmTesting:
+    XSync(this->display, False);
     x11_DeInstallXErrorHandler(this);
   }
 
@@ -399,29 +405,17 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
    */
 
   if (!this->use_shm) {
-    char *data;
-
-    switch (format) {
-    case XINE_IMGFMT_YV12:
-      data = malloc (width * height * 3/2);
-      break;
-    case XINE_IMGFMT_YUY2:
-      data = malloc (width * height * 2);
-      break;
-    default:
-      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "create_ximage: unknown format %08x\n",format);
-      _x_abort();
+    image = XvCreateImage (this->display, this->xv_port, xv_format, NULL, width, height);
+    if (image) {
+      image->data = xine_malloc_aligned (image->data_size);
+      if (!image->data) {
+        XFree (image);
+        image = NULL;
+      }
     }
-
-    if (data) {
-      image = XvCreateImage (this->display, this->xv_port, xv_format, data, width, height);
-      if (!image)
-        free (data);
-    } else
-      image = NULL;
-
     shminfo->shmaddr = 0;
   }
+
   return image;
 }
 
@@ -442,7 +436,7 @@ static void dispose_ximage (xv_driver_t *this,
 
   }
   else {
-    free (myimage->data);
+    xine_free_aligned (myimage->data);
 
     XFree (myimage);
   }
@@ -1118,6 +1112,7 @@ static void xv_restore_port_attributes(xv_driver_t *this) {
   UNLOCK_DISPLAY(this);
 
   xine_list_delete( this->port_attributes );
+  this->port_attributes = NULL;
 }
 
 static void xv_dispose (vo_driver_t *this_gen) {
@@ -1125,13 +1120,19 @@ static void xv_dispose (vo_driver_t *this_gen) {
   int          i;
 
   /* restore port attributes to their initial values */
-  xv_restore_port_attributes(this);
+  if (this->port_attributes) {
+    xv_restore_port_attributes(this);
+  }
 
   LOCK_DISPLAY(this);
-  if(XvUngrabPort (this->display, this->xv_port, CurrentTime) != Success) {
-    xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": xv_exit: XvUngrabPort() failed.\n");
+  if (this->xv_port) {
+    if (XvUngrabPort (this->display, this->xv_port, CurrentTime) != Success) {
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": xv_exit: XvUngrabPort() failed.\n");
+    }
+  }
+  if (this->gc) {
+    XFreeGC(this->display, this->gc);
   }
-  XFreeGC(this->display, this->gc);
   UNLOCK_DISPLAY(this);
 
   for( i=0; i < VO_NUM_RECENT_FRAMES; i++ ) {
@@ -1366,8 +1367,6 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   if (!this)
     return NULL;
 
-  _x_alphablend_init(&this->alphablend_extra_data, class->xine);
-
   this->display           = visual->display;
   this->screen            = visual->screen;
   this->config            = config;
@@ -1385,6 +1384,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   if (Success != XvQueryExtension(this->display, &ver,&rel, &req, &ev,&err)) {
     xprintf (class->xine, XINE_VERBOSITY_LOG, _("%s: Xv extension not present.\n"), LOG_MODULE);
     UNLOCK_DISPLAY(this);
+    free(this);
     return NULL;
   }
 
@@ -1395,6 +1395,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   if (Success != XvQueryAdaptors(this->display,DefaultRootWindow(this->display), &adaptors, &adaptor_info))  {
     xprintf(class->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": XvQueryAdaptors failed.\n");
     UNLOCK_DISPLAY(this);
+    free(this);
     return NULL;
   }
 
@@ -1433,6 +1434,7 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
 
     /* XvFreeAdaptorInfo (adaptor_info); this crashed on me (gb)*/
     UNLOCK_DISPLAY(this);
+    free(this);
     return NULL;
   }
   else
@@ -1443,6 +1445,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
 
   UNLOCK_DISPLAY(this);
 
+  _x_alphablend_init(&this->alphablend_extra_data, class->xine);
+
   this->xv_port           = xv_port;
 
   _x_vo_scale_init (&this->sc, 1, 0, config );
@@ -1689,6 +1693,19 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
     UNLOCK_DISPLAY(this);
   }
 
+  // XXX
+  // Bail out if both YV12 and YUY2 are not supported.
+  // Proper fix would be implementing format conversions.
+  //  - decoder should check video output CAP_ flags
+  //  - video out could insert automatic conversion filter
+  //  - ...
+  if (!this->xv_format_yuy2 || !this->xv_format_yv12) {
+    xprintf(this->xine, XINE_VERBOSITY_LOG,
+            LOG_MODULE ": this adaptor does not support YV12 and YUY2 formats.\n");
+    xv_dispose(&this->vo_driver);
+    return NULL;
+  }
+
   /*
    * check max. supported image size
    */
@@ -1716,7 +1733,8 @@ static vo_driver_t *open_plugin_2 (video_driver_class_t *class_gen, const void *
   LOCK_DISPLAY(this);
   myimage = create_ximage (this, &myshminfo, 100, 100,
 			   (this->xv_format_yv12 != 0) ? XINE_IMGFMT_YV12 : XINE_IMGFMT_YUY2);
-  dispose_ximage (this, &myshminfo, myimage);
+  if (myimage)
+    dispose_ximage (this, &myshminfo, myimage);
   UNLOCK_DISPLAY(this);
 
   this->use_pitch_alignment =
