@@ -1,7 +1,7 @@
 /*
  * yuv2rgb.c
  *
- * Copyright (C) 2003-2012 the xine project
+ * Copyright (C) 2003-2016 the xine project
  * This file is part of xine, a free video player.
  *
  * based on work from mpeg2dec:
@@ -31,11 +31,7 @@
 #include <string.h>
 #include <inttypes.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <mem.h>
-#else
-#  include <libavutil/mem.h>
-#endif
+
 
 #include "yuv2rgb.h"
 
@@ -3294,7 +3290,7 @@ static yuv2rgb_t *yuv2rgb_create_converter (yuv2rgb_factory_t *factory) {
 static void yuv2rgb_factory_dispose (yuv2rgb_factory_t *this) {
 
   free (this->table_base);
-  av_free(this->table_mmx);
+  xine_free_aligned(this->table_mmx);
   free (this);
 }
 
