@@ -31,12 +31,6 @@
 #include <string.h>
 #include <inttypes.h>
 
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <mem.h>
-#else
-#  include <libavutil/mem.h>
-#endif
-
 #include "yuv2rgb.h"
 #include <xine/xineutils.h>
 #include "xine_mmx.h"
@@ -87,7 +81,7 @@ void mmx_yuv2rgb_set_csc_levels(yuv2rgb_factory_t *this,
 
   /* 'table_mmx' is 64bit aligned for better performance */
   if (this->table_mmx == NULL) {
-    this->table_mmx = av_mallocz(sizeof(mmx_csc_t));
+    this->table_mmx = xine_mallocz_aligned(sizeof(mmx_csc_t));
   }
 
   /* full range mode */
@@ -1309,12 +1303,12 @@ void yuv2rgb_init_mmx (yuv2rgb_factory_t *this) {
   if (this->swapped) switch (this->mode) {
   case MODE_24_RGB:
     this->yuv2rgb_fun = mmx_bgr24;
-    break;
+    return;
   case MODE_24_BGR:
     this->yuv2rgb_fun = mmx_rgb24;
-    break;
+    return;
   default:
-    return; /* other swapped formats not yet */
+    return; /* other swapped formats yet unsupported */
   }
 
   switch (this->mode) {
