@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2012 Edgar Hucek <gimli|@dark-green.com>
- * Copyright (C) 2012-2013 xine developers
+ * Copyright (C) 2012-2016 xine developers
  *
  * This file is part of xine, a free video player.
  *
@@ -36,14 +36,6 @@ extern "C" {
 #endif
 
 #include <va/va_x11.h>
-#include <pthread.h>
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <avcodec.h>
-#else
-#  include <libavcodec/avcodec.h>
-#endif
-
-#include "../combined/ffmpeg/ffmpeg_compat.h"
 
 #define NUM_OUTPUT_SURFACES 22
 
@@ -61,6 +53,7 @@ struct vaapi_equalizer {
 };
 
 typedef struct ff_vaapi_context_s ff_vaapi_context_t;
+typedef struct ff_vaapi_surface_s ff_vaapi_surface_t;
 
 struct ff_vaapi_context_s {
   VADisplay         va_display;
@@ -68,53 +61,60 @@ struct ff_vaapi_context_s {
   VAConfigID        va_config_id;
   int               width;
   int               height;
-  int               sw_width;
-  int               sw_height;
-  int               va_profile;
-  unsigned int      va_colorspace;
-  VAImage           va_subpic_image;
-  VASubpictureID    va_subpic_id;
-  int               va_subpic_width;
-  int               va_subpic_height;
-  int               is_bound;
-  void              *gl_surface;
-  unsigned int      soft_head;
   unsigned int      valid_context;
+
+  /* decoding surfaces */
+  VASurfaceID        *va_surface_ids;
+  ff_vaapi_surface_t *va_render_surfaces;
   unsigned int      va_head;
-  unsigned int      va_soft_head;
+
   vo_driver_t       *driver;
-  unsigned int      last_sub_image_fmt;
-  VASurfaceID       last_sub_surface_id;
   struct vaapi_equalizer va_equalizer;
   VAImageFormat     *va_image_formats;
   int               va_num_image_formats;
-  VAImageFormat     *va_subpic_formats;
-  int               va_num_subpic_formats;
 };
 
-typedef struct ff_vaapi_surface_s ff_vaapi_surface_t;
 typedef struct vaapi_accel_s vaapi_accel_t;
 
 struct ff_vaapi_surface_s {
   unsigned int        index;
-  vaapi_accel_t       *accel;
   VASurfaceID         va_surface_id;
   unsigned int        status;
 };
 
+  /*
+   *
+   */
+
+#define IMGFMT_VAAPI               0x56410000 /* 'VA'00 */
+#define IMGFMT_VAAPI_MASK          0xFFFF0000
+#define IMGFMT_VAAPI_CODEC_MASK    0x000000F0
+#define IMGFMT_VAAPI_CODEC(fmt)    ((fmt) & IMGFMT_VAAPI_CODEC_MASK)
+#define IMGFMT_VAAPI_CODEC_MPEG2   (0x10)
+#define IMGFMT_VAAPI_CODEC_MPEG4   (0x20)
+#define IMGFMT_VAAPI_CODEC_H264    (0x30)
+#define IMGFMT_VAAPI_CODEC_VC1     (0x40)
+#define IMGFMT_VAAPI_CODEC_HEVC    (0x50)
+#define IMGFMT_VAAPI_MPEG2         (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_MPEG2)
+#define IMGFMT_VAAPI_MPEG2_IDCT    (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_MPEG2|1)
+#define IMGFMT_VAAPI_MPEG2_MOCO    (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_MPEG2|2)
+#define IMGFMT_VAAPI_MPEG4         (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_MPEG4)
+#define IMGFMT_VAAPI_H263          (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_MPEG4|1)
+#define IMGFMT_VAAPI_H264          (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_H264)
+#define IMGFMT_VAAPI_HEVC          (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_HEVC)
+#define IMGFMT_VAAPI_HEVC_MAIN10   (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_HEVC|1)
+#define IMGFMT_VAAPI_VC1           (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_VC1)
+#define IMGFMT_VAAPI_WMV3          (IMGFMT_VAAPI|IMGFMT_VAAPI_CODEC_VC1|1)
+
 struct vaapi_accel_s {
   unsigned int        index;
   vo_frame_t          *vo_frame;
 
-#if AVVIDEO > 1
-  int (*avcodec_decode_video2)(vo_frame_t *frame_gen, AVCodecContext *avctx, AVFrame *picture,
-                               int *got_picture_ptr, AVPacket *avpkt);
-#else
-  int (*avcodec_decode_video)(vo_frame_t *frame_gen, AVCodecContext *avctx, AVFrame *picture,
-                              int *got_picture_ptr, uint8_t *buf, int buf_size);
-#endif
-  VAStatus (*vaapi_init)(vo_frame_t *frame_gen, int va_profile, int width, int height, int softrender);
-  int (*profile_from_imgfmt)(vo_frame_t *frame_gen, enum PixelFormat pix_fmt, int codec_id, int vaapi_mpeg_sofdec);
+  int  (*lock_vaapi)(vo_frame_t *frame_gen);
+  void (*unlock_vaapi)(vo_frame_t *frame_gen);
+
+  VAStatus (*vaapi_init)(vo_frame_t *frame_gen, int va_profile, int width, int height);
+  int (*profile_from_imgfmt)(vo_frame_t *frame_gen, unsigned img_fmt);
   ff_vaapi_context_t *(*get_context)(vo_frame_t *frame_gen);
   int (*guarded_render)(vo_frame_t *frame_gen);
   ff_vaapi_surface_t *(*get_vaapi_surface)(vo_frame_t *frame_gen);
