@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -811,6 +811,7 @@ static const video_db_t video_db[] = {
 {
   {
     ME_FOURCC('h','e','v','c'),
+    ME_FOURCC('h','v','c','1'),
     0
   },
   BUF_VIDEO_HEVC,
@@ -1239,6 +1240,14 @@ static const audio_db_t audio_db[] = {
   BUF_AUDIO_OPUS,
   "Opus Audio"
 },
+{
+  {
+    ME_FOURCC('t', 'r', 'h', 'd'),
+    0
+  },
+  BUF_AUDIO_TRUEHD,
+  "TrueHD Audio"
+},
 { { 0 }, 0, "last entry" }
 };
 
