@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -34,11 +34,6 @@
 #include <unistd.h>
 #include <xine/configfile.h>
 #include "bswap.h"
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#  include <base64.h>
-#else
-#  include <libavutil/base64.h>
-#endif
 
 #define LOG_MODULE "configfile"
 #define LOG_VERBOSE
@@ -1322,7 +1317,7 @@ static char* config_get_serialized_entry (config_values_t *this, const char *key
 
     char **cur_value = entry->enum_values;
     if (cur_value) {
-      while (*cur_value && (value_count < (sizeof(value_len) / sizeof(int) ))) {
+      while (*cur_value && ((size_t)value_count < (sizeof(value_len) / sizeof(int) ))) {
         value_len[value_count] = strlen(*cur_value);
         total_len += sizeof(int32_t) + value_len[value_count];
         value_count++;
@@ -1331,8 +1326,11 @@ static char* config_get_serialized_entry (config_values_t *this, const char *key
     }
 
     /* Now we have the length needed to serialize the entry and the length of each string */
-    uint8_t *buffer = malloc (total_len);
-    if (!buffer) return NULL;
+    uint8_t *buffer = malloc (total_len + 4);
+    if (!buffer) {
+      pthread_mutex_unlock(&this->config_lock);
+      return NULL;
+    }
 
     /* Let's go */
 
@@ -1360,12 +1358,9 @@ static char* config_get_serialized_entry (config_values_t *this, const char *key
     }
 
     /* and now the output encoding */
-    /* We're going to encode total_len bytes in base64
-     * libavutil's base64 encoding functions want the size to
-     * be at least len * 4 / 3 + 12, so let's use that!
-     */
-    output = malloc(total_len * 4 / 3 + 12);
-    av_base64_encode(output, total_len * 4 / 3 + 12, buffer, total_len);
+    /* We're going to encode total_len bytes in base64 */
+    output = malloc((total_len * 4 + 2) / 3 + 4);
+    xine_base64_encode (buffer, output, total_len);
 
     free(buffer);
   }
@@ -1375,7 +1370,7 @@ static char* config_get_serialized_entry (config_values_t *this, const char *key
 
 }
 
-static int get_int(uint8_t *buffer, int buffer_size, int pos, int *value) {
+static size_t get_int(uint8_t *buffer, size_t buffer_size, size_t pos, int *value) {
   int32_t value_int32;
 
   if ((pos + sizeof(int32_t)) > buffer_size)
@@ -1386,9 +1381,9 @@ static int get_int(uint8_t *buffer, int buffer_size, int pos, int *value) {
   return sizeof(int32_t);
 }
 
-static int get_string(uint8_t *buffer, int buffer_size, int pos, char **value) {
+static size_t get_string(uint8_t *buffer, int buffer_size, size_t pos, char **value) {
   int len;
-  int bytes = get_int(buffer, buffer_size, pos, &len);
+  size_t bytes = get_int(buffer, buffer_size, pos, &len);
   *value = NULL;
 
   if (!bytes || (len < 0) || (len > 1024*64))
@@ -1431,15 +1426,18 @@ static char* config_register_serialized_entry (config_values_t *this, const char
   char **enum_values = NULL;
 
   int    bytes;
-  int    pos;
+  size_t pos;
   void  *output = NULL;
   size_t output_len;
   int    value_count = 0;
   int    i;
 
-  output_len = strlen(value) * 3 / 4 + 1;
-  output = malloc(output_len);
-  av_base64_decode(output, value, output_len);
+  output = malloc ((strlen (value) * 3 + 3) / 4 + 1);
+  output_len = xine_base64_decode (value, output);
+  {
+    uint8_t *q = output;
+    q[output_len] = 0;
+  }
 
   pos = 0;
   pos += bytes = get_int(output, output_len, pos, &type);
@@ -1610,8 +1608,10 @@ int _x_config_change_opt(config_values_t *config, const char *opt) {
     /* someone messed with the config entry */
     return -1;
 
+  if ( *opt == '\0' ) return 0;
+
   key = strdup(opt);
-  if ( !key || *key == '\0' ) return 0;
+  if ( !key ) return 0;
 
   value = strrchr(key, ':');
   if ( !value || *value == '\0' ) {
