@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project,
+ * Copyright (C) 2000-2017 the xine project,
  *
  * This file is part of xine, a free video player.
  *
@@ -52,7 +52,7 @@
 static int _x_io_tcp_connect_ipv4(xine_stream_t *stream, const char *host, int port) {
 
   struct hostent *h;
-  int             i, s;
+  int             i;
 
   h = gethostbyname(host);
   if (h == NULL) {
@@ -60,32 +60,35 @@ static int _x_io_tcp_connect_ipv4(xine_stream_t *stream, const char *host, int p
     return -1;
   }
 
-  s = xine_socket_cloexec(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-  if (s == -1) {
-    _x_message(stream, XINE_MSG_CONNECTION_REFUSED, "failed to create socket", strerror(errno), NULL);
-    return -1;
-  }
+  for (i = 0; h->h_addr_list[i]; i++) {
+
+    int s = xine_socket_cloexec(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+    if (s == -1) {
+      _x_message(stream, XINE_MSG_CONNECTION_REFUSED, "failed to create socket", strerror(errno), NULL);
+      return -1;
+    }
 
 #ifndef WIN32
-  if (fcntl (s, F_SETFL, fcntl (s, F_GETFL) | O_NONBLOCK) == -1) {
-    _x_message(stream, XINE_MSG_CONNECTION_REFUSED, "can't put socket in non-blocking mode", strerror(errno), NULL);
-    return -1;
-  }
-#else
-  {
-	unsigned long non_block = 1;
-	int rc;
-
-    rc = ioctlsocket(s, FIONBIO, &non_block);
-
-    if (rc == SOCKET_ERROR) {
+    if (fcntl (s, F_SETFL, fcntl (s, F_GETFL) | O_NONBLOCK) == -1) {
       _x_message(stream, XINE_MSG_CONNECTION_REFUSED, "can't put socket in non-blocking mode", strerror(errno), NULL);
-	  return -1;
+      close(s);
+      return -1;
+    }
+#else
+    {
+      unsigned long non_block = 1;
+      int rc;
+
+      rc = ioctlsocket(s, FIONBIO, &non_block);
+
+      if (rc == SOCKET_ERROR) {
+        _x_message(stream, XINE_MSG_CONNECTION_REFUSED, "can't put socket in non-blocking mode", strerror(errno), NULL);
+        close(s);
+        return -1;
+      }
     }
-  }
 #endif
 
-  for (i = 0; h->h_addr_list[i]; i++) {
     struct in_addr ia;
     union {
       struct sockaddr    sa;
@@ -111,6 +114,7 @@ static int _x_io_tcp_connect_ipv4(xine_stream_t *stream, const char *host, int p
 
     return s;
   }
+
   return -1;
 }
 #endif
