@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -66,6 +66,8 @@
 
 #include "xine_private.h"
 
+#include "builtins.h"
+
 #define LINE_MAX_LENGTH   (1024 * 32)  /* 32 KiB */
 
 #if 0
@@ -262,12 +264,13 @@ static plugin_node_t *_get_cached_node (xine_t *this,
   list_size = xine_sarray_size (list);
   for (list_id = 0; list_id < list_size; list_id++) {
     plugin_node_t *node = xine_sarray_get (list, list_id);
-    if( !previous_node &&
-	node->file->filesize == filesize &&
-	node->file->filemtime == filemtime &&
-	!strcmp( node->file->filename, filename )) {
-
-      return node;
+    if (!previous_node) {
+      if (node->file) {
+        if (node->file->filesize == filesize &&
+            node->file->filemtime == filemtime &&
+            !strcmp( node->file->filename, filename ))
+          return node;
+      }
     }
 
     /* skip previously returned items */
@@ -298,12 +301,11 @@ static plugin_file_t *_insert_file (xine_t *this,
   return entry;
 }
 
-
 static void _insert_node (xine_t *this,
 			  xine_sarray_t *list,
 			  plugin_file_t *file,
 			  plugin_node_t *node_cache,
-			  plugin_info_t *info,
+			  const plugin_info_t *info,
 			  int api_version){
 
   plugin_catalog_t     *catalog = this->plugin_catalog;
@@ -487,18 +489,51 @@ static plugin_catalog_t *XINE_MALLOC _new_catalog(void){
 }
 
 static void _register_plugins_internal(xine_t *this, plugin_file_t *file,
-                                       plugin_node_t *node_cache, plugin_info_t *info) {
+                                       plugin_node_t *node_cache, const plugin_info_t *info) {
+  /* user supplied xine_register_plugins () */
+  static const char * const st_names[10] = {
+    "user/none",
+    "user/input",
+    "user/demux",
+    "user/audio_decoder",
+    "user/video_decoder",
+    "user/spu_decoder",
+    "user/audio_out",
+    "user/video_out",
+    "user/post",
+    "user"
+  };
+  const char * const *names = st_names;
+#ifdef XINE_MAKE_BUILTINS
+  static const char * const builtin_names[10] = {
+    "libxine/builtins/none",
+    "libxine/builtins/input",
+    "libxine/builtins/demux",
+    "libxine/builtins/audio_decoder",
+    "libxine/builtins/video_decoder",
+    "libxine/builtins/spu_decoder",
+    "libxine/builtins/audio_out",
+    "libxine/builtins/video_out",
+    "libxine/builtins/post",
+    "libxine/builtins"
+  };
+  if (info == xine_builtin_plugin_info)
+    names = builtin_names;
+#endif
   _x_assert(this);
   _x_assert(info);
 
   while ( info && info->type != PLUGIN_NONE ) {
-
-    if (file && file->filename)
-      xine_log (this, XINE_LOG_PLUGIN,
-		_("load_plugins: plugin %s found\n"), file->filename);
-    else
-      xine_log (this, XINE_LOG_PLUGIN,
-		_("load_plugins: static plugin found\n"));
+    const char *fn;
+    if (file && file->filename) {
+      fn = file->filename;
+    } else {
+      int n = info->type & PLUGIN_TYPE_MASK;
+      if (n > 9)
+        n = 9;
+      fn = names[n];
+    }
+    xine_log (this, XINE_LOG_PLUGIN, _("load_plugins: plugin %s:%s found\n"), fn, info->id);
 
     if (this->plugin_catalog->plugin_count >= PLUGIN_MAX ||
 	(this->plugin_catalog->decoder_count >= DECODER_MAX &&
@@ -644,6 +679,7 @@ static void collect_plugins(xine_t *this, char *path){
 
 	      xine_log (this, XINE_LOG_PLUGIN,
 			_("load_plugins: can't get plugin info from %s:\n%s\n"), str, error);
+              dlclose(lib);
 	    }
 	  }
 	  break;
@@ -832,21 +868,25 @@ static void _load_required_plugins(xine_t *this, xine_sarray_t *list) {
     /*
      * preload plugins if not cached
      */
-    if( (node->info->type & PLUGIN_MUST_PRELOAD) && !node->plugin_class &&
-        node->file->lib_handle ) {
+    do {
+      if (!(node->info->type & PLUGIN_MUST_PRELOAD)) /* no preload needed */
+        break;
+      if (node->plugin_class) /* is already loaded */
+        break;
+      if (node->file && !node->file->lib_handle) /* lib unavailable */
+        break;
 
-      lprintf("preload plugin %s from %s\n", node->info->id, node->file->filename);
+      lprintf ("preload plugin %s from %s\n", node->info->id, node->file ? node->file->filename : "libxine/builtins");
 
       if (! _load_plugin_class (this, node, NULL)) {
 	/* in case of failure remove from list */
 
 	xine_sarray_remove(list, list_id);
 	list_size = xine_sarray_size(list);
-
-      } else
-	list_id++;
-    } else
-      list_id++;
+        list_id--;
+      }
+    } while (0);
+    list_id++;
   }
 }
 
@@ -882,9 +922,14 @@ static void save_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *list) {
     node = xine_sarray_get(list, list_id);
 
     file = node->file;
-    fprintf(fp, "[%s]\n", file->filename );
-    fprintf(fp, "size=%" PRId64 "\n", (uint64_t) file->filesize );
-    fprintf(fp, "mtime=%" PRId64 "\n", (uint64_t) file->filemtime );
+    if (file) {
+      fprintf(fp, "[%s]\n", file->filename );
+      fprintf(fp, "size=%" PRId64 "\n", (uint64_t) file->filesize );
+      fprintf(fp, "mtime=%" PRId64 "\n", (uint64_t) file->filemtime );
+    } else {
+      /* dump builtins for debugging */
+      fprintf(fp, "[libxine/builtins]\n");
+    }
     fprintf(fp, "type=%d\n", node->info->type );
     fprintf(fp, "api=%d\n", node->info->API );
     fprintf(fp, "id=%s\n", node->info->id );
@@ -973,6 +1018,7 @@ static void load_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *plugins) {
   char *value;
   size_t line_len;
   int version_ok = 0;
+  int skip = 0;
 
   line = malloc(LINE_MAX_LENGTH);
   if (!line)
@@ -1000,7 +1046,15 @@ static void load_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *plugins) {
 
       if( node ) {
         xine_sarray_add (plugins, node);
+        node = NULL;
       }
+
+      if (!strcmp (line, "[libxine/builtins")) {
+        skip = 1;
+        continue;
+      }
+
+      skip = 0;
       node                = calloc(1, sizeof(plugin_node_t));
       file                = calloc(1, sizeof(plugin_file_t));
       node->file          = file;
@@ -1013,6 +1067,9 @@ static void load_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *plugins) {
       post_info           = NULL;
     }
 
+    if (skip)
+      continue;
+
     if ((value = strchr (line, '='))) {
 
       *value = (char) 0;
@@ -1023,16 +1080,22 @@ static void load_plugin_list(xine_t *this, FILE *fp, xine_sarray_t *plugins) {
           sscanf(value," %d",&i);
           if( i == CACHE_CATALOG_VERSION )
             version_ok = 1;
-          else
+          else {
+            free(line);
             return;
+          }
         }
       } else if (node) {
         if( !strcmp("size",line) ) {
-          sscanf(value," %" SCNu64,&llu);
-          file->filesize = (off_t) llu;
+          if (file) {
+            sscanf(value," %" SCNu64,&llu);
+            file->filesize = (off_t) llu;
+          }
         } else if( !strcmp("mtime",line) ) {
-          sscanf(value," %" SCNu64,&llu);
-          file->filemtime = (time_t) llu;
+          if (file) {
+            sscanf(value," %" SCNu64,&llu);
+            file->filemtime = (time_t) llu;
+          }
         } else if( !strcmp("type",line) ) {
           sscanf(value," %d",&i);
           node->info->type = i;
@@ -1283,6 +1346,11 @@ void _x_scan_plugins (xine_t *this) {
   this->plugin_catalog = _new_catalog();
   XINE_PROFILE(load_cached_catalog (this));
 
+#ifdef XINE_MAKE_BUILTINS
+  lprintf ("collect_plugins in libxine\n");
+  _register_plugins_internal (this, NULL, NULL , xine_builtin_plugin_info);
+#endif
+
   if ((pluginpath = getenv("XINE_PLUGIN_PATH")) != NULL && *pluginpath) {
     char *p = pluginpath;
     while (p && p[0])
@@ -1326,7 +1394,7 @@ void _x_scan_plugins (xine_t *this) {
 
   load_required_plugins (this);
 
-  if ((_x_flags & XINE_FLAG_NO_WRITE_CACHE) == 0)
+  if ((this->flags & XINE_FLAG_NO_WRITE_CACHE) == 0)
     XINE_PROFILE(save_catalog (this));
 
   map_decoders (this);
@@ -1611,17 +1679,18 @@ demux_plugin_t *_x_find_demux_plugin_last_probe(xine_stream_t *stream, const cha
 
   stream->content_detection_method = METHOD_BY_CONTENT;
 
-  if (!last_demux->plugin_class && !_load_plugin_class(xine, last_demux, NULL))
-    return NULL;
+  pthread_mutex_lock (&catalog->lock);
+  if (last_demux->plugin_class || _load_plugin_class(xine, last_demux, NULL)) {
 
   if ((plugin = ((demux_class_t *)last_demux->plugin_class)->open_plugin(last_demux->plugin_class, stream, input))) {
     xprintf (stream->xine, XINE_VERBOSITY_LOG, _("load_plugins: using demuxer '%s'\n"), last_demux_name);
     inc_node_ref(last_demux);
     plugin->node = last_demux;
-    return plugin;
   }
+  }
+  pthread_mutex_unlock (&catalog->lock);
 
-  return NULL;
+  return plugin;
 }
 
 
@@ -2569,7 +2638,6 @@ xine_post_t *xine_post_init(xine_t *xine, const char *name, int inputs,
 	i = 0;
 	ite = xine_list_front(post->input);
 	while (ite) {
-	  input = xine_list_get_value (post->input, ite);
 	  i++;
 	  ite = xine_list_next (post->input, ite);
 	}
@@ -2586,7 +2654,6 @@ xine_post_t *xine_post_init(xine_t *xine, const char *name, int inputs,
 	i = 0;
 	ite = xine_list_front (post->output);
 	while (ite) {
-	  output = xine_list_get_value (post->output, ite);
 	  i++;
 	  ite = xine_list_next (post->output, ite);
 	}
