@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -50,8 +50,10 @@ static void XINE_FORMAT_PRINTF(2, 0)
 
   if ( ! this->lines[this->cur] )
     this->lines[this->cur] = malloc(SCRATCH_LINE_LEN_MAX+1);
-  if ( ! this->lines[this->cur] )
+  if ( ! this->lines[this->cur] ) {
+    pthread_mutex_unlock (&this->lock);
     return;
+  }
 
   l = strftime (this->lines[this->cur], SCRATCH_LINE_LEN_MAX, "%X: ", &tm);
   vsnprintf (this->lines[this->cur] + l, SCRATCH_LINE_LEN_MAX - l, format, argp);
