@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -52,7 +52,8 @@
 #include <yuv2rgb.h>
 
 #define NUM_FRAME_BUFFERS          15
-#define MAX_USEC_TO_SLEEP       20000
+/* 24/25/30 fps are most common, do these in a single wait */
+#define MAX_USEC_TO_SLEEP       42000
 #define DEFAULT_FRAME_DURATION   3000    /* 30 frames per second */
 
 /* wait this delay if the first frame is still referenced */
@@ -107,8 +108,10 @@ typedef struct {
   xine_list_t              *streams;
   pthread_mutex_t           streams_lock;
 
-  img_buf_fifo_t           *free_img_buf_queue;
-  img_buf_fifo_t           *display_img_buf_queue;
+  img_buf_fifo_t            free_img_buf_queue;
+  img_buf_fifo_t            display_img_buf_queue;
+
+  pthread_cond_t            done_flushing;
 
   vo_frame_t               *img_backup;
 
@@ -159,6 +162,11 @@ typedef struct {
   pthread_mutex_t           trigger_drawing_mutex;
   pthread_cond_t            trigger_drawing_cond;
   int                       trigger_drawing;
+
+  /* frames usage stats */
+  int                       frames_total;
+  int                       frames_extref;
+  int                       frames_peak_used;
 } vos_t;
 
 
@@ -166,22 +174,26 @@ typedef struct {
  * frame queue (fifo) util functions
  */
 
-static img_buf_fifo_t *XINE_MALLOC vo_new_img_buf_queue () {
+static void vo_open_img_buf_queue (img_buf_fifo_t *queue) {
+  queue->first           = NULL;
+  queue->last            = NULL;
+  queue->num_buffers     = 0;
+  queue->num_buffers_max = 0;
 
-  img_buf_fifo_t *queue;
+  queue->locked_for_read = 0;
+  pthread_mutex_init (&queue->mutex, NULL);
+  pthread_cond_init  (&queue->not_empty, NULL);
+}
 
-  queue = (img_buf_fifo_t *) calloc(1, sizeof(img_buf_fifo_t));
-  if( queue ) {
-    queue->first           = NULL;
-    queue->last            = NULL;
-    queue->num_buffers     = 0;
-    queue->num_buffers_max = 0;
+static void vo_close_img_buf_queue (img_buf_fifo_t *queue) {
+  queue->first           = NULL;
+  queue->last            = NULL;
+  queue->num_buffers     = 0;
+  queue->num_buffers_max = 0;
 
-    queue->locked_for_read = 0;
-    pthread_mutex_init (&queue->mutex, NULL);
-    pthread_cond_init  (&queue->not_empty, NULL);
-  }
-  return queue;
+  queue->locked_for_read = 0;
+  pthread_mutex_destroy (&queue->mutex);
+  pthread_cond_destroy  (&queue->not_empty);
 }
 
 static void vo_append_to_img_buf_queue_int (img_buf_fifo_t *queue,
@@ -303,6 +315,22 @@ static vo_frame_t *vo_remove_from_img_buf_queue_int (img_buf_fifo_t *queue, int
   return img;
 }
 
+static vo_frame_t *vo_pop_from_img_buf_queue_int (img_buf_fifo_t *queue) {
+  vo_frame_t *img;
+
+  img = queue->first;
+  queue->first = img->next;
+  img->next = NULL;
+  if (!queue->first) {
+    queue->last = NULL;
+    queue->num_buffers = 0;
+  } else {
+    queue->num_buffers--;
+  }
+
+  return img;
+}
+
 static vo_frame_t *vo_remove_from_img_buf_queue (img_buf_fifo_t *queue) {
   vo_frame_t *img;
 
@@ -334,6 +362,11 @@ static void vo_frame_inc_lock (vo_frame_t *img) {
   pthread_mutex_lock (&img->mutex);
 
   img->lock_counter++;
+  if (img->lock_counter == 2) {
+    vos_t *this = (vos_t *)img->port;
+    if (this->frames_extref < this->frames_total)
+      this->frames_extref++;
+  }
 
   pthread_mutex_unlock (&img->mutex);
 }
@@ -343,11 +376,16 @@ static void vo_frame_dec_lock (vo_frame_t *img) {
   pthread_mutex_lock (&img->mutex);
 
   img->lock_counter--;
+  if (img->lock_counter == 1) {
+    vos_t *this = (vos_t *)img->port;
+    if (this->frames_extref > 0)
+      this->frames_extref--;
+  } else
   if (!img->lock_counter) {
     vos_t *this = (vos_t *) img->port;
     if (img->stream)
       _x_refcounter_dec(img->stream->refcounter);
-    vo_append_to_img_buf_queue (this->free_img_buf_queue, img);
+    vo_append_to_img_buf_queue (&this->free_img_buf_queue, img);
   }
 
   pthread_mutex_unlock (&img->mutex);
@@ -716,7 +754,7 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
 
   while (1) {
 
-    while (!(img = vo_remove_from_img_buf_queue_nonblock (this->free_img_buf_queue,
+    while (!(img = vo_remove_from_img_buf_queue_nonblock (&this->free_img_buf_queue,
                    width, height, ratio, format, flags)))
       if (this->xine->port_ticket->ticket_revoked)
         this->xine->port_ticket->renew(this->xine->port_ticket, 1);
@@ -762,10 +800,21 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
     xprintf (this->xine, XINE_VERBOSITY_LOG,
       _("video_out: found an unusable frame (%dx%d, format %0x08x) - no memory??\n"),
       width, height, img->format);
-    vo_append_to_img_buf_queue (this->free_img_buf_queue, img);
+    vo_append_to_img_buf_queue (&this->free_img_buf_queue, img);
 
   }
 
+  /* update frame usage stats. No need to lock queues for that I guess :-) */
+  {
+    int frames_used;
+    frames_used = this->frames_total;
+    frames_used -= this->free_img_buf_queue.num_buffers;
+    frames_used -= this->display_img_buf_queue.num_buffers;
+    frames_used += this->frames_extref;
+    if (frames_used > this->frames_peak_used)
+      this->frames_peak_used = frames_used;
+  }
+
   lprintf ("get_frame (%d x %d) done\n", width, height);
 
   return img;
@@ -837,7 +886,7 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
     frames_to_skip = ((-1 * diff) / duration + this->frame_drop_limit) * 2;
 
     /* do not skip decoding until output fifo frames are consumed */
-    if (this->display_img_buf_queue->num_buffers >= this->frame_drop_limit ||
+    if (this->display_img_buf_queue.num_buffers >= this->frame_drop_limit ||
         frames_to_skip < 0)
       frames_to_skip = 0;
 
@@ -939,7 +988,7 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
 
     if (!img_already_locked)
       vo_frame_inc_lock( img );
-    vo_append_to_img_buf_queue (this->display_img_buf_queue, img);
+    vo_append_to_img_buf_queue (&this->display_img_buf_queue, img);
 
   } else {
     lprintf ("bad_frame\n");
@@ -1042,18 +1091,18 @@ static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
 
   vo_frame_t *dupl;
 
-  if( !this->free_img_buf_queue->first)
+  if( !this->free_img_buf_queue.first)
     return NULL;
 
-  dupl = this->free_img_buf_queue->first;
-  this->free_img_buf_queue->first = dupl->next;
+  dupl = this->free_img_buf_queue.first;
+  this->free_img_buf_queue.first = dupl->next;
   dupl->next = NULL;
-  if (!this->free_img_buf_queue->first) {
-    this->free_img_buf_queue->last = NULL;
-    this->free_img_buf_queue->num_buffers = 0;
+  if (!this->free_img_buf_queue.first) {
+    this->free_img_buf_queue.last = NULL;
+    this->free_img_buf_queue.num_buffers = 0;
   }
   else {
-    this->free_img_buf_queue->num_buffers--;
+    this->free_img_buf_queue.num_buffers--;
   }
 
   pthread_mutex_lock (&dupl->mutex);
@@ -1080,14 +1129,14 @@ static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
 
   if (img->width && !dupl->width) {
     /* driver failed to set up render space */
-    if (this->free_img_buf_queue->last)
-      this->free_img_buf_queue->last->next = dupl;
-    this->free_img_buf_queue->last = dupl;
-    if (this->free_img_buf_queue->first)
-      this->free_img_buf_queue->num_buffers++;
+    if (this->free_img_buf_queue.last)
+      this->free_img_buf_queue.last->next = dupl;
+    this->free_img_buf_queue.last = dupl;
+    if (this->free_img_buf_queue.first)
+      this->free_img_buf_queue.num_buffers++;
     else {
-      this->free_img_buf_queue->first = dupl;
-      this->free_img_buf_queue->num_buffers = 1;
+      this->free_img_buf_queue.first = dupl;
+      this->free_img_buf_queue.num_buffers = 1;
     }
     return NULL;
   }
@@ -1141,29 +1190,59 @@ static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
 }
 
 
-static void expire_frames (vos_t *this, int64_t cur_vpts) {
+static vo_frame_t *next_frame (vos_t *this, int64_t *vpts) {
 
   int64_t       pts;
   int64_t       diff;
   vo_frame_t   *img;
   int           duration;
 
-  pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+  pthread_mutex_lock(&this->display_img_buf_queue.mutex);
 
-  img = this->display_img_buf_queue->first;
+  img = this->display_img_buf_queue.first;
 
-  /*
-   * throw away expired frames
-   */
-
-  diff = 1000000; /* always enter the while-loop */
-  duration = 0;
+  /* when flushing, drop everything now, and return latest "first" frame if any */
+  if (this->discard_frames) {
+    vo_frame_t *first_frame = NULL, *last_frame = NULL;
+    while (img) {
+      img = vo_pop_from_img_buf_queue_int (&this->display_img_buf_queue);
+      if (img->is_first != 0) {
+        if (first_frame)
+          vo_frame_dec_lock (first_frame);
+        first_frame = img;
+      } else {
+        if (last_frame)
+          vo_frame_dec_lock (last_frame);
+        last_frame = img;
+      }
+      img = this->display_img_buf_queue.first;
+    }
+    pthread_mutex_unlock (&this->display_img_buf_queue.mutex);
+    pthread_cond_broadcast (&this->done_flushing);
+    if (first_frame) {
+      first_frame->vpts = *vpts;
+      first_frame->future_frame = NULL;
+      if (last_frame)
+        vo_frame_dec_lock (last_frame);
+      if (this->img_backup)
+        vo_frame_dec_lock (this->img_backup);
+      vo_frame_inc_lock (first_frame);
+      this->img_backup = first_frame;
+      return first_frame;
+    }
+    if (last_frame) {
+      if (this->img_backup)
+        vo_frame_dec_lock (this->img_backup);
+      this->img_backup = last_frame;
+    }
+    *vpts = 0;
+    return NULL;
+  }
 
   while (img) {
 
     if (img->is_first > 0) {
       lprintf("expire_frames: first_frame !\n");
-
       /*
        * before displaying the first frame without
        * "metronom prebuffering" we should make sure it's
@@ -1171,15 +1250,16 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
        */
       if( img->lock_counter == 1 ) {
         /* display it immediately */
-        img->vpts = cur_vpts;
+        img->vpts = *vpts;
       } else {
         /* poll */
-        lprintf("frame still referenced %d times, is_first=%d\n", img->lock_counter, img->is_first);
-        img->vpts = cur_vpts + FIRST_FRAME_POLL_DELAY;
+        lprintf ("frame still referenced %d times, is_first=%d\n", img->lock_counter, img->is_first);
+        img->vpts = *vpts + FIRST_FRAME_POLL_DELAY;
       }
       img->is_first--;
       /* make sure to wake up xine_play even if this first frame gets discarded */
-      if (img->is_first == 0) img->is_first = -1;
+      if (img->is_first == 0)
+        img->is_first = -1;
       break;
     }
 
@@ -1192,196 +1272,104 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
       duration = img->duration;
 
     pts = img->vpts;
-    diff = cur_vpts - pts;
+    diff = *vpts - pts;
 
-    if (diff > duration || this->discard_frames) {
-
-      if( !this->discard_frames ) {
-        xine_log(this->xine, XINE_LOG_MSG,
-	         _("video_out: throwing away image with pts %" PRId64 " because it's too old (diff : %" PRId64 ").\n"), pts, diff);
-
-        this->num_frames_discarded++;
-      }
-
-      img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
-
-      if (img->stream) {
-	pthread_mutex_lock( &img->stream->current_extra_info_lock );
-	_x_extra_info_merge( img->stream->current_extra_info, img->extra_info );
-	pthread_mutex_unlock( &img->stream->current_extra_info_lock );
-	/* wake up xine_play now if we just discarded first frame */
-	if (img->is_first != 0) {
-	  xine_list_iterator_t ite;
-	  pthread_mutex_lock (&this->streams_lock);
-	  for (ite = xine_list_front(this->streams); ite;
-	    ite = xine_list_next(this->streams, ite)) {
-	    xine_stream_t *stream = xine_list_get_value (this->streams, ite);
-	    if (stream == XINE_ANON_STREAM) continue;
-	    pthread_mutex_lock (&stream->first_frame_lock);
-	    if (stream->first_frame_flag) {
-	      stream->first_frame_flag = 0;
-	      pthread_cond_broadcast (&stream->first_frame_reached);
-	    }
-	    pthread_mutex_unlock (&stream->first_frame_lock);
-	  }
-	  pthread_mutex_unlock(&this->streams_lock);
-	  xine_log (this->xine, XINE_LOG_MSG, _("video_out: just discarded first frame after seek\n"));
-	}
-      }
-
-      /* when flushing frames, keep the first one as backup */
-      if( this->discard_frames ) {
-
-        if (!this->img_backup) {
-	  this->img_backup = img;
-        } else {
-	  vo_frame_dec_lock( img );
-        }
-
-      } else {
-        /*
-         * last frame? back it up for
-         * still frame creation
-         */
-
-        if (!this->display_img_buf_queue->first) {
-
-	  if (this->img_backup) {
-	    lprintf("overwriting frame backup\n");
-
-	    vo_frame_dec_lock( this->img_backup );
-	  }
-
-	  lprintf("possible still frame (old)\n");
-
-	  this->img_backup = img;
-
-	  /* wait 4 frames before drawing this one.
-	     this allow slower systems to recover. */
-	  this->redraw_needed = 4;
-        } else {
-	  vo_frame_dec_lock( img );
-        }
-      }
-      img = this->display_img_buf_queue->first;
-
-    } else
+    if (diff <= duration)
       break;
+
+    xine_log (this->xine, XINE_LOG_MSG,
+      _("video_out: throwing away image with pts %" PRId64 " because it's too old (diff : %" PRId64 ").\n"),
+      pts, diff);
+
+    this->num_frames_discarded++;
+
+    img = vo_pop_from_img_buf_queue_int (&this->display_img_buf_queue);
+
+    if (img->stream) {
+      pthread_mutex_lock (&img->stream->current_extra_info_lock);
+      _x_extra_info_merge (img->stream->current_extra_info, img->extra_info);
+      pthread_mutex_unlock (&img->stream->current_extra_info_lock);
+      /* wake up xine_play now if we just discarded first frame */
+      if (img->is_first != 0) {
+        xine_list_iterator_t ite;
+        pthread_mutex_lock (&this->streams_lock);
+        for (ite = xine_list_front(this->streams); ite;
+          ite = xine_list_next(this->streams, ite)) {
+          xine_stream_t *stream = xine_list_get_value (this->streams, ite);
+          if (stream == XINE_ANON_STREAM)
+            continue;
+          pthread_mutex_lock (&stream->first_frame_lock);
+          if (stream->first_frame_flag) {
+            stream->first_frame_flag = 0;
+            pthread_cond_broadcast (&stream->first_frame_reached);
+          }
+          pthread_mutex_unlock (&stream->first_frame_lock);
+        }
+        pthread_mutex_unlock (&this->streams_lock);
+        xine_log (this->xine, XINE_LOG_MSG, _("video_out: just discarded first frame after seek\n"));
+      }
+    }
+
+    /* last frame? back it up for still frame creation */
+    if (!this->display_img_buf_queue.first) {
+      if (this->img_backup) {
+        lprintf ("overwriting frame backup\n");
+        vo_frame_dec_lock (this->img_backup);
+      }
+      lprintf ("possible still frame (old)\n");
+      this->img_backup = img;
+      /* wait 4 frames before drawing this one. this allow slower systems to recover. */
+      this->redraw_needed = 4;
+    } else {
+      vo_frame_dec_lock (img);
+    }
+
+    img = this->display_img_buf_queue.first;
   }
 
-  pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
-}
-
-/* If it's not the time to display the next frame,
- * the vpts of the next frame (if any) is returned, 0 otherwise.
- */
-static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts,
-                                   int64_t *next_frame_vpts) {
-
-  vo_frame_t   *img;
-
-  pthread_mutex_lock(&this->display_img_buf_queue->mutex);
-
-  img = this->display_img_buf_queue->first;
-
-  *next_frame_vpts = 0;
-
-  /*
-   * still frame detection:
-   */
-
-  /* no frame? => still frame detection */
-
-  if (!img) {
-
-    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
-
-    lprintf ("no frame\n");
-
-    if (this->img_backup && (this->redraw_needed==1)) {
-
-      lprintf("generating still frame (cur_vpts = %" PRId64 ") \n", cur_vpts);
-
-      /* keep playing still frames */
-      pthread_mutex_lock( &this->free_img_buf_queue->mutex );
-      img = duplicate_frame (this, this->img_backup );
-      pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
-      if( img ) {
-        img->vpts = cur_vpts;
-        /* extra info of the backup is thrown away, because it is not up to date */
-        _x_extra_info_reset(img->extra_info);
-        img->future_frame = NULL;
-      }
-      return img;
-
-    } else {
-
-      if( this->redraw_needed )
-        this->redraw_needed--;
-
-      lprintf ("no frame, but no backup frame\n");
-
-      return NULL;
-    }
-  } else {
-
-    int64_t diff;
-
-    diff = cur_vpts - img->vpts;
-
-    /*
-     * time to display frame "img" ?
-     */
-
+  if (img) {
+    diff = *vpts - img->vpts;
+    /* time to display frame "img" ? */
     lprintf ("diff %" PRId64 "\n", diff);
-
     if (diff < 0) {
-      *next_frame_vpts = img->vpts;
-      pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+      *vpts = img->vpts;
+      pthread_mutex_unlock (&this->display_img_buf_queue.mutex);
       return NULL;
     }
-
-    if (this->img_backup) {
-      lprintf("freeing frame backup\n");
-
-      vo_frame_dec_lock( this->img_backup );
-      this->img_backup = NULL;
-    }
-
-    /*
-     * last frame? make backup for possible still image
-     */
-    pthread_mutex_lock( &this->free_img_buf_queue->mutex );
-    if (img && !img->next) {
-
-      if (!img->stream ||
-          _x_stream_info_get(img->stream, XINE_STREAM_INFO_VIDEO_HAS_STILL) ||
-          !img->stream->video_fifo ||
-          img->stream->video_fifo->size(img->stream->video_fifo) < 10) {
-
-        lprintf ("possible still frame\n");
-
-        this->img_backup = duplicate_frame (this, img);
-      }
-    }
-    pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
-
-    /*
-     * remove frame from display queue and show it
-     */
-
-    if ( img ) {
-      if ( img->next )
-        img->future_frame = img->next;
-      else
-        img->future_frame = NULL;
-    }
-    
-    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
-    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
-
+    /* remove frame from display queue and show it */
+    img->future_frame = img->next;
+    img = vo_pop_from_img_buf_queue_int (&this->display_img_buf_queue);
+    pthread_mutex_unlock (&this->display_img_buf_queue.mutex);
+    /* reuse as still frame */
+    vo_frame_inc_lock (img);
+    if (this->img_backup)
+      vo_frame_dec_lock (this->img_backup);
+    this->img_backup = img;
     return img;
   }
+    
+  pthread_mutex_unlock (&this->display_img_buf_queue.mutex);
+  lprintf ("no frame\n");
+  if (this->img_backup && (this->redraw_needed == 1)) {
+    lprintf ("generating still frame (vpts = %" PRId64 ") \n", *vpts);
+    /* keep playing still frames */
+    pthread_mutex_lock (&this->free_img_buf_queue.mutex);
+    img = duplicate_frame (this, this->img_backup);
+    pthread_mutex_unlock (&this->free_img_buf_queue.mutex);
+    if (img) {
+      img->vpts = *vpts;
+      /* extra info of the backup is thrown away, because it is not up to date */
+      _x_extra_info_reset (img->extra_info);
+      img->future_frame = NULL;
+    }
+    return img;
+  } else {
+    if (this->redraw_needed)
+      this->redraw_needed--;
+    lprintf ("no frame, but no backup frame\n");
+    *vpts = 0;
+    return NULL;
+  }
 }
 
 static void overlay_and_display_frame (vos_t *this,
@@ -1426,6 +1414,15 @@ static void overlay_and_display_frame (vos_t *this,
     pthread_mutex_unlock(&this->streams_lock);
   }
 
+  /* calling the frontend's frame output hook (via driver->display_frame () here)
+   * while flushing (xine_stop ()) may freeze.
+   */
+  if (this->discard_frames && this->display_img_buf_queue.first) {
+    img->free (img);
+    this->redraw_needed = 0;
+    return;
+  }
+
   if (this->overlay_source) {
     this->overlay_source->multiple_overlay_blend (this->overlay_source,
 						  vpts,
@@ -1447,6 +1444,12 @@ static void check_redraw_needed (vos_t *this, int64_t vpts) {
       this->redraw_needed = 1;
   }
 
+  /* calling the frontend's frame output hook (via driver->redraw_needed () here)
+   * while flushing (xine_stop ()) may freeze.
+   */
+  if (this->discard_frames && this->display_img_buf_queue.first)
+    return;
+
   if( this->driver->redraw_needed (this->driver) )
     this->redraw_needed = 1;
 }
@@ -1484,47 +1487,47 @@ static vo_frame_t *force_duplicate_frame (vos_t *this, vo_frame_t *s) {
   if (!s)
     return NULL;
 
-  pthread_mutex_lock (&this->free_img_buf_queue->mutex);
-  img = this->free_img_buf_queue->first;
+  pthread_mutex_lock (&this->free_img_buf_queue.mutex);
+  img = this->free_img_buf_queue.first;
 
   if (!img) {
     /* OK we run out of free frames. Try to whistle back a frame already waiting for display.
        Search for one that is _not_ a DR1 reference frame that the decoder wants unchanged */
-    pthread_mutex_lock (&this->display_img_buf_queue->mutex);
-    for (img = this->display_img_buf_queue->first; img; img = img->next) {
+    pthread_mutex_lock (&this->display_img_buf_queue.mutex);
+    for (img = this->display_img_buf_queue.first; img; img = img->next) {
       if (img->lock_counter == 1) break;
       prev = img;
     }
 
     if (img) {
-      if (img == this->display_img_buf_queue->first)
-        this->display_img_buf_queue->first = img->next;
+      if (img == this->display_img_buf_queue.first)
+        this->display_img_buf_queue.first = img->next;
       if (prev)
         prev->next = img->next;
-      if (img == this->display_img_buf_queue->last)
-        this->display_img_buf_queue->last = prev;
-      if (!this->display_img_buf_queue->first)
-        this->display_img_buf_queue->num_buffers = 0;
+      if (img == this->display_img_buf_queue.last)
+        this->display_img_buf_queue.last = prev;
+      if (!this->display_img_buf_queue.first)
+        this->display_img_buf_queue.num_buffers = 0;
       else
-        this->display_img_buf_queue->num_buffers--;
+        this->display_img_buf_queue.num_buffers--;
 
       img->next = NULL;
 
       if (img != s) {
         /* Now put it into free queue where it gets taken from next */
-        this->free_img_buf_queue->first = img;
-        this->free_img_buf_queue->last  = img;
-        this->free_img_buf_queue->num_buffers = 1;
+        this->free_img_buf_queue.first = img;
+        this->free_img_buf_queue.last  = img;
+        this->free_img_buf_queue.num_buffers = 1;
         img->lock_counter = 0;
       }
     }
-    pthread_mutex_unlock (&this->display_img_buf_queue->mutex);
+    pthread_mutex_unlock (&this->display_img_buf_queue.mutex);
   }
 
   if (img != s)
     img = duplicate_frame (this, s);
 
-  pthread_mutex_unlock (&this->free_img_buf_queue->mutex);
+  pthread_mutex_unlock (&this->free_img_buf_queue.mutex);
 
   return img;
 }
@@ -1538,16 +1541,21 @@ static void paused_loop( vos_t *this, int64_t vpts )
   vo_frame_t   *img;
 
   /* prevent decoder thread from allocating new frames */
-  pthread_mutex_lock (&this->free_img_buf_queue->mutex);
-  this->free_img_buf_queue->locked_for_read = 1;
-  pthread_mutex_unlock (&this->free_img_buf_queue->mutex);
+  pthread_mutex_lock (&this->free_img_buf_queue.mutex);
+  this->free_img_buf_queue.locked_for_read = 1;
+  pthread_mutex_unlock (&this->free_img_buf_queue.mutex);
 
   while (this->clock->speed == XINE_SPEED_PAUSE && this->video_loop_running) {
 
     /* set img_backup to play the same frame several times */
     if (!this->img_backup) {
-      this->img_backup = force_duplicate_frame (this, this->display_img_buf_queue->first);
-      this->redraw_needed = 1;
+      vo_frame_t *f;
+      f = this->display_img_buf_queue.first;
+      if (f) {
+        vo_frame_inc_lock (f);
+        this->img_backup = f;
+        this->redraw_needed = 1;
+      }
     }
 
     check_redraw_needed( this, vpts );
@@ -1562,11 +1570,15 @@ static void paused_loop( vos_t *this, int64_t vpts )
     }
 
     interruptable_sleep(this, 20000);
+
+    /* dont let folks wait forever in vain */
+    if (this->discard_frames)
+      pthread_cond_broadcast (&this->done_flushing);
   }
 
-  pthread_mutex_lock (&this->free_img_buf_queue->mutex);
-  this->free_img_buf_queue->locked_for_read = 0;
-  pthread_mutex_unlock (&this->free_img_buf_queue->mutex);
+  pthread_mutex_lock (&this->free_img_buf_queue.mutex);
+  this->free_img_buf_queue.locked_for_read = 0;
+  pthread_mutex_unlock (&this->free_img_buf_queue.mutex);
 }
 
 static void video_out_update_disable_flush_from_video_out(void *disable_decoder_flush_from_video_out, xine_cfg_entry_t *entry)
@@ -1607,18 +1619,18 @@ static void *video_out_loop (void *this_gen) {
   lprintf ("loop starting...\n");
 
   while ( this->video_loop_running ) {
-
-    /*
-     * get current time and find frame to display
+    struct timeval now;
+    /* record current time as both speed dependent virtual presentation timestamp (vpts)
+     * and absolute system time, and hope these are halfway in sync.
      */
-
-    vpts = this->clock->get_current_time (this->clock);
-
+    vpts = next_frame_vpts = this->clock->get_current_time (this->clock);
+    gettimeofday (&now, NULL);
     lprintf ("loop iteration at %" PRId64 "\n", vpts);
 
-    expire_frames (this, vpts);
-
-    img = get_next_frame (this, vpts, &next_frame_vpts);
+    /*
+     * find frame to display
+     */
+    img = next_frame (this, &next_frame_vpts);
 
     /*
      * if we have found a frame, display it
@@ -1641,7 +1653,7 @@ static void *video_out_loop (void *this_gen) {
      */
 
     diff = vpts - this->last_delivery_pts;
-    if (diff > 30000 && !this->display_img_buf_queue->first) {
+    if (diff > 30000 && !this->display_img_buf_queue.first) {
       xine_list_iterator_t ite;
 
       pthread_mutex_lock(&this->streams_lock);
@@ -1672,61 +1684,81 @@ static void *video_out_loop (void *this_gen) {
     if (img) {
       next_frame_vpts = img->vpts + img->duration;
     }
-    /* else next_frame_vpts is returned by get_next_frame */
+    /* else next_frame_vpts is returned by next_frame */
 
     lprintf ("next_frame_vpts is %" PRId64 "\n", next_frame_vpts);
+    if ((next_frame_vpts - vpts) > 2 * 90000)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+      "video_out: vpts/clock error, next_vpts=%" PRId64 " cur_vpts=%" PRId64 "\n", next_frame_vpts, vpts);
 
-    do {
-      vpts = this->clock->get_current_time (this->clock);
+    /* get diff time for next iteration */
+    if (next_frame_vpts && this->clock->speed > 0)
+      usec_to_sleep = (next_frame_vpts - vpts) * 100 * XINE_FINE_SPEED_NORMAL / (9 * this->clock->speed);
+    else
+      /* we don't know when the next frame is due, only wait a little */
+      usec_to_sleep = 20000;
 
-      if (this->clock->speed == XINE_SPEED_PAUSE)
-        paused_loop (this, vpts);
-
-      if (next_frame_vpts && this->clock->speed > 0) {
-        usec_to_sleep = (next_frame_vpts - vpts) * 100 * XINE_FINE_SPEED_NORMAL / (9 * this->clock->speed);
-      } else {
-        /* we don't know when the next frame is due, only wait a little */
-        usec_to_sleep = 1000;
-        next_frame_vpts = vpts; /* wait only once */
-      }
-
-      /* limit usec_to_sleep to maintain responsiveness */
-      if (usec_to_sleep > MAX_USEC_TO_SLEEP)
-        usec_to_sleep = MAX_USEC_TO_SLEEP;
-
-      lprintf ("%" PRId64 " usec to sleep at master vpts %" PRId64 "\n", usec_to_sleep, vpts);
-
-      if ( (next_frame_vpts - vpts) > 2*90000 )
-        xprintf(this->xine, XINE_VERBOSITY_DEBUG,
-		"video_out: vpts/clock error, next_vpts=%" PRId64 " cur_vpts=%" PRId64 "\n", next_frame_vpts,vpts);
-
-      if (usec_to_sleep > 0)
-      {
-        /* honor trigger update only when a backup img is available */
-        if (0 == interruptable_sleep(this, usec_to_sleep) && this->img_backup)
-          break;
-      }
+    while (this->video_loop_running) {
+      int timedout, wait;
 
       if (this->discard_frames)
         break;
 
-    } while ( (usec_to_sleep > 0) && this->video_loop_running);
+      if (this->clock->speed == XINE_SPEED_PAUSE) {
+        paused_loop (this, vpts);
+        break;
+      }
+
+      /* limit usec_to_sleep to maintain responsiveness */
+      wait = usec_to_sleep;
+      if (wait <= 0)
+        break;
+      if (wait > MAX_USEC_TO_SLEEP)
+        wait = MAX_USEC_TO_SLEEP;
+
+      lprintf ("%d usec to sleep at master vpts %" PRId64 "\n", wait, vpts);
+
+      /* next stop absolute time */
+      now.tv_usec += wait;
+      if (now.tv_usec >= 1000000) {
+        now.tv_sec++;
+        now.tv_usec -= 1000000;
+      }
+      usec_to_sleep -= wait;
+
+      timedout = 0;
+      pthread_mutex_lock (&this->trigger_drawing_mutex);
+      if (!this->trigger_drawing) {
+        struct timespec abstime;
+        abstime.tv_sec = now.tv_sec;
+        abstime.tv_nsec = now.tv_usec * 1000;
+        timedout = pthread_cond_timedwait (&this->trigger_drawing_cond, &this->trigger_drawing_mutex, &abstime);
+      }
+      this->trigger_drawing = 0;
+      pthread_mutex_unlock (&this->trigger_drawing_mutex);
+      /* honor trigger update only when a backup img is available */
+      if (!timedout && this->img_backup)
+        break;
+    }
   }
 
   /*
    * throw away undisplayed frames
    */
 
-  pthread_mutex_lock(&this->display_img_buf_queue->mutex);
-  img = this->display_img_buf_queue->first;
+  pthread_mutex_lock(&this->display_img_buf_queue.mutex);
+  img = this->display_img_buf_queue.first;
   while (img) {
 
-    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
+    img = vo_pop_from_img_buf_queue_int (&this->display_img_buf_queue);
     vo_frame_dec_lock( img );
 
-    img = this->display_img_buf_queue->first;
+    img = this->display_img_buf_queue.first;
   }
-  pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+  pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
+  /* dont let folks wait forever in vain */
+  if (this->discard_frames)
+    pthread_cond_broadcast (&this->done_flushing);
 
   if (this->img_backup) {
     vo_frame_dec_lock( this->img_backup );
@@ -1765,10 +1797,10 @@ int xine_get_next_video_frame (xine_video_port_t *this_gen,
 
     /* FIXME: ugly, use conditions and locks instead? */
 
-    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
-    img = this->display_img_buf_queue->first;
+    pthread_mutex_lock(&this->display_img_buf_queue.mutex);
+    img = this->display_img_buf_queue.first;
     if (!img) {
-      pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+      pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
       if (stream != XINE_ANON_STREAM && stream->video_fifo->fifo_size == 0 &&
           stream->demux_plugin->get_status(stream->demux_plugin) != DEMUX_OK)
         /* no further data can be expected here */
@@ -1782,8 +1814,8 @@ int xine_get_next_video_frame (xine_video_port_t *this_gen,
    * remove frame from display queue and show it
    */
 
-  img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
-  pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+  img = vo_remove_from_img_buf_queue_int (&this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
+  pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
 
   frame->vpts         = img->vpts;
   frame->duration     = img->duration;
@@ -1867,15 +1899,15 @@ static int vo_get_property (xine_video_port_t *this_gen, int property) {
     break;
 
   case VO_PROP_BUFS_IN_FIFO:
-    ret = this->video_loop_running ? this->display_img_buf_queue->num_buffers : -1;
+    ret = this->video_loop_running ? this->display_img_buf_queue.num_buffers : -1;
     break;
 
   case VO_PROP_BUFS_FREE:
-    ret = this->video_loop_running ? this->free_img_buf_queue->num_buffers : -1;
+    ret = this->video_loop_running ? this->free_img_buf_queue.num_buffers : -1;
     break;
 
   case VO_PROP_BUFS_TOTAL:
-    ret = this->video_loop_running ? this->free_img_buf_queue->num_buffers_max : -1;
+    ret = this->video_loop_running ? this->free_img_buf_queue.num_buffers_max : -1;
     break;
 
   case VO_PROP_NUM_STREAMS:
@@ -1943,7 +1975,7 @@ static int vo_set_property (xine_video_port_t *this_gen, int property, int value
 
   case VO_PROP_DISCARD_FRAMES:
     /* recursive discard frames setting */
-    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+    pthread_mutex_lock(&this->display_img_buf_queue.mutex);
     if(value)
       this->discard_frames++;
     else if (this->discard_frames)
@@ -1951,24 +1983,24 @@ static int vo_set_property (xine_video_port_t *this_gen, int property, int value
     else
       xprintf (this->xine, XINE_VERBOSITY_DEBUG,
 	       "vo_set_property: discard_frames is already zero\n");
-    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+    pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
     ret = this->discard_frames;
 
     /* discard buffers here because we have no output thread */
     if (this->grab_only && this->discard_frames) {
       vo_frame_t *img;
 
-      pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+      pthread_mutex_lock(&this->display_img_buf_queue.mutex);
 
-      while ((img = this->display_img_buf_queue->first)) {
+      while ((img = this->display_img_buf_queue.first)) {
 
         lprintf ("flushing out frame\n");
 
-        img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue, 1, 0, 0, 0, 0, 0);
+        img = vo_pop_from_img_buf_queue_int (&this->display_img_buf_queue);
 
         vo_frame_dec_lock (img);
       }
-      pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+      pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
     }
     break;
 
@@ -2066,13 +2098,18 @@ static void vo_free_img_buffers (xine_video_port_t *this_gen) {
   vos_t      *this = (vos_t *) this_gen;
   vo_frame_t *img;
 
-  while (this->free_img_buf_queue->first) {
-    img = vo_remove_from_img_buf_queue (this->free_img_buf_queue);
+  /* print frame usage stats */
+  xprintf (this->xine, XINE_VERBOSITY_LOG,
+    _("video_out: max frames used: %d of %d\n"),
+    this->frames_peak_used, this->frames_total);
+
+  while (this->free_img_buf_queue.first) {
+    img = vo_remove_from_img_buf_queue (&this->free_img_buf_queue);
     img->dispose (img);
   }
 
-  while (this->display_img_buf_queue->first) {
-    img = vo_remove_from_img_buf_queue (this->display_img_buf_queue) ;
+  while (this->display_img_buf_queue.first) {
+    img = vo_remove_from_img_buf_queue (&this->display_img_buf_queue) ;
     img->dispose (img);
   }
 
@@ -2106,8 +2143,8 @@ static void vo_exit (xine_video_port_t *this_gen) {
   xine_list_delete(this->streams);
   pthread_mutex_destroy(&this->streams_lock);
 
-  free (this->free_img_buf_queue);
-  free (this->display_img_buf_queue);
+  vo_close_img_buf_queue (&this->free_img_buf_queue);
+  vo_close_img_buf_queue (&this->display_img_buf_queue);
 
   pthread_cond_destroy(&this->trigger_drawing_cond);
   pthread_mutex_destroy(&this->trigger_drawing_mutex);
@@ -2115,6 +2152,8 @@ static void vo_exit (xine_video_port_t *this_gen) {
   pthread_mutex_destroy(&this->grab_lock);
   pthread_cond_destroy(&this->grab_cond);
 
+  pthread_cond_destroy(&this->done_flushing);
+
   free (this);
 }
 
@@ -2171,26 +2210,20 @@ static void vo_enable_overlay (xine_video_port_t *this_gen, int overlay_enabled)
  */
 static void vo_flush (xine_video_port_t *this_gen) {
   vos_t      *this = (vos_t *) this_gen;
-  vo_frame_t *img;
 
   if( this->video_loop_running ) {
-    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+    pthread_mutex_lock(&this->display_img_buf_queue.mutex);
     this->discard_frames++;
-    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+    /* wake up render thread */
+    pthread_mutex_lock (&this->trigger_drawing_mutex);
+    this->trigger_drawing = 1;
+    pthread_cond_signal (&this->trigger_drawing_cond);
+    pthread_mutex_unlock (&this->trigger_drawing_mutex);
 
-    /* do not try this in paused mode */
-    while(this->clock->speed != XINE_SPEED_PAUSE) {
-      pthread_mutex_lock(&this->display_img_buf_queue->mutex);
-      img = this->display_img_buf_queue->first;
-      pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
-      if(!img)
-        break;
-      xine_usec_sleep (20000); /* pthread_cond_t could be used here */
-    }
-
-    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+    while (this->display_img_buf_queue.first)
+      pthread_cond_wait (&this->done_flushing, &this->display_img_buf_queue.mutex);
     this->discard_frames--;
-    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+    pthread_mutex_unlock(&this->display_img_buf_queue.mutex);
   }
 }
 
@@ -2308,8 +2341,8 @@ xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabon
   this->num_frames_delivered  = 0;
   this->num_frames_skipped    = 0;
   this->num_frames_discarded  = 0;
-  this->free_img_buf_queue    = vo_new_img_buf_queue ();
-  this->display_img_buf_queue = vo_new_img_buf_queue ();
+  vo_open_img_buf_queue (&this->free_img_buf_queue);
+  vo_open_img_buf_queue (&this->display_img_buf_queue);
   this->video_loop_running    = 0;
 
   this->img_backup            = NULL;
@@ -2343,6 +2376,13 @@ xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabon
   if (num_frame_buffers<5)
     num_frame_buffers = 5;
 
+  /* init frame usage stats */
+  this->frames_total = num_frame_buffers;
+  this->frames_extref = 0;
+  this->frames_peak_used = 0;
+
+  pthread_cond_init (&this->done_flushing, NULL);
+
   /* Choose a frame_drop_limit which matches num_frame_buffers.
    * xxmc for example supplies only 8 buffers. 2 are occupied by
    * MPEG2 decoding, further 2 for displaying and the remaining 4 can
@@ -2380,7 +2420,7 @@ xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabon
 
     img->extra_info = &this->extra_info_base[i];
 
-    vo_append_to_img_buf_queue (this->free_img_buf_queue,
+    vo_append_to_img_buf_queue (&this->free_img_buf_queue,
 				img);
   }
 
@@ -2443,3 +2483,5 @@ xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabon
 
   return &this->vo;
 }
+
+
