@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2014 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -533,7 +533,8 @@ static int64_t rdtsc(int config_flags)
     __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
     return x;
   } else {
-    return times(NULL);
+    struct tms tp;
+    return times(&tp);
   }
 }
 #else
@@ -576,7 +577,7 @@ void xine_probe_fast_memcpy(xine_t *xine)
 {
   uint64_t          t;
   char             *buf1, *buf2;
-  int               i, j, best;
+  unsigned int      i, j, best;
   int               config_flags = -1;
   static const char *const memcpy_methods[] = {
     "probe", "libc",
