@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -33,6 +33,7 @@
 #include <xine/xine_internal.h>
 #endif
 #include "xine_private.h"
+#include "../xine-engine/bswap.h"
 
 #include <errno.h>
 #include <pwd.h>
@@ -241,7 +242,6 @@ static const lang_locale_t lang_locales[] = {
   { "zh_HK",    "big5-hkscs",  "big5-hkscs",  ""         },
   { "zh_TW",    "big-5",       "big-5",       ""         },
   { "zh_TW",    "euc-tw",      "euc-tw",      ""         },
-  { "" }
 };
 
 /**
@@ -385,10 +385,11 @@ const char *xine_get_homedir(void) {
   static char homedir[BUFSIZ] = {0,};
 
 #ifdef HAVE_GETPWUID_R
-  if(getpwuid_r(getuid(), &pwd, homedir, sizeof(homedir), &pw) != 0 || pw == NULL) {
+  if(getpwuid_r(getuid(), &pwd, homedir, sizeof(homedir), &pw) != 0 || pw == NULL)
 #else
-  if((pw = getpwuid(getuid())) == NULL) {
+  if((pw = getpwuid(getuid())) == NULL)
 #endif
+  {
     char *tmp = getenv("HOME");
     if(tmp) {
       strncpy(homedir, tmp, sizeof(homedir));
@@ -549,25 +550,22 @@ void xine_hexdump (const void *buf_gen, int length) {
 
 
 static const lang_locale_t *_get_first_lang_locale(const char *lcal) {
-  const lang_locale_t *llocale;
   size_t lang_len;
+  size_t i;
   char *mod;
 
   if(lcal && *lcal) {
-    llocale = &*lang_locales;
 
     if ((mod = strchr(lcal, '@')))
       lang_len = mod++ - lcal;
     else
       lang_len = strlen(lcal);
 
-    while(*(llocale->language)) {
-      if(!strncmp(lcal, llocale->language, lang_len)) {
-        if ((!mod && !llocale->modifier) || (mod && llocale->modifier && !strcmp(mod, llocale->modifier)))
-	  return llocale;
+    for (i = 0; i < sizeof(lang_locales)/sizeof(lang_locales[0]); i++) {
+      if(!strncmp(lcal, lang_locales[i].language, lang_len)) {
+        if ((!mod && !lang_locales[i].modifier[0]) || (mod && lang_locales[i].modifier[0] && !strcmp(mod, lang_locales[i].modifier)))
+          return &lang_locales[i];
       }
-
-      llocale++;
     }
   }
   return NULL;
@@ -620,7 +618,7 @@ char *xine_get_system_encoding(void) {
       else {
         const lang_locale_t *llocale = _get_first_lang_locale(lg);
 
-        if(llocale && llocale->encoding)
+        if(llocale)
           codeset = strdup(llocale->encoding);
       }
 
@@ -648,7 +646,7 @@ const char *xine_guess_spu_encoding(void) {
     if ((enc = strchr(lg, '.'))) *enc = '\0';
     llocale = _get_first_lang_locale(lg);
     free(lg);
-    if (llocale && llocale->spu_encoding) return llocale->spu_encoding;
+    if (llocale) return llocale->spu_encoding;
   }
 
   return "iso-8859-1";
@@ -672,50 +670,49 @@ void xine_xprintf(xine_t *xine, int verbose, const char *fmt, ...) {
 int xine_monotonic_clock(struct timeval *tv, struct timezone *tz)
 {
 #if _POSIX_TIMERS > 0 && defined(_POSIX_MONOTONIC_CLOCK) && defined(HAVE_POSIX_TIMERS)
-  static int initialized = 0;
-  static int use_clock_monotonic = 0;
+  static int xmc_mode = 0;
 
-  struct timespec tp;
+  do {
+    struct timespec ts;
 
-  if( !initialized ) {
-    struct timespec res;
-    int ret;
-
-    ret = clock_getres(CLOCK_MONOTONIC, &res);
-
-    if( ret != 0 ) {
-      lprintf("get resolution of monotonic clock failed\n");
-    } else {
-      /* require at least milisecond resolution */
-      if( res.tv_sec > 0 ||
-          res.tv_nsec > 1000000 ) {
-        lprintf("monotonic clock resolution (%d:%d) too bad\n",
-                 (int)res.tv_sec, (int)res.tv_nsec);
-      } else {
-        if( clock_gettime(CLOCK_MONOTONIC, &tp) != 0 ) {
-          lprintf("get monotonic clock failed\n");
-        } else {
-          lprintf("using monotonic clock\n");
-          use_clock_monotonic = 1;
-        }
-      }
+    if (xmc_mode > 1) {
+      if (clock_gettime (CLOCK_MONOTONIC, &ts))
+        break;
+      tv->tv_sec = ts.tv_sec;
+      tv->tv_usec = ts.tv_nsec / 1000;
+      return 0;
     }
-    initialized = 1;
-  }
 
-  if(use_clock_monotonic && !clock_gettime(CLOCK_MONOTONIC, &tp)) {
-    tv->tv_sec = tp.tv_sec;
-    tv->tv_usec = tp.tv_nsec / 1000;
+    if (xmc_mode == 1)
+      break;
+
+    xmc_mode = 1;
+
+    if (clock_getres (CLOCK_MONOTONIC, &ts)) {
+      lprintf ("get resolution of monotonic clock failed\n");
+      break;
+    }
+
+    /* require at least milisecond resolution */
+    if ((ts.tv_sec > 0) || (ts.tv_nsec > 1000000)) {
+      lprintf ("monotonic clock resolution (%d:%d) too bad\n", (int)ts.tv_sec, (int)ts.tv_nsec);
+      break;
+    }
+
+    if (clock_gettime (CLOCK_MONOTONIC, &ts)) {
+      lprintf ("get monotonic clock failed\n");
+      break;
+    }
+
+    lprintf ("using monotonic clock\n");
+    xmc_mode = 2;
+    tv->tv_sec = ts.tv_sec;
+    tv->tv_usec = ts.tv_nsec / 1000;
     return 0;
-  } else {
-    return gettimeofday(tv, tz);
-  }
-
-#else
-
-  return gettimeofday(tv, tz);
-
+  } while (0);
 #endif
+
+  return gettimeofday (tv, tz);
 }
 
 char *xine_strcat_realloc (char **dest, char *append)
@@ -791,3 +788,297 @@ int xine_socket_cloexec(int domain, int type, int protocol)
   return s;
 }
 
+/* get/resize/free aligned memory */
+
+#ifndef XINE_MEM_ALIGN
+#  define XINE_MEM_ALIGN 32
+#endif
+#define XINE_MEM_ADD (sizeof (size_t) + XINE_MEM_ALIGN)
+#define XINE_MEM_MASK ((unsigned long int)XINE_MEM_ALIGN - 1)
+
+void *xine_mallocz_aligned (size_t size) {
+  uint8_t *new;
+  size_t *sp;
+  new = calloc (1, size + XINE_MEM_ADD);
+  if (!new)
+    return NULL;
+  sp = (size_t *)new;
+  *sp = size;
+  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new[-1] = new - (uint8_t *)sp;
+  return new;
+}
+
+void *xine_malloc_aligned (size_t size) {
+  uint8_t *new;
+  size_t *sp;
+  new = malloc (size + XINE_MEM_ADD);
+  if (!new)
+    return NULL;
+  sp = (size_t *)new;
+  *sp = size;
+  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new[-1] = new - (uint8_t *)sp;
+  return new;
+}
+
+void xine_free_aligned (void *ptr) {
+  uint8_t *old = (uint8_t *)ptr;
+  if (!old)
+    return;
+  old -= old[-1];
+  free (old);
+}
+
+void *xine_realloc_aligned (void *ptr, size_t size) {
+  uint8_t *old = (uint8_t *)ptr, *new;
+  size_t *sp, s;
+  if (!size) {
+    if (old)
+      free (old - old[-1]);
+    return NULL;
+  }
+  new = malloc (size + XINE_MEM_ADD);
+  if (!new)
+    return NULL;
+  sp = (size_t *)new;
+  *sp = size;
+  new = (uint8_t *)(((unsigned long int)new + XINE_MEM_ADD) & ~XINE_MEM_MASK);
+  new[-1] = new - (uint8_t *)sp;
+  /* realloc () may break the alignment, requiring a slow memmove () afterwards */
+  if (old) {
+    sp = (size_t *)(old - old[-1]);
+    s = *sp;
+    if (size < s)
+      s = size;
+    xine_fast_memcpy (new, old, s);
+    free (sp);
+  }
+  return new;
+}
+
+/* Base64 transcoder, adapted from TJtools. */
+size_t xine_base64_encode (uint8_t *from, char *to, size_t size) {
+  static const uint8_t tab[64] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+  const uint8_t *p = from;
+  uint8_t *q = (uint8_t *)to;
+  int l = size;
+  from[size] = 0;
+  while (l > 0) {
+    uint32_t v = _X_BE_24 (p);
+    p += 3;
+    *q++ = tab[v >> 18];
+    *q++ = tab[(v >> 12) & 63];
+    *q++ = tab[(v >> 6) & 63];
+    *q++ = tab[v & 63];
+    l -= 3;
+  }
+  if (l < 0) {
+    q[-1] = '=';
+    if (l == -2) q[-2] = '=';
+  }
+  *q = 0;
+  return q - (uint8_t *)to;
+}
+
+size_t xine_base64_decode (const char *from, uint8_t *to) {
+  /* certain peopble use - _ instead of + /, lets support both ;-) */
+#define rr 128 /* repeat */
+#define ss 64  /* stop */
+  static const uint8_t tab_unbase64[256] = {
+    ss,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,
+    rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,rr,
+    rr,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,62,ss,62,ss,63,
+    52,53,54,55,56,57,58,59,60,61,ss,ss,ss,ss,ss,ss,
+    ss, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
+    15,16,17,18,19,20,21,22,23,24,25,ss,ss,ss,ss,63,
+    ss,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
+    41,42,43,44,45,46,47,48,49,50,51,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,
+    ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss,ss
+  };
+  const uint8_t *p = (const uint8_t *)from;
+  uint8_t *q = to;
+  while (1) {
+    uint32_t v, b;
+    do b = tab_unbase64[*p++]; while (b & rr);
+    if (b & ss) break;
+    v = b << 18;
+    do b = tab_unbase64[*p++]; while (b & rr);
+    if (b & ss) break;
+    v |= b << 12;
+    *q++ = v >> 16;
+    do b = tab_unbase64[*p++]; while (b & rr);
+    if (b & ss) break;
+    v |= b << 6;
+    *q++ = v >> 8;
+    do b = tab_unbase64[*p++]; while (b & rr);
+    if (b & ss) break;
+    v |= b;
+    *q++ = v;
+  }
+#undef rr
+#undef ss
+  return q - to;
+}
+
+/* XXX precalculate 5k instead? */
+static uint32_t tab_crc32_ieee[1280] = {0, 0,};
+static uint16_t tab_crc16_ansi[768] = {0, 0,};
+
+/* gcc -O3 recognizes this as bswap32 () */
+#define rev32(n) (((n) << 24) | (((n) << 8) & 0xff0000) | (((n) >> 8) & 0xff00) | ((n) >> 24))
+#define rev16(n) ((((n) << 8) | ((n) >> 8)) & 0xffff)
+
+uint32_t xine_crc32_ieee (uint32_t crc, const uint8_t *data, size_t len) {
+  uint32_t *t = tab_crc32_ieee;
+  if (!t[1]) {
+    uint32_t i;
+    for (i = 0; i < 256; i++) {
+      uint32_t j, u = i << 24;
+      for (j = 0; j < 8; j++)
+        u = (u << 1) ^ (((int32_t)u >> 31) & 0x4c11db7);
+      t[i] = rev32 (u);
+    }
+    for (i = 0; i < 256; i++) {
+      uint32_t v = t[i];
+#ifdef WORDS_BIGENDIAN
+      t[i + 256] = rev32 (v);
+      v = (v >> 8) ^ t[v & 255];
+      t[i + 512] = rev32 (v);
+      v = (v >> 8) ^ t[v & 255];
+      t[i + 768] = rev32 (v);
+      v = (v >> 8) ^ t[v & 255];
+      t[i + 1024] = rev32 (v);
+#else
+      t[i + 256] = v = (v >> 8) ^ t[v & 255];
+      t[i + 512] = v = (v >> 8) ^ t[v & 255];
+      t[i + 768] = (v >> 8) ^ t[v & 255];
+#endif
+    }
+  }
+  {
+    unsigned long int u;
+    const uint32_t *d32;
+    u = (~3ul - (unsigned long int)data) & 3;
+    if (u > len)
+      u = len;
+    len -= u;
+    while (u) {
+      crc = t[(uint8_t)crc ^ *data] ^ (crc >> 8);
+      data++;
+      u--;
+    }
+    d32 = (const uint32_t *)data;
+    u = len / 4;
+#ifdef WORDS_BIGENDIAN
+    crc = rev32 (crc);
+    while (u) {
+      crc ^= *d32++;
+      crc = t[(crc >> 24) + 1024]
+          ^ t[((crc >> 16) & 0xff) + 768]
+          ^ t[((crc >> 8) & 0xff) + 512]
+          ^ t[(crc & 0xff) + 256];
+      u--;
+    }
+    crc = rev32 (crc);
+#else
+    while (u) {
+      crc ^= *d32++;
+      crc = t[(crc & 0xff) + 768]
+          ^ t[((crc >> 8) & 0xff) + 512]
+          ^ t[((crc >> 16) & 0xff) + 256]
+          ^ t[crc >> 24];
+      u--;
+    }
+#endif
+    data = (const uint8_t *)d32;
+    u = len & 3;
+    while (u) {
+      crc = t[(uint8_t)crc ^ *data] ^ (crc >> 8);
+      data++;
+      u--;
+    }
+    return crc;
+  }
+}
+
+uint32_t xine_crc16_ansi (uint32_t crc, const uint8_t *data, size_t len) {
+  uint16_t *t = tab_crc16_ansi;
+  if (!t[1]) {
+    uint32_t i;
+    for (i = 0; i < 256; i++) {
+      uint32_t j, u = i << 24;
+      for (j = 0; j < 8; j++)
+        u = (u << 1) ^ (((int32_t)u >> 31) & 0x80050000);
+      t[i] = ((u >> 8) & 0xff00) | (u >> 24);
+    }
+    for (i = 0; i < 256; i++) {
+      uint16_t v = t[i];
+#ifdef WORDS_BIGENDIAN
+      t[i + 256] = rev16 (v);
+      v = (v >> 8) ^ t[v & 255];
+      t[i + 512] = rev16 (v);
+#else
+      t[i + 256] = (v >> 8) ^ t[v & 255];
+#endif
+    }
+  }
+  {
+    unsigned long int u;
+    const uint32_t *d32;
+    crc &= 0xffff;
+    u = (~3ul - (unsigned long int)data) & 3;
+    if (u > len)
+      u = len;
+    len -= u;
+    while (u) {
+      crc = t[(uint8_t)crc ^ *data] ^ (crc >> 8);
+      data++;
+      u--;
+    }
+    d32 = (const uint32_t *)data;
+    u = len / 4;
+#ifdef WORDS_BIGENDIAN
+    crc = rev16 (crc);
+    while (u) {
+      uint32_t v = *d32++;
+      crc ^= v >> 16;
+      crc = t[(crc >> 8) + 512]
+          ^ t[(crc & 0xff) + 256];
+      crc ^= v & 0xffff;
+      crc = t[(crc >> 8) + 512]
+          ^ t[(crc & 0xff) + 256];
+      u--;
+    }
+    crc = rev16 (crc);
+#else
+    while (u) {
+      uint32_t v = *d32++;
+      crc ^= v & 0xffff;
+      crc = t[(crc & 255) + 256]
+          ^ t[crc >> 8];
+      crc ^= v >> 16;
+      crc = t[(crc & 255) + 256]
+          ^ t[crc >> 8];
+      u--;
+    }
+#endif
+    data = (const uint8_t *)d32;
+    u = len & 3;
+    while (u) {
+      crc = t[(uint8_t)crc ^ *data] ^ (crc >> 8);
+      data++;
+      u--;
+    }
+    return crc;
+  }
+}
+
