@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2000-2011 the xine project
+* Copyright (C) 2000-2017 the xine project
 *
 * This file is part of xine, a free video player.
 *
@@ -55,6 +55,9 @@
 #include <dlfcn.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#ifdef HAVE_SYS_SYSMACROS_H
+#  include <sys/sysmacros.h>
+#endif
 #include <sys/utsname.h>
 #include <linux/major.h>
 #include <linux/hdreg.h>
@@ -261,6 +264,7 @@ static xine_health_check_t* _x_health_check_dma (xine_health_check_t* hc) {
   if (!is_scsi_dev) {
 
     if(ioctl (fd, HDIO_GET_DMA, &param)) {
+      close (fd);
       set_hc_result(hc, XINE_HEALTH_CHECK_FAIL,
 		    "FAILED -  HDIO_GET_DMA failed. Ensure the permissions for %s are 0664.\n",
 		    hc->dvd_dev);
@@ -268,6 +272,7 @@ static xine_health_check_t* _x_health_check_dma (xine_health_check_t* hc) {
     }
 
     if (param != 1) {
+      close (fd);
       set_hc_result(hc, XINE_HEALTH_CHECK_FAIL,
 		    "FAILED - DMA not turned on for %s.",
 		    hc->dvd_dev);
